' ########################################################################################
' Microsoft Windows
' File: AfxMenu.inc
' Contents: Menu wrapper functions
' Compiler: C/C++ Optimizing Compiler Version 19.00.23506 for x64
' Copyright (c) 2016 José Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
'Ported to bc9Basic -> c/c++ by James c. Fuller
'=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
$IFUSE USE_AfxCheckMenuItem
' Checks a menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'           identifier. Otherwise, it is a menu item position.
' Return Value: The return value specifies the previous state of the menu item (either
' MF_CHECKED or MF_UNCHECKED). If the menu item does not exist, the return value is -1.
' ========================================================================================
Function AfxCheckMenuItem (hMenu As HMENU, uItem AS DWORD, fByPosition AS LONG = FALSE) AS DWORD
	Dim AS DWORD dwFlags 
	If fByPosition Then 
		dwFlags = MF_BYPOSITION 
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	dwFlags = dwFlags BOR MF_CHECKED
	Function = CheckMenuItem(hMenu, uItem, dwFlags)
End Function
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUnCheckMenuItem
'------------------------------------------------------------------------------
' Unchecks a menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'           identifier. Otherwise, it is a menu item position.
' Return Value: The return value specifies the previous state of the menu item (either
' MF_CHECKED or MF_UNCHECKED). If the menu item does not exist, the return value is -1.
' ========================================================================================
Function AfxUnCheckMenuItem ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS DWORD
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	dwFlags = dwFlags BOR MF_UNCHECKED
	Function = CheckMenuItem(hMenu, uItem, dwFlags)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxToggleMenuItem
'------------------------------------------------------------------------------
' Toggles the checked state of a menu item.
' ========================================================================================
Function AfxToggleMenuItem ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS DWORD
	Dim As DWORD dwFlags
	If fByPosition Then 
		dwFlags = MF_BYPOSITION 
	Else dwFlags = MF_BYCOMMAND
	EndIf
	If GetMenuState(hMenu, uItem, dwFlags) BAND MF_CHECKED = MF_CHECKED Then
		dwFlags = dwFlags BOR MF_UNCHECKED
	Else
		dwFlags = dwFlags BOR MF_CHECKED
	End If
	Function = CheckMenuItem(hMenu, uItem, dwFlags)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemChecked
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is checked; FALSE otherwise.
' ========================================================================================
Function AfxIsMenuItemChecked ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOL
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	If GetMenuState(hMenu, uItem, dwFlags) BAND MF_CHECKED = MF_CHECKED Then
		Function = TRUE
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemEnabled
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is enabled; FALSE otherwise.
'------------------------------------------------------------------------------
Function AfxIsMenuItemEnabled ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	Dim As DWORD dwRes = GetMenuState(hMenu, uItem, dwFlags)
	If ((dwRes BAND MF_DISABLED) <> MF_DISABLED) AND ((dwRes BAND MF_GRAYED) <> MF_GRAYED) Then
		Function = TRUE
	EndIf
	Function = FALSE
End Function

'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemDisabled
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is disabled; FALSE otherwise.
'------------------------------------------------------------------------------
Function AfxIsMenuItemDisabled ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	Dim As DWORD dwRes = GetMenuState(hMenu, uItem, dwFlags)
	If ((dwRes BAND MF_DISABLED) = MF_DISABLED) Then
		Function =  TRUE
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemGrayed
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is grayed; FALSE otherwise.
' ========================================================================================
Function AfxIsMenuItemGrayed ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	Dim As DWORD dwRes = GetMenuState(hMenu, uItem, dwFlags)
	If ((dwRes BAND MF_GRAYED) = MF_GRAYED) Then
		Function = TRUE
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemHighlighted
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is highlighted; FALSE otherwise.
'------------------------------------------------------------------------------
Function AfxIsMenuItemHighlighted ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	Dim As DWORD dwRes = GetMenuState(hMenu, uItem, dwFlags)
	If ((dwRes BAND MF_HILITE) = MF_HILITE) Then
		Function = TRUE
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemSeparator
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is a separator; FALSE otherwise.
'------------------------------------------------------------------------------
Function AfxIsMenuItemSeparator ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	Dim As DWORD dwRes = GetMenuState(hMenu, uItem, dwFlags)
	If ((dwRes BAND MF_SEPARATOR) = MF_SEPARATOR) Then
		Function =  TRUE
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemPopup
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is a submenu; FALSE otherwise.
' ========================================================================================
Function AfxIsMenuItemPopup ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	Dim As DWORD dwRes = GetMenuState(hMenu, uItem, dwFlags)
	If ((dwRes BAND MF_POPUP) = MF_POPUP) Then
		Function = TRUE
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxIsMenuItemOwnerDraw
'------------------------------------------------------------------------------
' Returns TRUE if the specified menu item is ownerdraw; FALSE otherwise.
' ========================================================================================
Function AfxIsMenuItemOwnerDraw ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	Dim As DWORD dwFlags
	If fByPosition Then
		dwFlags = MF_BYPOSITION
	Else
		dwFlags = MF_BYCOMMAND
	EndIf
	Dim As DWORD dwRes = GetMenuState(hMenu, uItem, dwFlags)
	If ((dwRes BAND MF_OWNERDRAW) = MF_OWNERDRAW) Then
		Function = TRUE
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetMenuItemState
'------------------------------------------------------------------------------
' Retrieves the state of the specified menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'           identifier. Otherwise, it is a menu item position.
' Return Value: 0 on failure or one or more of the following values:
' - MFS_CHECKED   The item is checked
' - MFS_DEFAULT   The menu item is the default.
' - MFS_DISABLED  The item is disabled.
' - MFS_ENABLED   The item is enabled.
' - MFS_GRAYED    The item is grayed.
' - MFS_HILITE    The item is highlighted
' - MFS_UNCHECKED The item is unchecked.
' - MFS_UNHILITE  The item is not highlighed.
' Note: To get extended error information, use the GetLastError function.
' ========================================================================================
Function AfxGetMenuItemState ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS DWORD
	Dim AS MENUITEMINFOW mii 
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STATE
	If GetMenuItemInfoW(hMenu, uItem, fByPosition, &mii) = 0 Then
		Function =  0
	EndIf
	Function = mii.fState
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxSetMenuItemState
'------------------------------------------------------------------------------
' Sets the state of the specified menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fState = The menu item state. It can be one or more of these values:
' - MFS_CHECKED   Checks the menu item.
' - MFS_DEFAULT   Specifies that the menu item is the default.
' - MFS_DISABLED  Disables the menu item and grays it so that it cannot be selected.
' - MFS_ENABLED   Enables the menu item so that it can be selected. This is the default state.
' - MFS_GRAYED    Disables the menu item and grays it so that it cannot be selected.
' - MFS_HILITE    Highlights the menu item.
' - MFS_UNCHECKED Unchecks the menu item.
' - MFS_UNHILITE  Removes the highlight from the menu item. This is the default state.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'                 identifier. Otherwise, it is a menu item position.
' Return Value: TRUE or FALSE. To get extended error information, use the GetLastError function.
' Note: The application must call the DrawMenuBar function whenever a menu changes,
' whether or not the menu is in a displayed window.
' ========================================================================================
Function AfxSetMenuItemState ( hMenu AS HMENU,  uItem AS DWORD,  fState AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	DIM AS MENUITEMINFOW mii 
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STATE
	mii.fState = fState
	Function = SetMenuItemInfoW(hMenu, uItem, fByPosition, &mii)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxEnableMenuItem
'------------------------------------------------------------------------------
' Enables the specified menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'                 identifier. Otherwise, it is a menu item position.
' Return Value: TRUE or FALSE. To get extended error information, use the GetLastError function.
' Note: The application must call the DrawMenuBar function whenever a menu changes,
' whether or not the menu is in a displayed window.
' ========================================================================================
Function AfxEnableMenuItem ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	DIM AS MENUITEMINFOW mii
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STATE
	mii.fState = MFS_ENABLED
	Function = SetMenuItemInfoW(hMenu, uItem, fByPosition, &mii)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxDisableMenuItem
'------------------------------------------------------------------------------
' Disables the specified menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'                 identifier. Otherwise, it is a menu item position.
' Return Value: TRUE or FALSE. To get extended error information, use the GetLastError function.
' Note: The application must call the DrawMenuBar function whenever a menu changes,
' whether or not the menu is in a displayed window.
' ========================================================================================
Function AfxDisableMenuItem ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	DIM mii AS MENUITEMINFOW
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STATE
	mii.fState = MFS_DISABLED
	Function = SetMenuItemInfoW(hMenu, uItem, fByPosition, &mii)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGrayMenuItem
'------------------------------------------------------------------------------
' Grays the specified menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'                 identifier. Otherwise, it is a menu item position.
' Return Value: TRUE or FALSE. To get extended error information, use the GetLastError function.
' Note: The application must call the DrawMenuBar function whenever a menu changes,
' whether or not the menu is in a displayed window.
' ========================================================================================
Function AfxGrayMenuItem ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	DIM AS MENUITEMINFOW mii
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STATE
	mii.fState = MFS_GRAYED
	Function = SetMenuItemInfoW(hMenu, uItem, fByPosition, &mii)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxHiliteMenuItem
'------------------------------------------------------------------------------
' Highlights the specified menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'                 identifier. Otherwise, it is a menu item position.
' Return Value: TRUE or FALSE. To get extended error information, use the GetLastError function.
' Note: The application must call the DrawMenuBar function whenever a menu changes,
' whether or not the menu is in a displayed window.
' ========================================================================================
Function AfxHiliteMenuItem ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS BOOLEAN
	DIM AS MENUITEMINFOW mii
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STATE
	mii.fState = MFS_HILITE
	Function = SetMenuItemInfoW(hMenu, uItem, fByPosition, &mii)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxRemoveCloseMenu
'------------------------------------------------------------------------------
' Removes the system menu close option and disables the X button.
' Parameter: hwnd = Handle of the window that owns the menu.
' Return value: TRUE of FALSE.
' ========================================================================================
Function AfxRemoveCloseMenu ( hwnd AS HWND) AS BOOLEAN
	' // Get the system menu handle
	DIM AS HMENU hMenu = GetSystemMenu(hwnd, 0)
	If hMenu = NULL Then
		Function = FALSE
	EndIf
	' // Get the number of menu items
	DIM  AS LONG cbItems = GetMenuItemCount(hMenu)
	If cbItems = 0 Then
		Function = FALSE
	EndIf
	' // Remove the close menu item
	If RemoveMenu(hMenu, cbItems - 1, MF_REMOVE BOR MF_BYPOSITION) = 0 Then
		Function =  FALSE
	EndIf
	' // Remove the separator line
	If RemoveMenu(hMenu, cbItems - 2, MF_REMOVE BOR MF_BYPOSITION) = 0 Then
		Function =  FALSE
	EndIf
	' // Redraw the menu (this refreshes the caption bar, dimming the X button)
	DrawMenuBar(hwnd)
	Function = TRUE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxRightJustifyMenuItem
'------------------------------------------------------------------------------
' Right justifies a top level menu item. This is usually used to have the Help menu item
' right-justified on the menu bar.
' - hwnd  = [in] A handle to the menu that contains the menu item.
' - uItem = [in] The zero-based position of the menu item to change.
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxRightJustifyMenuItem ( hMenu AS HMENU,  uItem AS DWORD) AS BOOLEAN
	Dim As MENUITEMINFOW mii  
	Dim As BOOL bRetVal
	Dim buffer$ * MAX_PATH + 1
	mii.cbSize = SIZEOF(MENUITEMINFOW)
	mii.dwTypeData = buffer
	mii.cch = MAX_PATH
	mii.fType = MF_STRING
	mii.fState = MFS_DEFAULT
	mii.fMask = MIIM_ID OR MIIM_DATA OR MIIM_TYPE OR MIIM_SUBMENU
	If GetMenuItemInfoW(hMenu, uItem, TRUE, &mii) Then
		mii.fType = mii.fType BOR MF_HELP
		bRetVal = SetMenuItemInfoW(hMenu, uItem, TRUE, &mii)
	End If
	
	Function = bRetVal
End Function

'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxSetMenuItemBold
'------------------------------------------------------------------------------
' Changes the text of a menu item to bold.
' - hwnd  = [in] A handle to the menu that contains the menu item.
' - uItem = [in] The zero-based position of the menu item to change.
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxSetMenuItemBold ( hMenu AS HMENU,  uItem AS DWORD) AS BOOLEAN
	Dim As MENUITEMINFOW mii
	Dim buffer$ * MAX_PATH + 1
	Dim As BOOL bRetVal
	mii.cbSize = SIZEOF(MENUITEMINFOW)
	mii.dwTypeData = buffer
	mii.cch = MAX_PATH
	mii.fType = MF_STRING
	mii.fMask = MIIM_ID OR MIIM_DATA OR MIIM_TYPE OR MIIM_SUBMENU OR MIIM_STATE
	If GetMenuItemInfoW(hMenu, uItem, TRUE, &mii) Then
		mii.fState = mii.fState BOR 0x1000
		bRetVal = SetMenuItemInfoW(hMenu, uItem, TRUE, &mii)
	END IF
	Function = bRetVal
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxSetMenuItemText
'------------------------------------------------------------------------------
' Sets the text of the specified menu item.
' - hMenu = A handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - wszText = Text to set.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'           identifier. Otherwise, it is a menu item position.
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxSetMenuItemText ( hMenu AS HMENU,  uItem AS DWORD, szText As char Ptr,  fByPosition AS LONG = FALSE) AS BOOLEAN
	DIM AS MENUITEMINFOW mii 
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STRING
	mii.dwTypeData = szText
	Function =  SetMenuItemInfoW(hMenu, uItem, fByPosition, &mii)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetMenuItemTextLen
'------------------------------------------------------------------------------
' Returns the lengnth of the specified menu item.
' ========================================================================================
Function AfxGetMenuItemTextLen ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG = FALSE) AS LONG
	' // Fills the MENUITEMINFOW structure
	DIM AS MENUITEMINFOW mii
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STRING
	mii.dwTypeData = NULL
	' // Get the needed size of the buffer
	If GetMenuItemInfoW(hMenu, uItem, fByPosition, &mii) = 0 Then
		Function =  0
	EndIf
	Function = mii.cch
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetMenuItemText
'------------------------------------------------------------------------------
' Retrieves the text of the specified menu item.
' - hMenu = Handle to the menu that contains the menu item.
' - uItem = The identifier or position of the menu item to get information about.
'           The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of uItem. If this parameter is FALSE, uItem is a menu item
'           identifier. Otherwise, it is a menu item position.
' - pwszText: A pointer to a buffer to receive the retrieved text.
' - cchTextMax : Maximum number of characters to return. Both this value and the size of the
'      buffer pointed by pwszText must be one character bigger that the wanted length of the text
'      to return to make room for the null character terminator.
' Return value: TRUE or FALSE.
' Usage example:
' DIM wsz AS WSTRING * 260
' AfxGetMenuItemText(hSubMenu, 1, TRUE, @wsz, 260)
' AfxMsg wsz
' ========================================================================================
Function AfxGetMenuItemText ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG,  pwszText AS char PTR,  cchTextMax AS LONG) AS BOOLEAN
	If pwszText = NULL Then
		Function = FALSE
	EndIf
	' // Fills the MENUITEMINFOW structure
	DIM AS MENUITEMINFOW mii 
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STRING
	mii.dwTypeData = pwszText
	mii.cch = cchTextMax
	Function = GetMenuItemInfoW(hMenu, uItem, fByPosition, &mii)
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetMenuItemText
'------------------------------------------------------------------------------
Function AfxGetMenuItemText ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS LONG) As string
	DIM AS MENUITEMINFOW mii 
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_STRING
	mii.dwTypeData = NULL
	' // Get the needed size of the buffer
	If GetMenuItemInfoW(hMenu, uItem, fByPosition, &mii) = 0 Then
		Function = ""
	EndIf
	' // Make room for the trailing null
	mii.cch += 1
	' // Allocate the buffer
	DIM buffer$ * mii.cch
	' // Get the menu string
	mii.dwTypeData = buffer
	If GetMenuItemInfoW(hMenu, uItem, fByPosition, &mii) Then
	  	Function =  buffer
	Else
		Function = ""	
	End If
	
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetMenuFont
'------------------------------------------------------------------------------
' Retrieves information about the font used in menu bars.
' Return value: TRUE on succes or FALSE on failure.
' If the function fails, the return value is zero.
' To get extended error information, call GetLastError.
' ========================================================================================
Function AfxGetMenuFont ( plfw AS LOGFONTW PTR) AS BOOLEAN
	DIM AS NONCLIENTMETRICSW ncm 
	If plfw = NULL Then
		Function = FALSE
	EndIf
	If AfxWindowsVersion() >= 6 Then
		ncm.cbSize = 504 
	Else
		ncm.cbSize = 500
	EndIf
	If SystemParametersInfoW(SPI_GETNONCLIENTMETRICS, SIZEOF(ncm), &ncm, 0) = 0 Then
		Function = FALSE
	EndIf
	*plfw = ncm.lfMenuFont
	Function = TRUE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetMenuFontPointSize
'------------------------------------------------------------------------------
' Retrieves the point size of the font used in menu bars.
' If the function fails, the return value is 0.
' ========================================================================================
Function AfxGetMenuFontPointSize () AS LONG
	DIM AS NONCLIENTMETRICSW ncm 
	If AfxWindowsVersion() >= 6 Then
		ncm.cbSize = 504
	Else
		ncm.cbSize = 500
	EndIf
	If SystemParametersInfoW(SPI_GETNONCLIENTMETRICS, SIZEOF(ncm), &ncm, 0) = 0 Then
		Function = 0
	EndIf
	DIM AS HDC hDC  = CreateDCW("DISPLAY", NULL, NULL, NULL)
	If hDC = NULL Then
		Function = 0
	EndIf
	DIM AS LONG cyPixelsPerInch  = GetDeviceCaps(hDC, LOGPIXELSY)
	DeleteDC(hDC)
	DIM AS LONG nPointSize  = MulDiv(ncm.lfMenuFont.lfHeight, 72, cyPixelsPerInch)
	If nPointSize < 0 Then
		nPointSize = -nPointSize
	EndIf
	Function =  nPointSize
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetMenuRect
'------------------------------------------------------------------------------
' Calculates the size of a menu bar or a drop-down menu.
' - hwnd = Handle of the window that owns the menu.
' - hmenu = Handle of the menu.
' - prcmenu = Pointer to a variable of type RECT where to return the retrieved values.
' Return Value:
' If the function succeeds, the return value is 0.
' If the function fails, the return value is a system error code.
' ========================================================================================
Function AfxGetMenuRect ( hwnd AS HWND,  hmenu AS HMENU,  prcmenu AS RECT PTR) AS LONG
	DIM AS LONG i, nRes 
	Dim As RECT rc 
	For i = 1 To GetMenuItemCount(hmenu)
		nRes = GetMenuItemRect(hwnd, hmenu, i, &rc)
		If nRes = -1 Then
			nRes = GetLastError()
			Exit For
		EndIf
		UnionRect(prcmenu, prcmenu, &rc)
	NEXT
	Function = nRes
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
' ########################################################################################
'                              *** VISUAL STYLE MENUS ***
' ########################################################################################

' Windows Vista and posterior Windows versions provide menus that are part of the visual
' schema. These menus are rendered using visual styles, which can be added to existing
' applications. Adding code for new features to existing code must be done carefully to
' avoid breaking existing application behavior. Certain situations can cause visual styling
' to be disabled in an application. These situations include:
'    - Customizing menus using owner-draw menu items (MFT_OWNERDRAW)
'    - Using menu breaks (MFT_MENUBREAK or MFT_MENUBARBREAK)
'    - Using HBMMENU_CALLBACK to defer bitmap rendering
'    - Using a destroyed menu handle
' These situations prevent visual style menus from being rendered. Owner-draw menus can be
' used in Windows Vista and posterior Windows versions, but the menus will not be visually
' styled.
' Windows Vista and posterior Windows versions provide alpha-blended bitmaps, which enables
' menu items to be shown without using owner-draw menu items.
' Requirements:
'    - The bitmap is a 32bpp DIB section.
'    - The DIB section has BI_RGB compression.
'    - The bitmap contains pre-multiplied alpha pixels.
'    - The bitmap is stored in hbmpChecked, hbmpUnchecked, or hbmpItem fields.
' Note: MFT_BITMAP items do not support PARGB32 bitmaps.
' The following functions use the the Graphics Device Interface (GDI) to convert icons to
' bitmaps. Another solution is to use the Windows Imaging Component (WIC).
' Usage example:
'    DIM hSubMenu AS HMENU = GetSubMenu(hMenu, 1)
'    DIM hIcon AS HICON
'    hIcon = LoadImageW(NULL, "MyIcon.ico", IMAGE_ICON, 32, 32, LR_LOADFROMFILE)
'    IF hIcon THEN AfxAddIconToMenuItem(hSubMenu, 0, TRUE, hIcon)
' PNG icons can be used by converting them to an icon with AfxGdipImageFromFileEx:
'    hIcon = AfxGdipImageFromFileEx("MyIcon.png")
'    IF hIcon THEN AfxAddIconToMenuItem(hSubMenu, 0, TRUE, hIcon)
'==============================================================================
$IFUSE USE_AfxInitBitmapInfo
'------------------------------------------------------------------------------
' Initializes the BITMAPINFO structure
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxInitBitmapInfo ( pbmi AS BITMAPINFO PTR,  cbInfo AS ULONG,  cx AS LONG,  cy AS LONG,  bpp AS WORD) AS BOOLEAN
	If pbmi = NULL Then
		Function = FALSE
	EndIf
	!//<---UNICODE AWARE
	memset(pbmi, 0, cbInfo)
	!//>---UNICODE AWARE
	pbmi->bmiHeader.biSize = SIZEOF(BITMAPINFOHEADER)
	pbmi->bmiHeader.biPlanes = 1
	pbmi->bmiHeader.biCompression = BI_RGB
	pbmi->bmiHeader.biWidth = cx
	pbmi->bmiHeader.biHeight = cy
	pbmi->bmiHeader.biBitCount = bpp
	Function = TRUE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxCreate32BitHBITMAP
'------------------------------------------------------------------------------
' Creates a 32bpp and a BI_RGB DIB section.
' Paramaters:
' - hDC     = Handle to the device context.
' - psize   = Size of the icon
' - ppvBits = A pointer to a variable that receives a pointer to the location of the
'             DIB bit values. Can be NULL.
' - phbmp   = A pointer to a variable that receives the handle to the newly created DIB.
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxCreate32BitHBITMAP( hDC AS HDC,  psize AS SIZE PTR,  ppvBits AS void PTR PTR,  phBmp AS HBITMAP PTR) AS BOOLEAN
	If psize = NULL OR phBmp = NULL Then
		Function =  FALSE
	EndIf
	*phBmp = NULL
	DIM AS BITMAPINFO bmi 
	AfxInitBitmapInfo(&bmi, SIZEOF(bmi), psize->cx, psize->cy, 32)
	DIM AS HDC hdcUsed  =  iif((hDC <> NULL), hDC, GetDC(NULL))
	If hdcUsed = NULL Then
		Function = FALSE
	EndIf
	*phBmp = CreateDIBSection(hdcUsed, &bmi, DIB_RGB_COLORS, ppvBits, NULL, 0)
	ReleaseDC(NULL, hdcUsed)
	Function = TRUE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxAddBitmapToMenuItem
'------------------------------------------------------------------------------
' Adds a bitmap to the menu item.
' Parameters:
' - hMenu       = A handle to the menu that contains the menu item.
' - nMenuItem   = The identifier or position of the menu item to change.
'                 The meaning of this parameter depends on the value of fByPosition.
' - fByPosition = The meaning of nMenuItem. If this parameter is FALSE, nMenuItem is a
'                 menu item identifier. Otherwise, it is a menu item position.
' - hbmp        = Bitmap handle.
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxAddBitmapToMenuItem ( hMenu AS HMENU,  nMenuItem AS LONG,  fByPosition AS BOOLEAN,  hbmp AS HBITMAP) AS BOOLEAN
	DIM AS MENUITEMINFOW mii 
	mii.cbSize = SIZEOF(mii)
	mii.fMask = MIIM_BITMAP
	mii.hbmpItem = hbmp
	If  SetMenuItemInfoW(hMenu, nMenuItem, fByPosition, &mii) = 0 Then
		Function = FALSE
	EndIf
	Function =  TRUE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxConvertToPARGB32
'------------------------------------------------------------------------------
' Convert a bitmap to a PARGB32 bitmap.
' Parameters:
' - hDC       = Handle of the device context.
' - pargb     = Pointer to the address of the buffer bitmap pixels.
' - hbmp      = Bitmap handle.
' - sizeImage = Size of the bitmap.
' - cxRow     = The width, in pixels, of the buffer bitmap.
'               This value is not necessarily equal to the buffer width. It may be larger.
' Return value: TRUE or FALSE.
' Remarks: MFT_BITMAP items do not support PARGB32 bitmaps.
' ========================================================================================
Function AfxConvertToPARGB32 ( hDC AS HDC,  pargb AS DWORD PTR,  hbmp AS HBITMAP,  sizeImage AS SIZE,  cxRow AS LONG) AS BOOLEAN
	If hDC = NULL OR pargb = NULL OR hbmp = NULL Then
		Function = FALSE
	EndIf
	DIM AS BITMAPINFO bmi 
	AfxInitBitmapInfo(&bmi, SIZEOF(bmi), sizeImage.cx, sizeImage.cy, 32)
	DIM AS HANDLE hHeap = GetProcessHeap()
	DIM AS void PTR pvBits  = HeapAlloc(hHeap, 0, bmi.bmiHeader.biWidth * 4 * bmi.bmiHeader.biHeight)
	If pvBits Then
		If GetDIBits(hDC, hbmp, 0, bmi.bmiHeader.biHeight, pvBits, &bmi, DIB_RGB_COLORS) = bmi.bmiHeader.biHeight Then
			DIM AS ULONG cxDelta  = cxRow - bmi.bmiHeader.biWidth
			DIM AS DWORD PTR pargbMask  = (DWORD PTR) pvBits
			DIM AS LONG x, y
			For y = bmi.bmiHeader.biHeight To 1 STEP -1
				For x = bmi.bmiHeader.biWidth TO 1 STEP -1
					If *pargbMask Then
						' // transparent pixel
						*pargb = 0
						pargbMask +=1
						pargb += 1
					Else
						' // opaque pixel
						*pargb OR= 0xFF000000
						pargb +=1
					End if
					pargb += cxDelta
				Next x
			Next y
		End If
	End If
	HeapFree(hHeap, 0, pvBits)
	Function = TRUE
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxHasAlpha
'------------------------------------------------------------------------------
' Checks if the bitmap has the alpha channel set.
' Parameters:
' - pargb     = Pointer to the address of the buffer bitmap pixels.
' - sizeImage = Size of the bitmap.
' - cxRow     = The width, in pixels, of the buffer bitmap.
'               This value is not necessarily equal to the buffer width. It may be larger.
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxHasAlpha ( pargb AS DWORD PTR,  sizeImage AS SIZE,  cxRow AS LONG) AS BOOLEAN
	If pargb = NULL Then
		Function = FALSE
	EndIf
	DIM AS ULONG cxDelta  = cxRow - sizeImage.cx
	DIM AS LONG x , y 
	For y = sizeImage.cy TO 1 STEP -1
		For x = sizeImage.cx TO 1 STEP -1
			If (*pargb AND 0xFF000000) Then
				Function = TRUE
			EndIf
			pargb += 1
		Next x
		pargb += cxDelta
	Next y
	Function = FALSE

End Function

'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxConvertBufferToPARGB32
'------------------------------------------------------------------------------
' Converts a buffered bitmap to an PARGB32 bitmap.
' Parameters:
' - hPaintBuffer = The handle of the buffered paint context, obtained through BeginBufferedPaint.
' - hDC          = The Handle of the device context.
' - hIcon        = The icon handle.
' - sizeIcon     = The size of the icon.
' Return value: TRUE or FALSE.
' ========================================================================================
Function AfxConvertBufferToPARGB32 ( hPaintBuffer AS HPAINTBUFFER,  hDC AS HDC,  hIcon AS HICON,  sizeIcon AS SIZE) AS BOOLEAN
	If hPaintBuffer = NULL OR hDC = NULL OR hIcon = NULL Then
		Function = FALSE
	EndIf
	Dim As BOOL bRetVal
	DIM AS RGBQUAD PTR prgbQuad
	Dim As int cxRow
	Dim As HRESULT hr
	'If GetBufferedPaintBits(hPaintBuffer, &prgbQuad, &cxRow) = 0 Then
	hr = GetBufferedPaintBits(hPaintBuffer, &prgbQuad, &cxRow)
	If SUCCEEDED(hr) Then	
		DIM AS DWORD PTR pargb  = (DWORD PTR) prgbQuad
		If BNOT AfxHasAlpha(pargb, sizeIcon, cxRow) Then
			DIM AS ICONINFO info 
			IF GetIconInfo(hIcon, &info) Then
				IF info.hbmMask Then
					hr  =  AfxConvertToPARGB32(hDC, pargb, info.hbmMask, sizeIcon, cxRow)
					DeleteObject(info.hbmColor)
					DeleteObject(info.hbmMask)
					Function = bRetVal
				END IF
			END IF
		END IF
	End If
	If SUCCEEDED(hr) Then
		Function = TRUE
	Else
		Function = FALSE	
	EndIf
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxAddIconToMenuItem
'------------------------------------------------------------------------------
' Converts a hIcon to a bitmap and adds it to the specified hbmpItem field of HMENU item.
' The caller is responsible for destroying the bitmap generated. The icon will be destroyed
' if fAutoDestroy is set to true. The hbmpItem field of the menu item can be used to keep
' track of the bitmap by passing NULL to phbmp.
' Parameters:
' - hMenu        = Menu handle that contains the item to which an icon will be added.
' - uItem        = The identifier or position of the menu item to change.
'                  The meaning of this parameter depends on the value of fByPosition.
' - fByPosition  = The meaning of nMenuItem. If this parameter is FALSE, nMenuItem is a
'                  menu item identifier. Otherwise, it is a menu item position.
' - hIcon        = Handle of the icon to add to the menu.
' - fAutoDestroy = TRUE (the default) or FALSE.
'                  If TRUE, AddIconToMenuItem destroys the icon before returning.
' - phbmp        = Location where the bitmap representation of the icon is stored. Can be NULL.
' ========================================================================================
Function AfxAddIconToMenuItem ( hMenu AS HMENU,  uItem AS DWORD,  fByPosition AS BOOLEAN,  hIcon AS HICON,  fAutoDestroy AS BOOLEAN = TRUE,  phbmp AS HBITMAP PTR = NULL) AS BOOLEAN
	If hMenu = NULL OR hIcon = NULL Then
		Function = FALSE
	EndIf
	DIM AS HBITMAP hbmp
	Dim As SIZE sizIcon 
	Dim As RECT rcIcon
	sizIcon.cx = GetSystemMetrics(SM_CXSMICON)
	sizIcon.cy = GetSystemMetrics(SM_CYSMICON)
	SetRect(&rcIcon, 0, 0, sizIcon.cx, sizIcon.cy)
	Dim  AS HDC hdcDest = CreateCompatibleDC(NULL)
	If hdcDest = NULL Then
		Function = FALSE
	EndIf
	DIM  As LONG hr = (AfxCreate32BitHBITMAP(hdcDest, &sizIcon, NULL, &hbmp))
	If hr Then
		DIM  AS HBITMAP hbmpOld = (HBITMAP) SelectObject(hdcDest, hbmp)
		If hbmpOld Then
			DIM AS BLENDFUNCTION bfAlpha = {AC_SRC_OVER, 0, 255, AC_SRC_ALPHA}
			DIM AS BP_PAINTPARAMS paintParams 
			paintParams.cbSize = SIZEOF(paintParams)
			paintParams.dwFlags = BPPF_ERASE
			paintParams.pBlendFunction = &bfAlpha
			DIM AS HDC hdcBuffer 
			DIM AS HPAINTBUFFER hPaintBuffer = BeginBufferedPaint(hdcDest, &rcIcon, BPBF_DIB, &paintParams, &hdcBuffer)
			IF hPaintBuffer Then
				If DrawIconEx(hdcBuffer, 0, 0, hIcon, sizIcon.cx, sizIcon.cy, 0, NULL, DI_NORMAL) Then
					' // If icon did not have an alpha channel, we need to convert buffer to PARGB32.
					hr = AfxConvertBufferToPARGB32(hPaintBuffer, hdcDest, hIcon, sizIcon)
				End if
				' // This will write the buffer contents to the destination bitmap.
				EndBufferedPaint(hPaintBuffer, TRUE)
			End If
			SelectObject(hdcDest, hbmpOld)
		End if
	End if
	DeleteDC(hdcDest)
	If hr Then
		hr = AfxAddBitmapToMenuItem(hMenu, uItem, fByPosition, hbmp)
	EndIf
	If hr = FALSE Then
		DeleteObject(hbmp)
		hbmp = NULL
	EndIf
	If fAutoDestroy Then
		DestroyIcon(hIcon)
	EndIf
	If phbmp Then
		*phbmp = hbmp
	EndIf
	Function =  hr
End Function
'------------------------------------------------------------------------------
$ENDUSE

