' ########################################################################################
' Microsoft Windows
' File: AfxNls.inc
' Contents: National language support functions
' Compiler: Ported to bc9Basic jcfuller 09/03/2016
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
'==============================================================================
$IFUSE USE_AfxGetLocaleAbbrevDayName
'------------------------------------------------------------------------------
' ========================================================================================
' Returns the abbreviated native day name for the specified day (1 = Monday, etc.).
' ========================================================================================
FUNCTION AfxGetLocaleAbbrevDayName (nDay AS LONG,nLocale AS LONG = LOCALE_USER_DEFAULT) As string
	DIM wszBuf$
	IF nDay < 1 OR nDay > 7 THEN
		Function = ""
	EndIf
	GetLocaleInfo(nLocale, LOCALE_SABBREVDAYNAME1 + nDay - 1, wszBuf$, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleAbbrevMonthName
'------------------------------------------------------------------------------
' Returns the abbreviated native month name for the specified month.
' ========================================================================================
FUNCTION AfxGetLocaleAbbrevMonthName (BYVAL nMonth AS LONG, BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) As string
	DIM wszBuf$
	If nMonth < 1 Or nMonth > 12 Then
		Function = ""
	EndIf
	GetLocaleInfo(nLocale, LOCALE_SABBREVMONTHNAME1 + nMonth - 1, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleCurrencySymbol
'------------------------------------------------------------------------------
' String used as the local monetary symbol. The maximum number of characters allowed for
' this string is 13, including a terminating null character. For example, in the United
' States, this symbol is "$".
' ========================================================================================
FUNCTION AfxGetLocaleCurrencySymbol (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) As string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SCURRENCY, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleDayName
'------------------------------------------------------------------------------
' Returns the native day name for the specified day (1 = Monday, etc.).
' ========================================================================================
FUNCTION AfxGetLocaleDayName (BYVAL nDay AS LONG, BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) As String
	DIM wszBuf$
	IF nDay < 1 OR nDay > 7 THEN 
		Function = ""
	EndIf
	GetLocaleInfo(nLocale, LOCALE_SDAYNAME1 + nDay - 1, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleDecimalSeparator
'------------------------------------------------------------------------------
' Character(s) used for the decimal separator, for example, "." in "3.14" or "," in "3,14".
' The maximum number of characters allowed for this string is four, including a terminating
' null character.
' ========================================================================================
FUNCTION AfxGetLocaleDecimalSeparator (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SDECIMAL, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleDigitsGrouping
'------------------------------------------------------------------------------
' Sizes for each group of digits to the left of the decimal. The maximum number of characters
' allowed for this string is ten, including a terminating null character. An explicit size
' is needed for each group, and sizes are separated by semicolons. If the last value is 0,
' the preceding value is repeated. For example, to group thousands, specify 3;0. Indic
' locales group the first thousand and then group by hundreds. For example, 12,34,56,789
' is represented by 3;2;0.
' ========================================================================================
FUNCTION AfxGetLocaleDigitsGrouping (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS String
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SGROUPING, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleEnglishCountryName
'------------------------------------------------------------------------------
' Full English name of the country/region.
' ========================================================================================
FUNCTION AfxGetLocaleEnglishCountryName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SENGCOUNTRY, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleEnglishCurrencyName
'------------------------------------------------------------------------------
' The full English name of the currency associated with the locale.
' ========================================================================================
FUNCTION AfxGetLocaleEnglishCurrencyName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SENGCURRNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleEnglishDisplayName
'------------------------------------------------------------------------------
' Windows 7 and later: Display name of the locale in English. Usually the display name
' consists of the language and the country/region, for example, German (Germany) for
' Deutsch (Deutschland).
' ========================================================================================
FUNCTION AfxGetLocaleEnglishDisplayName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SENGLISHDISPLAYNAME, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleEnglishLanguageName
'------------------------------------------------------------------------------
' Full English name of the language from ISO Standard 639.
' ========================================================================================
FUNCTION AfxGetLocaleEnglishLanguageName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SENGLANGUAGE, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleInternationalMonetarySymbol
'------------------------------------------------------------------------------
' Three characters of the international monetary symbol specified in ISO 4217, followed by
' the character separating this string from the amount. The maximum number of characters
' allowed for this string is nine, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleInternationalMonetarySymbol (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SINTLSYMBOL, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleISO639LanguageName
'------------------------------------------------------------------------------
' The abbreviated name of the language based entirely on the ISO Standard 639 values, in
' lowercase form, for example, "en" for English. The maximum number of characters allowed
' for this string is nine, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleISO639LanguageName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SISO639LANGNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleISO639LanguageName2
'------------------------------------------------------------------------------
' Windows Vista and later: Three-letter ISO language name, in lowercase form (ISO 639-2
' three-letter code for the language), for example, "eng" for English. The maximum number
' of characters allowed for this string is nine, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleISO639LanguageName2 (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SISO639LANGNAME2, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleISO3166CountryName
'------------------------------------------------------------------------------
' Country/region name, based on ISO Standard 3166, for example, "US" for the United States.
' The maximum number of characters allowed for this string is nine, including a terminating
' null character.
' ========================================================================================
FUNCTION AfxGetLocaleISO3166CountryName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SISO3166CTRYNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleISO3166CountryName2
'------------------------------------------------------------------------------
' Windows Vista and later: Three-letter ISO region name (ISO 3166 three-letter code for the
' country/region), for example, "USA" for the United States. The maximum number of characters
' allowed for this string is nine, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleISO3166CountryName2 (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SISO3166CTRYNAME2, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleListItemsSeparator
'------------------------------------------------------------------------------
' Character(s) used to separate list items, for example, a comma is used in many locales.
' The maximum number of characters allowed for this string is four, including a
' terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleListItemsSeparator (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SLIST, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleLongDateFormatString
'------------------------------------------------------------------------------
' Long date formatting string for the locale. The maximum number of characters allowed for
' this string is 80, including a terminating null character. The string can consist of a
' combination of day, month, year, and era format pictures and any string of characters
' enclosed in single quotes. Characters in single quotes remain as specified. For example,
' the Spanish (Spain) long date is "dddd, dd' de 'MMMM' de 'yyyy". Locales can define
' multiple long date formats.
' ========================================================================================
FUNCTION AfxGetLocaleLongDateFormatString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SLONGDATE, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleMonetaryDecimalSeparator
'------------------------------------------------------------------------------
' Character(s) used as the monetary decimal separator. The maximum number of characters
' allowed for this string is four, including a terminating null character. For example,
' if a monetary amount is displayed as "$3.40", just as "three dollars and forty cents"
' is displayed in the United States, then the monetary decimal separator is ".".
' ========================================================================================
FUNCTION AfxGetLocaleMonetaryDecimalSeparator (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SMONDECIMALSEP, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleMonetaryGrouping
'------------------------------------------------------------------------------
' Sizes for each group of monetary digits to the left of the decimal. The maximum number
' of characters allowed for this string is ten, including a terminating null character.
' An explicit size is needed for each group, and sizes are separated by semicolons. If the
' last value is 0, the preceding value is repeated. For example, to group thousands,
' specify 3;0. Indic languages group the first thousand and then group by hundreds. For
' example 12,34,56,789 is represented by 3;2;0.
' ========================================================================================
FUNCTION AfxGetLocaleMonetaryGrouping (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SMONGROUPING, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleMonetaryThousandsSeparator
'------------------------------------------------------------------------------
' Character(s) used as the monetary separator between groups of digits to the left of the
' decimal. The maximum number of characters allowed for this string is four, including a
' terminating null character. Typically, the groups represent thousands. However, depending
' on the value specified for LOCALE_SMONGROUPING, they can represent something else.
' ========================================================================================
FUNCTION AfxGetLocaleMonetaryThousandsSeparator (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SMONTHOUSANDSEP, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleMonthDayFormatString
'------------------------------------------------------------------------------
' Windows 7 and later: Format string for displaying only the month and the day. The
' formatting is similar to that defined for LOCALE_SLONGDATE. For example, if the month/day
' pattern is the full month name followed by the day number with leading zeros, as in
' "September 03", the format string is "MMMM dd". The string can consist of a combination
' of day and month format pictures and any string of characters enclosed in single quotes.
' Characters in single quotes remain as specified, for example, 'de' for Spanish (Spain).
' A locale can specify only one month/day format.
' ========================================================================================
FUNCTION AfxGetLocaleMonthDayFormatString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SMONTHDAY, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleMonthName
'------------------------------------------------------------------------------
' Returns the native month name for the specified month.
' ========================================================================================
FUNCTION AfxGetLocaleMonthName (BYVAL nMonth AS LONG, BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	IF nMonth < 1 OR nMonth > 12 THEN
		Function = ""
	EndIf
	GetLocaleInfo(nLocale, LOCALE_SMONTHNAME1 + nMonth - 1, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleName
'------------------------------------------------------------------------------
' Windows Vista and later: Locale name, a multi-part tag to uniquely identify the locale.
' The maximum number of characters allowed for this string is 85, including a terminating
' null character. The tag is based on the language tagging conventions of RFC 4646.
' ========================================================================================
FUNCTION AfxGetLocaleName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SNAME, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNanString
'------------------------------------------------------------------------------
' Windows Vista and later: String value for "Not a number", for example, "Nan" for the
' English (United States) locale. There is no limit on the number of characters allowed
' for this string.
' ========================================================================================
FUNCTION AfxGetLocaleNanString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	Dim As LONG nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SNAN, NULL, 0)
	Dim pBuf$ *nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SNAN, pBuf, nLen)
	Function = pBuf$
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNativeCountryName
'------------------------------------------------------------------------------
' Native name of the country/region, for example, Espaa for Spain. The maximum number of
' characters allowed for this string is 80, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleNativeCountryName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SNATIVECTRYNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNativeCurrencyName
'------------------------------------------------------------------------------
' The native name of the currency associated with the locale, in the native language of
' the locale. There is no limit on the number of characters allowed for this string.
' ========================================================================================
FUNCTION AfxGetLocaleNativeCurrencyName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SNATIVECURRNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNativeDigits
'------------------------------------------------------------------------------
' Native equivalents of ASCII 0 through 9. The maximum number of characters allowed for
' this string is eleven, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleNativeDigits (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SNATIVEDIGITS, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNativeDisplayName
'------------------------------------------------------------------------------
' Windows 7 and later: Display name of the locale in its native language, for example,
' Deutsch (Deutschland) for the locale German (Germany).
' ========================================================================================
FUNCTION AfxGetLocaleNativeDisplayName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SNATIVEDISPLAYNAME, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNativeLanguageName
'------------------------------------------------------------------------------
' Native name of the language, for example, ??????? for Armenian (Armenia). The maximum
' number of characters allowed for this string is 80, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleNativeLanguageName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SNATIVELANGNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNegativeInfinityString
'------------------------------------------------------------------------------
' Windows Vista and later: String value for "negative infinity", for example, "-Infinity"
' for the English (United States) locale. There is no limit on the number of characters
' allowed for this string.
' ========================================================================================
FUNCTION AfxGetLocaleNegativeInfinityString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
   'DIM nLen AS LONG, wstrBuf AS CWSTR
   'nLen = GetLocaleInfoW(nLocale, LOCALE_SNEGINFINITY, NULL, 0)
   'IF nLen = 0 THEN RETURN ""
   'wstrBuf = SPACE(nLen)
   'nLen = GetLocaleInfoW(nLocale, LOCALE_SNEGINFINITY, @wstrBuf, nLen)
   'RETURN TRIM(wstrBuf)
	Dim As LONG nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SNEGINFINITY, NULL, 0)
	Dim pBuf$ *nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SNEGINFINITY, pBuf, nLen)
	Function = Trim$(pBuf$)
   
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleNegativeSign
'------------------------------------------------------------------------------
' String value for the negative sign, for example, "-" for the English (United States) locale.
' The maximum number of characters allowed for this string is five, including a terminating
' null character.
' ========================================================================================
FUNCTION AfxGetLocaleNegativeSign (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SNEGATIVESIGN, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleOpenTypeLanguageTag
'------------------------------------------------------------------------------
' Windows 7 and later: OpenType language tag used to retrieve culturally appropriate
' typographic features from a font.
' ========================================================================================
FUNCTION AfxGetLocaleOpenTypeLanguageTag (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SOPENTYPELANGUAGETAG, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocalePermilleSymbol
'------------------------------------------------------------------------------
' Windows 7 and later: Symbol used to indicate the permille (U+2030) symbol, that is, .
' ========================================================================================
FUNCTION AfxGetLocalePermilleSymbol (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SPERMILLE, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocalePositiveInfinityString
'------------------------------------------------------------------------------
' Windows Vista and later: String value for "positive infinity", for example, "Infinity"
' for the English (United States) locale. There is no limit on the number of characters
' allowed for the string.
' ========================================================================================
FUNCTION AfxGetLocalePositiveInfinityString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	Dim As LONG nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SPOSINFINITY, NULL, 0)
	Dim pBuf$ *nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SPOSINFINITY, pBuf, nLen)
	Function = Trim$(pBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocalePositiveSign
'------------------------------------------------------------------------------
' String value for the positive sign, for example, "+" for the English (United States)
' locale. The maximum number of characters allowed for this string is five, including a
' terminating null character.
'NOT WORKING
' ========================================================================================
FUNCTION AfxGetLocalePositiveSign (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SPOSITIVESIGN, wszBuf, cSizeOfDefaultString)
	Function =  TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleScriptsList
'------------------------------------------------------------------------------
' Windows Vista and later: A string representing a list of scripts, using the 4-character
' notation used in ISO 15924. Each script name consists of four Latin characters and the
' list is arranged in alphabetical order with each name, including the last, followed by a
' semicolon.
' ========================================================================================
FUNCTION AfxGetLocaleScriptsList (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	Dim As LONG nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SSCRIPTS, NULL, 0)
	Dim pBuf$ * nLen
	nLen = GetLocaleInfo(nLocale, LOCALE_SSCRIPTS, pBuf, nLen)
	Function = Trim$(pBuf$)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleShortDateFormatString
'------------------------------------------------------------------------------
' Short date formatting string for the locale. The maximum number of characters allowed
' for this string is 80, including a terminating null character. The string can consist of
' a combination of day, month, year, and era format pictures. For example, "M/d/yyyy"
' indicates that September 3, 2004 is written 9/3/2004.
' ========================================================================================
FUNCTION AfxGetLocaleShortDateFormatString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SSHORTDATE, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleShortestDayName
'------------------------------------------------------------------------------
' Windows Vista and later: Short native name of the specified day of the week.
' ========================================================================================
FUNCTION AfxGetLocaleShortestDayName (BYVAL nDay AS LONG, BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	IF nDay < 1 OR nDay > 7 Then
		Function =  ""
	EndIf
	GetLocaleInfo(nLocale, LOCALE_SSHORTESTDAYNAME1 + nDay - 1, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleShortTimeFormatString
'------------------------------------------------------------------------------
' Windows 7 and later: Short time formatting string for the locale. Patterns are typically
' derived by removing the "ss" (seconds) value from the long time format pattern.
' For example, if the long time format is "h:mm:ss tt", the short time format is most
' likely "h:mm tt". This constant can specify multiple formats in a semicolon-delimited
' list. However, the preferred short time format should be the first value listed.
' ========================================================================================
FUNCTION AfxGetLocaleShortTimeFormatString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SSHORTTIME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleSortName
'------------------------------------------------------------------------------
' The full localized name of the sort for the specified locale identifier, dependent on
' the language of the shell. This constant is used to determine casing and sorting behavior.
' ========================================================================================
FUNCTION AfxGetLocaleSortName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SSORTNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleThousandsSeparator
'------------------------------------------------------------------------------
' Characters that are used to separate groups of digits to the left of the decimal. The
' maximum number of characters allowed for this string is four, including a terminating
' null character. Typically, these groups represent thousands. However, depending on the
' value specified for LOCALE_SGROUPING, they can represent something else.
' ========================================================================================
FUNCTION AfxGetLocaleThousandsSeparator (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_STHOUSAND, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleDateSeparator
'------------------------------------------------------------------------------
' Character(s) for the date separator. The maximum number of characters allowed for this
' string is four, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleDateSeparator (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SDATE, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleTimeFormatString
'------------------------------------------------------------------------------
' Time formatting strings for the locale. The maximum number of characters allowed for
' this string is 80, including a terminating null character. The string can consist of a
' combination of hour, minute, and second format pictures.
' ========================================================================================
FUNCTION AfxGetLocaleTimeFormatString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_STIMEFORMAT, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleTimeSeparator
'------------------------------------------------------------------------------
' Character(s) for the time separator. The maximum number of characters allowed for this
' string is four, including a terminating null character.
' ========================================================================================
FUNCTION AfxGetLocaleTimeSeparator (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_STIME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocaleYearMonthFormatString
'------------------------------------------------------------------------------
' The year-month formatting string for the locale. The maximum number of characters allowed
' for this string is 80, including a terminating null character. This string shows the
' proper format for a date string that contains only the year and the month, using format
' pictures as defined in Day, Month, Year, and Era Format Pictures.
' ========================================================================================
FUNCTION AfxGetLocaleYearMonthFormatString (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SYEARMONTH, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocalizedCountryName
'------------------------------------------------------------------------------
' Windows 7 and later: Full localized name of the country/region, for example, Deutschland
' for Germany. The maximum number of characters allowed for this string is 80, including a
' terminating null character. Since this name is based on the localization of the product,
' it changes for each localized version.
' ========================================================================================
FUNCTION AfxGetLocalizedCountryName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SLOCALIZEDCOUNTRYNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocalizedDisplayName
'------------------------------------------------------------------------------
' Windows 7 and later: Full localized name of the locale for the user interface language,
' for example, Deutsch (Deutschland) for German (Germany)" There is no limit on the number
' of characters allowed for this string. Since this name is based on the localization of
' the product, it changes for each localized version.
' ========================================================================================
FUNCTION AfxGetLocalizedDisplayName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SLOCALIZEDDISPLAYNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetLocalizedLanguageName
'------------------------------------------------------------------------------
' Windows Vista: Full localized primary name of the user interface language included in a
' localized display name, for example, Deutsch representing German. Since this name is
' based on the localization of the product, it changes for each localized version.
' ========================================================================================
FUNCTION AfxGetLocalizedLanguageName (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SLOCALIZEDDISPLAYNAME, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxGetSortLocale
'------------------------------------------------------------------------------
' Windows 7 and later: Name of the locale to use for sorting or casing behavior.
' ========================================================================================
FUNCTION AfxGetSortLocale (BYVAL nLocale AS LONG = LOCALE_USER_DEFAULT) AS string
	DIM wszBuf$
	GetLocaleInfo(nLocale, LOCALE_SSORTLOCALE, wszBuf, cSizeOfDefaultString)
	Function = TRIM$(wszBuf)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
