' ########################################################################################
' Microsoft Windows
' File: AfxPath.inc
' Contents: Path wrapper functions.
' Compiler: C/C++ Optimizing Compiler Version 19.00.23506 for x64
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
'Ported to bc9Basic -> c/c++ by James c. Fuller
'------------------------------------------------------------------------------
'Note all references to CWSTR are normal bc9 strings in the conversion
'==============================================================================
$IFUSE USE_TCLIB
CONST INTERNET_MAX_PATH_LENGTH = 2048
CONST INTERNET_MAX_SCHEME_LENGTH = 32
$CCODE UDT
#define INTERNET_MAX_URL_LENGTH ((INTERNET_MAX_SCHEME_LENGTH + sizeof("://")) + INTERNET_MAX_PATH_LENGTH)
$CCODE
'==============================================================================
$ENDUSE
$IFUSE USE_AfxPathAddBackSlash
'------------------------------------------------------------------------------
' Adds a backslash to the end of a string to create the correct syntax for a path.
' If the source path already has a trailing backslash, no backslash will be added.
' Usage example: DIM cws AS CWSTR = AfxPathAddBackSlash("C:\dir_name\dir_name\file_name")
' ========================================================================================
FUNCTION AfxPathAddBackSlash (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH 
	_wszPath$ = wszPath
	PathAddBackslash(_wszPath)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathRemoveBackslash
'------------------------------------------------------------------------------
' Removes the trailing backslash from a given path.
' Usage eample: DIM cws AS CWSTR = AfxPathRemoveBackslash("c:\a\b\File\")
' ========================================================================================
FUNCTION AfxPathRemoveBackslash (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathRemoveBackslash(_wszPath)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathAddExtension
'------------------------------------------------------------------------------
' Adds a file extension to a path string.
' If there is already a file name extension present, no extension will be added.
' If wszPath is an empt string, the result will be the file name extension only.
' If pwszExt points to a NULL string, an ".exe" extension will be added.
' Example: DIM cws AS CWSTR = AfxPathAddExtension("file") - output: file.exe
' Example: DIM cws AS CWSTR = AfxPathAddExtension("file.doc") - output: file.doc
' Example: DIM cws AS CWSTR = AfxPathAddExtension("file", ".txt") - output: file.txt
' Example: DIM cws AS CWSTR = AfxPathAddExtension("", ".txt") - output: .txt
' ========================================================================================
FUNCTION AfxPathAddExtension (wszPath AS CONST char ptr, pwszExt AS const char Ptr = NULL) AS string
	DIM _wszPath$ * MAX_PATH 
	_wszPath$ = wszPath
	PathAddExtension(_wszPath, pwszExt)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathRemoveExtension
'------------------------------------------------------------------------------
' Removes the file extension from a path, if there is one.
' Usage example: DIM cws AS CWSTR = AfxPathRemoveExtension("C:\TEST\sample.txt")
' ========================================================================================
FUNCTION AfxPathRemoveExtension (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathRemoveExtension(_wszPath)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathRenameExtension
'------------------------------------------------------------------------------
' Replaces the extension of a file name with a new extension. If the file name does not
' contain an extension, the extension will be attached to the end of the string.
' DIM cws AS CWSTR = AfxPathRenameExtension("C:\TEST\sample.bas", ".exe")
' DIM cws AS CWSTR = AfxPathRenameExtension("C:\TEST\sample", ".exe")
' ========================================================================================
FUNCTION AfxPathRenameExtension (wszPath AS CONST char ptr, wszExt AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH 
	_wszPath$ = wszPath
	PathRenameExtension(_wszPath, wszExt)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathAppend
'------------------------------------------------------------------------------
' Appends one path to the end of another.
' This function automatically inserts a backslash between the two strings, if one is not already present.
' The path supplied in wszPath cannot begin with "..\" or ".\" to produce a relative path
' string. If present, those periods are stripped from the output string. For example,
' appending "path3" to "..\path1\path2" results in an output of "\path1\path2\path3" rather
' than "..\path1\path2\path3".
' Usage exmaple: DIM cws AS CWSTR = AfxPathAppend("name_1\name_2", "name_3")
' ========================================================================================
FUNCTION AfxPathAppend (wszPath AS CONST char ptr, wszMore AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH  
	_wszPath$ = wszPath
	PathAppend(_wszPath, wszMore)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathBuildRoot
'------------------------------------------------------------------------------
' Creates a root path from a given drive number.
' nDrive = The desired drive number. It should be between 0 and 25.
' 0 = A, 1 = B, ..., Z = 25.
' Usage example: DIM cws AS CWSTR = AfxPathBuildRoot(2) - output: C:\
' ========================================================================================
FUNCTION AfxPathBuildRoot (BYVAL nDrive AS LONG) AS string
	DIM _wszRoot$ * 4
	PathBuildRoot(_wszRoot, nDrive)
	Function = _wszRoot
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathCanonicalize
'------------------------------------------------------------------------------
' Canonicalizes a path.
' This function allows the user to specify what to remove from a path by inserting special
' character sequences into the path. The ".." sequence indicates to remove a path segment
' from the current position to the previous path segment. The "." sequence indicates to skip
' over the next path segment to the following path segment. The root segment of the path
' cannot be removed.
' If there are more ".." sequences than there are path segments, the function returns just the root.
' Examples:
' DIM cws AS CWSTR = AfxPathCanonicalize("A:\name_1\.\name_2\..\sname_3") - output: A:\name_1\name_3
' DIM cws AS CWSTR = AfxPathCanonicalize("A:\name_1\..\name_2\.\name_3") - output: A:\name_2\name_3
' DIM cws AS CWSTR = AfxPathCanonicalize("A:\name_1\name_2\.\name_3\..\name_4") - output: A:\name_1\name_2\name_4
' DIM cws AS CWSTR = AfxPathCanonicalize("A:\name_1\.\name_2\.\name_3\..\name_4\..") - output: A:\name_1\name_2
' DIM cws AS CWSTR = AfxPathCanonicalize("C:\..") - output: C:\
' ========================================================================================
FUNCTION AfxPathCanonicalize (wszPath AS CONST char ptr) AS string
	DIM _wszDest$ * MAX_PATH
	PathCanonicalize(_wszDest, wszPath)
	Function =  _wszDest
END FUNCTION
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathCombine
'------------------------------------------------------------------------------
' Concatenates two strings that represent properly formed paths into one path, as well as
' any relative path pieces. The directory path should be in the form of A:,B:, ..., Z:.
' The file path should be in a correct form that represents the file name part of the path.
' If the directory path ends with a backslash, the backslash will be maintained.
' Note that while wszDir and wszFile are both optional parameters, they cannot both be NULL.
' Example: DIM cws AS CWSTR = AfxPathCombine("C:", "One\Two\Three") - output: C:\One\Two\Three
' ========================================================================================
FUNCTION AfxPathCombine (wszDir AS CONST char ptr,  wszFile AS CONST char ptr) AS string
	DIM _wszDest$ * MAX_PATH
	PathCombine(_wszDest, wszDir, wszFile)
	Function = _wszDest
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathCommonPrefix
'------------------------------------------------------------------------------
' Compares two paths to determine if they share a prefix and returns a string with the prefix
' characters, if any. A prefix is one of these types: "C:\", ".", "..", "..\".
' Example: DIM cws AS CWSTR = AfxPathCommonPrefix("C:\win\desktop\temp.txt", "c:\win\tray\sample.txt")
' Output: C:\win
' ========================================================================================
FUNCTION AfxPathCommonPrefix (wszPath1 AS CONST char ptr, wszPath2 AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	PathCommonPrefix(wszPath1, wszPath2, _wszPath)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathCompactPath
'------------------------------------------------------------------------------
' Truncates a file path to fit within a given pixel width by replacing path components with ellipses.
' This function uses the font currently selected in hDC to calculate the width of the text.
' This function will not compact the path beyond the base file name preceded by ellipses.
' - hdc : A handle to the device context used for font metrics. This value can be NULL.
' - wszPath : The path to be modified.
' - dx : The width, in pixels, in which the string must fit.
' Usage example: DIM cws AS CWSTR = AfxPathCompactPath(NULL, "C:\path1\path2\sample.txt", 180)
' ========================================================================================
FUNCTION AfxPathCompactPath (BYVAL hDC AS HDC, wszPath AS CONST char ptr, BYVAL dx AS DWORD) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathCompactPath(hDC, _wszPath, dx)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathCompactPathEx
'------------------------------------------------------------------------------
' Truncates a path to fit within a certain number of characters by replacing path components with ellipses.
' - wszPath : The path to be modified.
' - cchMax : The maximum number of characters to be contained in the new string.
' The '/' separator will be used instead of '\' if the original string used it. If wszPath
' points to a file name that is too long, instead of a path, the file name will be truncated
' to cchMax characters. For example, if the input file name is "My Filename" and cchMax is 10,
' PathCompactPathEx will return "My File...".
' Usage example: DIM cws AS CWSTR = AfxPathCompactPathEx("c:\test\My Filename", 18)
' ========================================================================================
FUNCTION AfxPathCompactPathEx (wszPath AS CONST char ptr, BYVAL cchMax AS DWORD) AS string
	DIM _wszOut$ * MAX_PATH
	IF cchMax > MAX_PATH Then
		cchMax = MAX_PATH - 1
	EndIf
	PathCompactPathEx(_wszOut, wszPath, cchMax + 1, 0)
	Function =  LEFT$(_wszOut, cchMax)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathCreateFromUrl
'------------------------------------------------------------------------------
' Takes a file URL and converts it to a Microsoft MS-DOS path.
' - wszUrl : A null-terminated string of maximum length INTERNET_MAX_URL_LENGTH that contains the URL.
' Example: DIM cws AS CWSTR = AfxPathCreateFromUrl("file:///C:/Documents%20and%20Settings/URIS.txt")
' Output: C:\Documents and Settings\URIS.txt
' ========================================================================================
FUNCTION AfxPathCreateFromUrl (wszUrl AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	DIM  AS DWORD cchPath = MAX_PATH
	PathCreateFromUrl(wszUrl, _wszPath, &cchPath, 0)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathMakePretty
'------------------------------------------------------------------------------
' Converts a path to all lowercase characters to give the path a consistent appearance.
' This function only operates on paths that are entirely uppercase. For example:
' C:\WINDOWS will be converted to c:\windows, but c:\Windows will not be changed.
' Example: DIM cws AS CWSTR = AfxPathMakePretty("C:\TEST\FILE")
' ========================================================================================
FUNCTION AfxPathMakePretty (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathMakePretty(_wszPath)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathQuoteSpaces
'------------------------------------------------------------------------------
' Searches a path for spaces. If spaces are found, the entire path is enclosed in
' quotation marks.
' Example: DIM cws AS CWSTR = AfxPathQuoteSpaces("C:\sample_one\sample two")
' Output: "C:\sample_one\sample two"
' ========================================================================================
FUNCTION AfxPathQuoteSpaces (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathQuoteSpaces(_wszPath)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathUnquoteSpaces
'------------------------------------------------------------------------------
' Removes quotes from the beginning and end of a path.
' Usage example: DIM cws AS CWSTR = AfxPathUnquoteSpaces("""C:\sample_one\sample two""")
' Output: C:\sample_one\sample two
' ========================================================================================
FUNCTION AfxPathUnquoteSpaces (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH 
	_wszPath$ = wszPath
	PathUnquoteSpaces(_wszPath)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathFileExists
'------------------------------------------------------------------------------
' Determines whether a path to a file system object such as a file or directory is valid.
' TRUE if the file exists; otherwise, FALSE. Call GetLastError for extended error information.
' This function tests the validity of the path. A path specified by Universal Naming
' Convention (UNC) is limited to a file only; that is, \\server\share\file is permitted.
' A UNC path to a server or server share is not permitted; that is, \\server or \\server\share.
' This function returns FALSE if a mounted remote drive is out of service.
' Example: DIM b AS BOOLEAN = AfxPathFileExists("C:\TEST\file.txt")
' ========================================================================================
FUNCTION AfxPathFileExists (wszPath AS CONST char ptr) AS BOOL
	Function =  PathFileExists(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathFindExtension
'------------------------------------------------------------------------------
' Searches a path for an extension.
' wszPath : The path to search, including the extension being searched for.
' Note that a valid file name extension cannot contain a space.
' Usage example: DIM cws AS CWSTR = AfxPathFindExtension("C:\TEST\filetxt")
' ========================================================================================
FUNCTION AfxPathFindExtension (wszPath AS CONST char Ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	Raw AS const char ptr pFileExt= PathFindExtension(_wszPath) 
	IF pFileExt THEN
		_wszPath$ = pFileExt
	EndIf
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathFindFileName
'------------------------------------------------------------------------------
' Searches a path for a file name.
' ========================================================================================
FUNCTION AfxPathFindFileName (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH 
	_wszPath$ = wszPath
	Raw As char ptr pFileName = PathFindFileName(_wszPath) 
	If pFileName Then
		_wszPath$ = pFileName$
	EndIf
	Function =  _wszPath
END FUNCTION

'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathFindNextComponent
'------------------------------------------------------------------------------
' Parses a path and returns the portion of that path that follows the first backslash.
' PathFindNextComponent walks a path string until it encounters a backslash ("\\"), ignores
' everything up to that point including the backslash, and returns the rest of the path.
' Therefore, if a path begins with a backslash (such as \path1\path2), the function simply
' removes the initial backslash and returns the rest (path1\path2).
' Usage example: DIM cws AS CWSTR = PathFindNextComponentW("C:\TEST\file.txt")
' ========================================================================================
FUNCTION AfxPathFindNextComponent (wszPath AS char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	Raw As char Ptr pwszComp = PathFindNextComponent(_wszPath) 
	IF pwszComp Then
		_wszPath$ = pwszComp$
	EndIf
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathSkipRoot
'------------------------------------------------------------------------------
' Parses a path, ignoring the drive letter or Universal Naming Convention (UNC)
' server/share path parts.
' ========================================================================================
FUNCTION AfxPathSkipRoot (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH 
	_wszPath$ = wszPath
	Raw As char Ptr pwszPath = PathSkipRoot(_wszPath) 
	If pwszPath Then
		_wszPath$ = pwszPath$
	EndIf
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
'=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
'NOT IMPLEMENTED
'=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
'$IFUSE USE_AfxPathFindSuffixArray
'------------------------------------------------------------------------------
' *** UNTESTED ***
' Determines if a given file name has one of a list of suffixes.
' - wszPath : The file name to be tested. A full path can be used.
' - apwszSuffix : An array of iArraySize string pointers. Each string pointed to is
'   null-terminated and contains one suffix. The strings can be of variable lengths.
' - iArraySize : The number of elements in the array pointed to by apszSuffix.
' This function uses a case-sensitive comparison. The suffix must match exactly.
' ========================================================================================
'PRIVATE FUNCTION AfxPathFindSuffixArray (BYREF wszPath AS WSTRING, BYVAL apwszSuffix AS WSTRING PTR PTR, BYVAL iArraySize AS LONG) AS CWSTR
'   DIM pwszSuffix AS WSTRING PTR = cast(WSTRING PTR, PathFindSuffixArrayW(wszPath, apwszSuffix, iArraySize))
'   IF pwszSuffix THEN RETURN *pwszSuffix
'END FUNCTION
'------------------------------------------------------------------------------
'$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathGetArgs
'------------------------------------------------------------------------------
' Finds the command line arguments within a given path.
' This function should not be used on generic command path templates (from users or the
' registry), but rather should be used only on templates that the application knows to be
' well formed.
' Example: DIM cws AS CWSTR = AfxPathGetArgs("test.exe temp.txt sample.doc")
' ========================================================================================
FUNCTION AfxPathGetArgs (wszPath AS CONST char ptr) AS string
	Raw As char Ptr pwszArgs = PathGetArgs(wszPath)
	If pwszArgs Then
		Function = pwszArgs$
	EndIf
	Function = ""
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathGetCharType
'------------------------------------------------------------------------------
' Determines the type of character with respect to a path.
' Returns one or more of the following values that define the type of character.
' GCT_INVALID   : The character is not valid in a path.
' GCT_LFNCHAR   : The character is valid in a long file name.
' GCT_SEPARATOR : The character is a path separator.
' GCT_SHORTCHAR : The character is valid in a short (8.3) file name.
' GCT_WILD      : The character is a wildcard character.
' ========================================================================================
FUNCTION AfxPathGetCharType (wszCh AS CONST char ptr) AS UINT
	Function = PathGetCharType(ASC(wszCh))
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathGetDriveNumber
'------------------------------------------------------------------------------
' Searches a path for a drive letter within the range of 'A' to 'Z' and returns the
' corresponding drive number, 0 through 25 (corresponding to 'A' through 'Z') if the path
' has a drive letter, or -1 otherwise.
' Example: DIM n AS LONG = AfxPathGetDriveNumber("C:\TEST\bar.txt") - output: 2
' ========================================================================================
FUNCTION AfxPathGetDriveNumber (wszPath AS CONST char ptr) AS LONG
	Function =  PathGetDriveNumber(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsContentType
'------------------------------------------------------------------------------
' Determines if a file's registered content type matches the specified content type.
' This function obtains the content type for the specified file type and compares
' that string with the wszContentType The comparison is not case sensitive.
' Returns nonzero if the file's registered content type matches wszContentType, or zero otherwise.
' Example: DIM b AS BOOLEAN = AfxPathIsContentType("C:\TEST\bar.txt", "text/plain") - output: true
' Example: DIM b AS BOOLEAN = AfxPathIsContentType("C:\TEST\bar.txt", "image/gif") - output: false
' ========================================================================================
FUNCTION AfxPathIsContentType (wszPath AS CONST char ptr, wszContentType AS CONST char ptr) AS BOOL
	Function = PathIsContentType(wszPath, wszContentType)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsDirectory
'------------------------------------------------------------------------------
' Verifies that a path is a valid directory.
' ========================================================================================
FUNCTION AfxPathIsDirectory (wszPath AS CONST char ptr) AS BOOL
	Function =  PathIsDirectory(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsDirectoryEmpty
'------------------------------------------------------------------------------
' Determines whether or not a specified path is an empty directory.
' ========================================================================================
FUNCTION AfxPathIsDirectoryEmpty (wszPath AS CONST char ptr) AS BOOL
	Function = PathIsDirectoryEmpty(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsFileSpec
'------------------------------------------------------------------------------
' Searches a path for any path delimiting characters (for example, ':' or '\' ).
' If there are no path delimiting characters present, the path is considered to be
' a File Spec path.
' ========================================================================================
FUNCTION AfxPathIsFileSpec (wszPath AS CONST char ptr) AS BOOL
	Function = PathIsFileSpec(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsHTMLFile
'------------------------------------------------------------------------------
' Determines if a file is an HTML file. The determination is made based on the
' content type that is registered for the file's extension.
' ========================================================================================
FUNCTION AfxPathIsHTMLFile (wszFile AS CONST char ptr) AS BOOL
	Function = PathIsHTMLFile(wszFile)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsLFNFileSpec
'------------------------------------------------------------------------------
' Determines whether or not a file name is in long format.
' ========================================================================================
FUNCTION AfxPathIsLFNFileSpec (wszName AS CONST char ptr) AS BOOL
	Function = PathIsLFNFileSpec(wszName)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsNetworkPath
'------------------------------------------------------------------------------
' Determines whether a path string represents a network resource.
' ========================================================================================
FUNCTION AfxPathIsNetworkPath (wszPath AS CONST char Ptr) AS BOOL
	Function = PathIsNetworkPath(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsPrefix
'------------------------------------------------------------------------------
' Searches a path to determine if it contains a valid prefix of the type passed by
' szPrefix. A prefix is one of these types: "C:\", ".", "..", "..\".
' Example: DIM b AS BOOLEAN = AfxPathIsPrefix("C:\", "C:\test\Some\sample")
' ========================================================================================
FUNCTION AfxPathIsPrefix (wszPrefix AS CONST char ptr, wszPath AS CONST char ptr) AS BOOL
	Function =  PathIsPrefix(wszPrefix, wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsRelative
'------------------------------------------------------------------------------
' Searches a path and determines if it is relative.
' ========================================================================================
FUNCTION AfxPathIsRelative (wszPath AS CONST char Ptr) AS BOOL
	Function = PathIsRelative(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsRoot
'------------------------------------------------------------------------------
' Parses a path to determine if it is a directory root.
' ========================================================================================
FUNCTION AfxPathIsRoot (wszPath AS CONST char ptr) AS BOOL
	Function = PathIsRoot(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsSameRoot
'------------------------------------------------------------------------------
' Compares two paths to determine if they have a root component.
' ========================================================================================
FUNCTION AfxPathIsSameRoot (wszPath1 AS CONST char ptr, wszPath2 AS CONST char Ptr) AS BOOL
	Function = PathIsSameRoot(wszPath1, wszPath2)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsSystemFolder
'------------------------------------------------------------------------------
' Determines if an existing folder contains the attributes that make it a system
' folder. Alternately indicates if certain attributes qualify a folder to be a
' system folder.
' ========================================================================================
FUNCTION AfxPathIsSystemFolder (wszPath AS CONST char ptr, dwAttrb AS DWORD) AS BOOL
	Function = PathIsSystemFolder(wszPath, dwAttrb)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsUNC
'------------------------------------------------------------------------------
' Determines if the string is a valid Universal Naming Convention (UNC) for a
' server and share path.
' ========================================================================================
FUNCTION AfxPathIsUNC (wszPath AS CONST char ptr) AS BOOL
	Function = PathIsUNC(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsUNCServer
'------------------------------------------------------------------------------
' Determines if a string is a valid Universal Naming Convention (UNC) for a server
' path only.
' ========================================================================================
FUNCTION AfxPathIsUNCServer (wszPath AS CONST char ptr) AS BOOL
	Function = PathIsUNCServer(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsUNCServerShare
'------------------------------------------------------------------------------
' Determines if a string is a valid Universal Naming Convention (UNC) share path,
' \\ server\ share.
' ========================================================================================
FUNCTION AfxPathIsUNCServerShare (wszPath AS CONST char ptr) AS BOOL
	Function = PathIsUNCServerShare(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathIsURL
'------------------------------------------------------------------------------
' Tests a given string to determine if it conforms to a valid URL format.
' ========================================================================================
FUNCTION AfxPathIsURL (wszPath AS CONST char ptr) AS BOOL
	Function = PathIsURL(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathMakeSystemFolder
'------------------------------------------------------------------------------
' Gives an existing folder the proper attributes to become a system folder.
' ========================================================================================
FUNCTION AfxPathMakeSystemFolder (wszPath AS CONST char ptr) AS BOOL
	Function = PathMakeSystemFolder(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathUnmakeSystemFolder
'------------------------------------------------------------------------------
' Removes the attributes from a folder that make it a system folder. This folder
' must actually exist in the file system.
' ========================================================================================
FUNCTION AfxPathUnmakeSystemFolder (wszPath AS CONST char ptr) AS BOOL
	Function = PathUnmakeSystemFolder(wszPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathFindOnPath
'------------------------------------------------------------------------------
' Searches for a file.
' PathFindOnPath searches for the file specified by wszFile. If no directories are specified
' in ppszOtherDirs, it attempts to find the file by searching standard directories such as
' System32 and the directories specified in the PATH environment variable. To expedite the
' process or enable PathFindOnPath to search a wider range of directories, use the ppszOtherDirs
' parameter to specify one or more directories to be searched first. If more than one file
' has the name specified by pszFile, PathFindOnPath returns the first instance it finds.
' ========================================================================================
'I'm not sure on the ppwszOtherDirs
FUNCTION AfxPathFindOnPath (wszFile AS CONST char ptr, ppwszOtherDirs AS const char ptr ptr = NULL) AS string
	DIM _wszFile$  * MAX_PATH
	_wszFile$ = wszFile
	PathFindOnPath(_wszFile, ppwszOtherDirs)
	Function = _wszFile
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathMatchSpec
'------------------------------------------------------------------------------
' Searches a string using an Microsoft MS-DOS wild card match type.
' ========================================================================================
FUNCTION AfxPathMatchSpec (wszFile AS CONST char ptr,wszSpec AS CONST char ptr) AS BOOL
	Function = PathMatchSpec(wszFile, wszSpec)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathRemoveArgs
'------------------------------------------------------------------------------
' Removes any arguments from a given path.
' This function should not be used on generic command path templates (from users or the
' registry), but rather it should be used only on templates that the application knows to
' be well formed.
' Usage: DIM cws AS CWSTR = AfxPathRemoveArgs("c:\a\b\FileA Arg1 Arg2")
' ========================================================================================
FUNCTION AfxPathRemoveArgs (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathRemoveArgs(_wszPath)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathRemoveBlanks
'------------------------------------------------------------------------------
' Removes all leading and trailing spaces from a path.
' ========================================================================================
FUNCTION AfxPathRemoveBlanks (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathRemoveBlanks(_wszPath)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathRemoveFileSpec
'------------------------------------------------------------------------------
' Removes the trailing file name and backslash from a path, if it has them.
' ========================================================================================
FUNCTION AfxPathRemoveFileSpec (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathRemoveFileSpec(_wszPath)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathSearchAndQualify
'------------------------------------------------------------------------------
' Determines if a given path is correctly formatted and fully qualified.
' ========================================================================================
FUNCTION AfxPathSearchAndQualify (wszPath AS CONST char ptr, wszFullyQualifiedPath AS char ptr, cchFullyQualifiedPath AS DWORD) AS BOOL
	Function = PathSearchAndQualify(wszPath, wszFullyQualifiedPath, cchFullyQualifiedPath)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathSetDlgItemPath
'------------------------------------------------------------------------------
' Sets the text of a child control in a window or dialog box, using
' PathCompactPath to make sure the path fits in the control.
' ========================================================================================
SUB AfxPathSetDlgItemPath (hDlg AS HWND, cID AS LONG, wszPath AS CONST char ptr)
	PathSetDlgItemPath(hDlg, cID, wszPath)
END SUB
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathStripPath
'------------------------------------------------------------------------------
' Removes the path portion of a fully qualified path and file.
' ========================================================================================
FUNCTION AfxPathStripPath (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathStripPath(_wszPath)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathStripToRoot
'------------------------------------------------------------------------------
' Removes all parts of the path except for the root information.
' ========================================================================================
FUNCTION AfxPathStripToRoot (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathStripToRoot(_wszPath)
	Function =  _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathUndecorate
'------------------------------------------------------------------------------
' Removes the decoration from a path string.
' ========================================================================================
FUNCTION AfxPathUndecorate (wszPath AS CONST char ptr) AS string
	DIM _wszPath$ * MAX_PATH
	_wszPath$ = wszPath
	PathUndecorate(_wszPath)
	Function = _wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathUnExpandEnvStrings
'------------------------------------------------------------------------------
' Takes a fully qualified path, and replaces several folder names with their
' associated environment string.
' ========================================================================================
FUNCTION AfxPathUnExpandEnvStrings (wszPath AS CONST char ptr) AS string
	DIM _wszBuf$ * MAX_PATH
	PathUnExpandEnvStrings(wszPath, _wszBuf, MAX_PATH)
	Function =  _wszBuf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathParseIconLocation
'------------------------------------------------------------------------------
' Parses a file location string containing a file location and icon index, and
' returns the valid icon index value.
' ========================================================================================
FUNCTION AfxPathParseIconLocation (wszIconFile AS CONST char ptr) AS string
	DIM _wszIconFile$ * MAX_PATH
	_wszIconFile$ = wszIconFile
	PathParseIconLocation(_wszIconFile)
	Function =  _wszIconFile
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathRelativePathTo
'------------------------------------------------------------------------------
' Creates a relative path from one file or folder to another.
' This function takes a pair of paths and generates a relative path from one to the other.
' The paths do not have to be fully qualified, but they must have a common prefix, or the
' function will fail and return FALSE.
' For example, let the starting point, wszFrom, be "c:\FolderA\FolderB\FolderC", and the
' ending point, wszTo, be "c:\FolderA\FolderD\FolderE". PathRelativePathTo will return the
' relative path from wszFrom to wszTo as: "..\..\FolderD\FolderE". You will get the same
' result if you set wszFrom to "\FolderA\FolderB\FolderC" and wszTo to "\FolderA\FolderD\FolderE".
' On the other hand, "c:\FolderA\FolderB" and "a:\FolderA\FolderD do not share a common
' prefix, and the function will fail. Note that "\\" is not considered a prefix and is ignored.
' If you set wszFrom to "\\FolderA\FolderB", and wszTo to "\\FolderC\FolderD", the function will fail.
' ========================================================================================
FUNCTION AfxPathRelativePathTo (wszFrom AS CONST char ptr, dwAttrFrom AS DWORD, wszTo AS CONST char ptr, dwAttrTo AS DWORD) AS string
	DIM _wszPath$ * MAX_PATH
	IF PathRelativePathToW(_wszPath, wszFrom, dwAttrFrom, wszTo, dwAttrTo) Then
		Function = _wszPath
	EndIf
	Function = ""
END FUNCTION
' ========================================================================================
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlApplyScheme
'------------------------------------------------------------------------------
' Takes a URL string, determines a scheme for it, and returns a string with an
' appropriate prefix.
' ========================================================================================
FUNCTION AfxUrlApplyScheme (wszUrl AS char ptr, dwFlags AS DWORD) AS string
	DIM _wszOut$ * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchOut = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr  = UrlApplyScheme(wszUrl, _wszOut, &cchOut, dwFlags)
	If hr = S_OK OR hr = S_FALSE THEN
		Function = _wszOut
	Else 
		Function = wszUrl
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlCanonicalize
'------------------------------------------------------------------------------
' Takes a URL string and converts it into canonical form.
' ========================================================================================
FUNCTION AfxUrlCanonicalize (wszUrl AS char ptr, BYVAL dwFlags AS DWORD) AS string
	DIM _wszCanonicalized$ * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchCanonicalized  = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr = UrlCanonicalize(wszUrl, _wszCanonicalized, &cchCanonicalized, dwFlags)
	IF hr = S_OK Then
		Function = _wszCanonicalized
	Else
		Function = wszUrl
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlCombine
'------------------------------------------------------------------------------
' When provided with a relative URL and its base, returns a URL in canonical form.
' ========================================================================================
FUNCTION AfxUrlCombine (wszBase AS char ptr, wszRelative AS char ptr, BYVAL dwFlags AS DWORD) AS string
	DIM _wszCombined$ * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchCombined = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr = UrlCombine(wszBase, wszRelative, _wszCombined, &cchCombined, dwFlags)
	IF hr = S_OK Then
		Function = _wszCombined
	Else
		Function =  wszBase
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlCompare
'------------------------------------------------------------------------------
' Makes a case-sensitive comparison of two URL strings.
' ========================================================================================
FUNCTION AfxUrlCompare (wszURL1 AS CONST char ptr, wszURL2 AS CONST char ptr, fIgnoreSlash AS LONG) AS BOOL
	Function = UrlCompare(wszURL1, wszURL2, fIgnoreSlash)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlCreateFromPath
'------------------------------------------------------------------------------
' Takes a Microsoft MS-DOS path and converts it to a canonicalized URL.
' ========================================================================================
FUNCTION AfxUrlCreateFromPath (wszPath AS char ptr) AS string
	DIM _wszUrl$ * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchUrl = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr  = UrlCreateFromPath(wszPath, _wszUrl, &cchUrl, 0)
	IF hr = S_OK OR hr = S_FALSE THEN
		Function = _wszUrl
	Else
		Function =  wszPath
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlEscape
'------------------------------------------------------------------------------
' Converts characters in a URL that might be altered during transport across the
' Internet ("unsafe" characters) into their corresponding escape sequences.
' ========================================================================================
FUNCTION AfxUrlEscape (wszURL AS char ptr, BYVAL dwFlags AS DWORD) AS string
	DIM _wszEscaped$ * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchEscaped = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr = UrlEscapeW(wszURL, _wszEscaped, &cchEscaped, 0)
	IF hr = S_OK THEN
		Function = _wszEscaped
	Else
		Function = wszURL
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlEscapeSpaces
'------------------------------------------------------------------------------
' Converts space characters into their corresponding escape sequence.
' ========================================================================================
FUNCTION AfxUrlEscapeSpaces (wszURL AS char ptr) AS string
	DIM _wszEscaped$ * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchEscaped = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr = UrlEscapeSpaces(wszURL, _wszEscaped, &cchEscaped)
	IF hr = S_OK THEN
		Function = _wszEscaped
	Else
		Function = wszURL
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlGetLocation
'------------------------------------------------------------------------------
' Retrieves the location from a URL.
' ========================================================================================
FUNCTION AfxUrlGetLocation (wszUrl AS CONST char ptr) AS string
	DIM _wszUrl$ * MAX_PATH
	_wszUrl$ = wszUrl
	DIM AS char ptr pwszLocation =  (char*)UrlGetLocation(_wszUrl)
	IF pwszLocation THEN
		_wszUrl$ = pwszLocation$
	EndIf
	Function =  _wszUrl
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlGetPart
'------------------------------------------------------------------------------
' Accepts a URL string and returns a specified part of that URL.
' ========================================================================================
FUNCTION AfxUrlGetPart (wszUrl AS CONST char ptr, dwPart AS DWORD, dwFlags AS DWORD) AS string
	DIM _wszOut$ * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchOut = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr = UrlGetPart(wszUrl, _wszOut, &cchOut, dwPart, dwFlags)
	IF hr = S_OK THEN
		Function = _wszOut
	Else
		Function = ""
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlHash
'------------------------------------------------------------------------------
' Hashes a URL string.
' ========================================================================================
FUNCTION AfxUrlHash (wszURL AS CONST char ptr, pbHash AS BYTE PTR,  cbHash AS DWORD) AS HRESULT
	Function = UrlHash(wszURL, pbHash, cbHash)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlIs
'------------------------------------------------------------------------------
' Tests whether or not a URL is a specified type.
' ========================================================================================
FUNCTION AfxUrlIs (wszURL AS CONST char ptr, nUrlIs AS URLIS) AS BOOL
	Function = UrlIs(wszURL, nUrlIs)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlIsFileUrl
'------------------------------------------------------------------------------
' Tests whether or not a URL is a specified type.
' ========================================================================================
FUNCTION AfxUrlIsFileUrl (wszURL AS CONST char ptr) AS BOOL
	Function = UrlIs(wszURL, URLIS_FILEURL)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlIsNoHistory
'------------------------------------------------------------------------------
' Returns whether or not a URL is a No History URL. (had to keep the "W")
' ========================================================================================
FUNCTION AfxUrlIsNoHistory (wszURL AS CONST char ptr) AS BOOL
	Function = UrlIsNoHistoryW(wszURL)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlIsOpaque
'------------------------------------------------------------------------------
' Returns whether a URL is opaque.
' ========================================================================================
FUNCTION AfxUrlIsOpaque (wszURL AS CONST char ptr) AS BOOL
	Function = UrlIsOpaque(wszURL)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlUnescape
'------------------------------------------------------------------------------
' Converts escape sequences back into ordinary characters.
' ========================================================================================
FUNCTION AfxUrlUnescape ( wszURL AS char ptr, BYVAL dwFlags AS DWORD) AS string
	DIM _wszUnescaped$  * INTERNET_MAX_URL_LENGTH
	DIM AS DWORD cchUnescaped = INTERNET_MAX_URL_LENGTH
	DIM AS HRESULT hr = UrlUnescape(wszURL, _wszUnescaped, &cchUnescaped, 0)
	IF hr = S_OK THEN
		IF (dwFlags BAND URL_UNESCAPE_INPLACE) <> URL_UNESCAPE_INPLACE THEN
			Function = _wszUnescaped
		Else
			Function = wszURL
		End If	
	Else
		Function = wszURL
	END IF
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxUrlUnescapeInPlace
'------------------------------------------------------------------------------
' Converts escape sequences back into ordinary characters and overwrites the
' original string.
' ========================================================================================
FUNCTION AfxUrlUnescapeInPlace (wszURL AS char ptr, BYVAL dwFlags AS DWORD) AS HRESULT
	Function = UrlUnescapeW(wszURL, NULL, NULL, dwFlags BOR URL_UNESCAPE_INPLACE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_AfxPathMatchSpecEx
'------------------------------------------------------------------------------
' Searches a path to determine whether it contains a file of a specified file type extension.
' Returns TRUE if a file name pattern specified in wszSpec matched the file name found in
' the string pointed to by pszFile; otherwise, it returns FALSE.
' Windows Vista. Not declared in the FB includes.
' Example: DIM b AS BOOLEAN = AfxPathMatchSpec("c:\test\file.txt", "*.txt")
' ========================================================================================
FUNCTION AfxPathMatchSpecEx (wszFile AS CONST char ptr, wszSpec AS CONST char ptr, dwFlags AS DWORD = 0) AS BOOL
	If PathMatchSpecEx(wszFile,wszSpec,dwFlags) = S_OK Then
		Function = TRUE
	Else
		Function = FALSE
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
