' ########################################################################################
' Microsoft Windows
' File: CFindFile.inc
' Contents: File find class. Performs local file searches.
' Compiler: C/C++ Optimizing Compiler Version 19.00.23506 for x64
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
'Ported to bc9Basic -> c/c++ by James c. Fuller
'=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
'Full $USE usage:
'------------------------------------------------------------------------------
'$USE USE_CFindFile::FileAttributes USE_CFindFile::IsDots USE_CFindFile::IsFile USE_CFindFile::IsFolder
'$USE USE_CFindFile::IsSystemFile USE_CFindFile::IsHiddenFile USE_CFindFile::IsReadOnlyFile USE_CFindFile::IsTemporaryFile
'$USE USE_CFindFile::IsNormalFile USE_CFindFile::IsCompressedFile USE_CFindFile::IsEncryptedFile
'$USE USE_CFindFile::IsNotContentIndexedFile USE_CFindFile::IsOffLineFile USE_CFindFile::IsReparsePointFile
'$USE USE_CFindFile::IsSparseFile USE_CFindFile::MatchesMask USE_CFindFile::FileName USE_CFindFile::FileNameX
'$USE USE_CFindFile::ShortFilename USE_CFindFile::FileExt USE_CFindFile::FilePath USE_CFindFile::FileURL
'$USE USE_CFindFile::Root USE_CFindFile::FileSize USE_CFindFile::CreationTime USE_CFindFile::LastAccessTime
'$USE USE_CFindFile::LastWriteTime
'------------------------------------------------------------------------------
'=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
Class CFindFile
	public:
		Raw As HANDLE m_hFile
		Raw As WIN32_FIND_DATAW m_FileInfo
		Raw As char m_wszFileSpec[MAX_PATH]
		Dim Constructor CFindFile()
		Dim Destructor ~CFindFile()
		Dim Sub Close()
		Dim Function FindFile (wszFileSpec AS char ptr) AS LONG_PTR
		Dim Function FindNext () AS LONG
		Dim Function FileAttributes () AS DWORD
		Dim Function IsDots () AS BOOL
		Dim Function IsFile () AS BOOL
		Dim Function IsFolder () AS BOOL
		Dim Function IsSystemFile () AS BOOL
		Dim Function IsHiddenFile () AS BOOL
		Dim Function IsReadOnlyFile () AS BOOL
		Dim Function IsTemporaryFile () AS BOOL
		Dim Function IsNormalFile () AS BOOL
		Dim Function IsCompressedFile () AS BOOL
		Dim Function IsEncryptedFile () AS BOOL
		Dim Function IsNotContentIndexedFile () AS BOOL
		Dim Function IsOffLineFile () AS BOOL
		Dim Function IsReparsePointFile () AS BOOL
		Dim Function IsSparseFile () AS BOOL
		Dim Function MatchesMask (BYVAL dwMask AS DWORD) AS BOOL
		Dim Function FileName () AS char ptr
		Dim Function ShortFileName () AS char ptr
		Dim Function FileNameX () AS char ptr
		Dim Function FileExt () AS char ptr
		Dim Function FilePath () AS char ptr
		Dim Function FileURL () AS char ptr
		Dim Function Root () AS char ptr
		Dim Function FileSize () AS ULONGLONG
		Dim Function CreationTime () AS FILETIME
		Dim Function LastAccessTime () AS FILETIME
		Dim Function LastWriteTime () AS FILETIME
End Class
CLASS_METHODS_START CFindFile
Constructor CFindFile()

End Constructor
'------------------------------------------------------------------------------
Destructor CFindFile()
	If m_hFile <> 0 AND m_hFile <> INVALID_HANDLE_VALUE Then
		FindClose(m_hFile)
	EndIf
End Destructor
'==============================================================================
' Close method
' Call this method to end the search, reset the context, and release all resources.
' After calling Close, you do not have to create a new CFindFile instance before
' calling FindFile to begin a new search.
'------------------------------------------------------------------------------
SUB Close()
	IF m_hFile <> 0 AND m_hFile <> INVALID_HANDLE_VALUE THEN
	  FindClose(m_hFile)
	  m_hFile = 0
	  m_wszFileSpec$ = ""
	'      RESET m_FileInfo
	END IF
END SUB
'==============================================================================
' Method FindFile
' Call this method to open a file search.
' Parameter:
' wszName
'   A string containing the name of the file to find (must not end in a trailing
'   backslash). If you pass an empty string, FindFile does a wildcard (*.*) search.
'   If the string ends with a wildcard, period (.), or directory name, the user must
'   have access permissions to the root and all subdirectories on the path.
'   To extend the MAX_PATH limit to 32,767 wide characters, prepend "\\?\" to the path.
' Return value:
'   S_OK on success or an error code on failure.
'   To get extended error information, call GetLastError.
'   If the function fails because no matching files can be found, the GetLastError
'   function returns ERROR_FILE_NOT_FOUND.
'------------------------------------------------------------------------------
FUNCTION FindFile ( wszFileSpec AS char ptr) AS LONG_PTR
	this->Close()   ' // Close previous search, if any
	m_wszFileSpec$ = wszFileSpec$   ' // For the GetRoot method
	IF m_wszFileSpec = "" THEN
		m_wszFileSpec$ = "*.*"
	EndIf
	m_hFile = FindFirstFile(m_wszFileSpec, &m_FileInfo)
	IF m_hFile = INVALID_HANDLE_VALUE THEN
		Function = (LONG_PTR) INVALID_HANDLE_VALUE
	EndIf
	Function =  S_OK
END FUNCTION
'==============================================================================
' Method FindNext
' Call this method to continue a file search from a previous call to FindFile.
' Return value:
'   Nonzero if there are more files; zero if the file found is the last one in the
'   directory or if an error occurred. To get extended error information, call GetLastError.
'   If the file found is the last file in the directory, or if no matching files can be
'   found, the GetLastError function returns ERROR_NO_MORE_FILES.
'------------------------------------------------------------------------------
FUNCTION FindNext () AS LONG
	Function = FindNextFileW(m_hFile, &m_FileInfo)
END FUNCTION
'==============================================================================
$IFUSE USE_CFindFile::FileAttributes
' Method FileAttributes
' Call this method to get the attributes of the last found file.
'------------------------------------------------------------------------------
FUNCTION FileAttributes () AS DWORD
	Function =  m_FileInfo.dwFileAttributes
END FUNCTION
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsDots
'==============================================================================
' Method IsDots
' Call this method to test for the current directory and parent directory markers while
' iterating through files.
' Return value:
'   TRUE if the found file has the name "." or "..", which indicates that the found
'   file is actually a directory. Otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsDots () AS BOOL
	Function = iif (ASC(m_FileInfo.cFileName) EqualTo 46,TRUE,FALSE)
END FUNCTION
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsFile
'==============================================================================
' Method IsFile
' Call this method to determine if the found file is a file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsFile () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_DIRECTORY) _
	                  NotEqualTo FILE_ATTRIBUTE_DIRECTORY),TRUE,FALSE)
END FUNCTION
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsFolder
'==============================================================================
' Method IsFolder
' Call this method to determine if the found file is a folder.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsFolder () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_DIRECTORY) _
	                  EqualTo FILE_ATTRIBUTE_DIRECTORY),TRUE,FALSE)
END FUNCTION
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsSystemFile
'------------------------------------------------------------------------------
' Method IsSystemFile
' Call this method to determine if the found file is a system file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsSystemFile () AS BOOL
	Function =	iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_SYSTEM) _ 
	                   EqualTo FILE_ATTRIBUTE_SYSTEM),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsHiddenFile
' Method IsHiddenFile
' Call this method to determine if the found file is a hidden file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsHiddenFile () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_HIDDEN) _
	                   EqualTo FILE_ATTRIBUTE_HIDDEN),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsReadOnlyFile
'------------------------------------------------------------------------------
' Method IsReadOnlyFile
' Call this method to determine if the found file is a read only file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsReadOnlyFile () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_READONLY) _
	                  EqualTo FILE_ATTRIBUTE_READONLY),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsTemporaryFile
'------------------------------------------------------------------------------
' Method IsTemporaryFile
' Call this method to determine if the found file is a temporary file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsTemporaryFile () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_TEMPORARY) _
	                  EqualTo FILE_ATTRIBUTE_TEMPORARY),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsNormalFile
'------------------------------------------------------------------------------
' Method IsNormalFile
' Call this method to determine if the found file is a normal file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsNormalFile () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_NORMAL) _
		              EqualTo FILE_ATTRIBUTE_NORMAL),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsCompressedFile
'------------------------------------------------------------------------------
' Method IsCompressedFile
' Call this method to determine if the found file is a compressed file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsCompressedFile () AS BOOL
	Function =  iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_COMPRESSED) _
	                   EqualTo FILE_ATTRIBUTE_COMPRESSED),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsEncryptedFile
'------------------------------------------------------------------------------
' Method IsEncryptedFile
' Call this method to determine if the found file is a encrypted file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsEncryptedFile () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_ENCRYPTED) _
	                  EqualTo FILE_ATTRIBUTE_ENCRYPTED),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsNotContentIndexedFile
'------------------------------------------------------------------------------
' Method IsNotContentIndexedFile
' Call this method to determine if the found file is not to be indexed by the content indexing service.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsNotContentIndexedFile () AS BOOL
	Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_NOT_CONTENT_INDEXED) _
	                  EqualTo FILE_ATTRIBUTE_NOT_CONTENT_INDEXED),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsOffLineFile
'------------------------------------------------------------------------------
' Method IsOffLineFile
' Call this method to determine if the found file is not available immediately.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsOffLineFile () AS BOOL
   Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_OFFLINE) _
                    EqualTo FILE_ATTRIBUTE_OFFLINE),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsReparsePointFile
'------------------------------------------------------------------------------
' Method IsReparsePointFile
' Call this method to determine if the found file is file or directory that has an
' associated reparse point, or a file that is a symbolic link.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsReparsePointFile () AS BOOL
   Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_REPARSE_POINT) _
                     EqualTo FILE_ATTRIBUTE_REPARSE_POINT),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::IsSparseFile
'------------------------------------------------------------------------------
' Method IsSparseFile
' Call this method to determine if the found file is a sparse file.
' Return value:
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION IsSparseFile () AS BOOL
   Function = iif (((m_FileInfo.dwFileAttributes BAND FILE_ATTRIBUTE_SPARSE_FILE) _
                     EqualTo FILE_ATTRIBUTE_SPARSE_FILE),TRUE,FALSE)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::MatchesMask
'------------------------------------------------------------------------------
' Method MatchesMask
' Call this method to test the file attributes on the found file.
' Parameter:
' dwMask
'   Specifies one or more file attributes, identified in the WIN32_FIND_DATA structure,
'   for the found file. To search for multiple attributes, use the bitwise OR operator.
'   Any combination of the following attributes is acceptable:
'   FILE_ATTRIBUTE_ARCHIVE: The file is an archive file. Applications use this attribute
'     to mark files for backup or removal.
'   FILE_ATTRIBUTE_COMPRESSED: The file or directory is compressed. For a file, this
'     means that all of the data in the file is compressed. For a directory, this means
'   that compression is the default for newly created files and subdirectories.
'   FILE_ATTRIBUTE_DIRECTORY: The file is a directory.
'   FILE_ATTRIBUTE_NORMAL: The file has no other attributes set. This attribute is valid
'     only if used alone. All other file attributes override this attribute.
'   FILE_ATTRIBUTE_HIDDEN: The file is hidden. It is not to be included in an ordinary
'     directory listing.
'   FILE_ATTRIBUTE_READONLY: The file is read only. Applications can read the file but
'     cannot write to it or delete it.
'   FILE_ATTRIBUTE_SYSTEM: The file is part of or is used exclusively by the operating system.
'   FILE_ATTRIBUTE_TEMPORARY: The file is being used for temporary storage. Applications
'     should write to the file only if absolutely necessary. Most of the file's data remains
'     in memory without being flushed to the media because the file will soon be deleted.
' Return value
'   TRUE if successful; otherwise FALSE.
'------------------------------------------------------------------------------
FUNCTION MatchesMask (BYVAL dwMask AS DWORD) AS BOOL
	Function =  (m_FileInfo.dwFileAttributes BAND dwMask)
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::FileName
'------------------------------------------------------------------------------
' Method FileName
' Call this method to get the title of the found file.
' Return value
'   The name of the most-recently-found file, excluding the extension. For example,
'   calling GetFileNameX to generate a user message about the file c:\myhtml\myfile.txt
'   returns the file name myfile.
'------------------------------------------------------------------------------
FUNCTION FileName () AS string
	DIM p AS LONG
	DIM wszFileName$ * MAX_PATH
	wszFileName$  = m_FileInfo.cFileName
	'p = INSTRREV(wszFileName, ANY ":/\")
	p = InstrAnyRev(wszFileName$,":/\")
	IF p THEN 
		wszFileName$ = MID$(wszFileName, p + 1)
	EndIf
	'p = INSTRREV(wszFileName, ".")
	p = InStrRev(wszFileName, ".")
	'IF p THEN wszFileName = LEFT(wszFileName, p - 1)
	If p Then
		wszFileName$ = LEFT$(wszFileName, p - 1)
	EndIf
	Function = wszFileName
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::FileNameX
'------------------------------------------------------------------------------
' Method FileNameX
' Call this method to get the name of the found file, includig the extension.
' Return value
'   The name of the most-recently-found file, including the extension. For example,
'   calling GetFileName to generate a user message about the file c:\myhtml\myfile.txt
'   returns the file name myfile.txt.
'------------------------------------------------------------------------------
FUNCTION FileNameX () AS string
   Function =  m_FileInfo.cFileName$
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::ShortFilename
' Method ShortFilename
' Call this method to get an alternative name for the file.
' This name is in the classic 8.3 file name format.
' Return value
'   The alternate file name of the most-recently-found file, including the extension.
'------------------------------------------------------------------------------
FUNCTION ShortFileName () AS string
   Function =  m_FileInfo.cAlternateFileName
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::FileExt
'------------------------------------------------------------------------------
' Method FileExt
' Call this method to get the extesion of the found file.
'------------------------------------------------------------------------------
FUNCTION FileExt () AS string
	DIM p AS LONG
	p = INSTRREV(m_FileInfo.cFileName, ".")
	IF p THEN 
		Function = MID$(m_FileInfo.cFileName, p + 1)
	EndIf
	Function = ""
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
'==============================================================================
$IFUSE USE_CFindFile::FilePath
'------------------------------------------------------------------------------
' Method FilePath
' GetFilePath returns the entire path for the file. For example, calling GetFilePath to
' generate a user message about the file c:\myhtml\myfile.txt returns the file path
' c:\myhtml\myfile.txt.
'------------------------------------------------------------------------------
FUNCTION FilePath () AS string
	DIM AS LONG nLen
	Dim buffer$ * 4096
	'nLen = .GetFullPathNameW(m_FileInfo.cFileName, SIZEOF(buffer) \ 2, buffer, NULL)
	nLen = GetFullPathName(m_FileInfo.cFileName, 2048, buffer, NULL)
	If nLen Then
		Function = LEFT$(buffer, nLen)
	Else
		Function = ""
	EndIf
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::FileURL
'------------------------------------------------------------------------------
' Method FileURL
' Call this member function to retrieve the specified URL.
' GetFileURL is similar to the member function GetFilePath, except that it returns the
' URL in the form file://path. For example, calling GetFileURL to get the complete URL
' for myfile.txt returns the URL file://c:\myhtml\myfile.txt.
' ========================================================================================
FUNCTION FileURL () AS string
	Function =  "file://" & this->FilePath$()
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::Root
'------------------------------------------------------------------------------
' Method Root
' Call this member function to get the root of the found file.
' This method returns the drive specifier and path name used to start a search.
' For example, calling FindFile with *.dat results in GetRoot returning an empty string.
' Passing a path, such as c:\windows\system\*.dll, to FindFile results GetRoot
' returning c:\windows\system\.
' ========================================================================================
FUNCTION Root () AS string
	DIM wszPath$  * MAX_PATH
	wszPath$ = m_wszFileSpec$
	PathRemoveFileSpec(wszPath)
	IF Len(wszPath) Then
		wszPath$ + wszPath$ + "\"
	End If	
	Function = wszPath
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::FileSize
'------------------------------------------------------------------------------
' Method FileSize
' Call this method to get the size of the found file, in bytes.
' ========================================================================================
FUNCTION FileSize () AS ULONGLONG
	Function =  (m_FileInfo.nFileSizeHigh * (0xFFFFFFFF + 1)) + m_FileInfo.nFileSizeLow
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::CreationTime
'------------------------------------------------------------------------------
' Method CreationTime
' Call this method to get the time the file was created.
' ========================================================================================
FUNCTION CreationTime () AS FILETIME
	Function = m_FileInfo.ftCreationTime
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::LastAccessTime
'------------------------------------------------------------------------------
' Method LastAccessTime
' Call this method to get the time the file was last accessed.
' ========================================================================================
FUNCTION LastAccessTime () AS FILETIME
	Function = m_FileInfo.ftLastAccessTime
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CFindFile::LastWriteTime
'------------------------------------------------------------------------------
' Method LastWriteTime
' Call this method to get the time, in local time format, the file was written to,
' truncated, or overwritten.
' ========================================================================================
FUNCTION LastWriteTime () AS FILETIME
	Function = m_FileInfo.ftLastWriteTime
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE

CLASS_METHODS_END
