' ########################################################################################
' Microsoft Windows
' File: CGraphCtx.inc
' Contents: Graphic control
' Compiler: FreeBasic 32 & 64-bit
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
' // Notification messages
'NM_CLICK     = NM_FIRST - 2
'NM_DBLCLK    = NM_FIRST - 3
'NM_RCLICK    = NM_FIRST - 5
'NM_RDBLCLK   = NM_FIRST - 6
'NM_SETFOCUS  = NM_FIRST - 7
'NM_KILLFOCUS = NM_FIRST - 8

' // Process them in the main window callback as follows:
'
' CASE WM_NOTIFY
'    DIM phdr AS NMHDR PTR
'    phdr = CAST(NMHDR PTR, lParam)
'    IF wParam = IDC_GRCTX THEN
'       SELECT CASE phdr->code
'          CASE NM_CLICK
'             ' Left button clicked
'          CASE NM_RCLICK
'             ' Right button clicked
'          CASE NM_SETFOCUS
'             ' The control has gained focus
'          CASE NM_KILLFOCUS
'             ' The control has lost focus
'       END SELECT
'    END IF
'==============================================================================

Class CGraphCtx
	private:
		Dim m_hCtl        AS HWND          ' // Handle of the control
		Dim m_hFont       AS HFONT         ' // Handle to font used by the control
		Dim m_hMemDc      AS HDC           ' // Memory compatible device context handle
		Dim m_hBmp        AS HBITMAP       ' // Bitmap handle
		Dim m_hOldBmp     AS HBITMAP       ' // Old Bitmap handle
		Dim m_vWidth      AS DWORD         ' // Width of the virtual buffer
		Dim m_vHeight     AS DWORD         ' // Height of the virtual buffer
		Dim m_bkColor     AS COLORREF      ' // Background color
		Dim m_MaxX        AS LONG          ' // Maximum horizontal range
		Dim m_MaxY        AS LONG          ' // Maximum vertical range
		Dim m_OrgX        AS LONG          ' // x-origin for current display
		Dim m_OrgY        AS LONG          ' // y-origin for current display
		Dim m_coord       AS RECT          ' // Client area
		Dim m_ppvBits     AS VOID PTR PTR   ' // Location of the DIB bit values
		Dim m_BitsPerPel  AS LONG          ' // Bits per pixel
		Dim m_Stretchable AS BOOL       ' // Contents are stretchable
		Dim m_StretchMode AS LONG          ' // Stretch mode
		Dim m_Resizable   AS BOOL       ' // Contents are resizable
	public:
		Dim Constructor CGraphCtx(pWindow As CWindow Ptr,cID As Integer,wszTitle As char ptr = "", _
		                          x As Long = 0,y As Long = 0,nWidth As Long = 0,nHeight As Long = 0, _
		                          dwStyle As DWORD = 0, dwExStyle As DWORD = 0, lpParam As LONG_PTR = 0)  
		Dim Destructor ~CGraphCtx()
		Dim Function hWindow() As HWND
		Dim Function SetVirtualBufferSize (nWidth AS LONG, nHeight AS LONG) AS LONG
		Dim Function GetVirtualBufferWidth () AS LONG
		Dim Function GetVirtualBufferHeight () AS LONG
		Dim Function GetMemDC () AS HDC
		Dim Function GethBmp () AS HBITMAP
		Dim Function GetBits () As void Ptr
		Dim Function Clear(RGBColor AS COLORREF) AS BOOL
		Dim Function Stretchable () AS BOOL
		Dim Sub Stretchable (bStretchable AS BOOL)
		Dim Function StretchMode () AS LONG
		Dim Sub StretchMode (nStretchMode AS LONG)
		Dim Function Resizable () AS BOOL
		Dim Sub Resizable (bResizable AS BOOL)
		Dim Sub LoadImageFromFile (wszFileName AS const char ptr)
		Dim Sub CreateBitmapFromFile (wszFileName AS const char ptr, dimPercent AS LONG = 0, bGrayScale AS LONG = FALSE)
		Dim Function SaveImage (wszFileName AS const char ptr, wszMimeType AS const char ptr = "image/bmp") AS LONG
		Dim Function SaveImageToBmp (wszFileName AS const char ptr) AS LONG
		Dim Function SaveImageToJpeg (wszFileName AS const char ptr) AS LONG
		Dim Function SaveImageToPng (wszFileName AS const char ptr) AS LONG
		Dim Function SaveImageToGif (wszFileName AS const char ptr) AS LONG
		Dim Function SaveImageToTiff (wszFileName AS const char ptr) AS LONG 
		'Dim Function PrintImage (bStretch AS BOOL = FALSE,  nStretchMode AS LONG = InterpolationModeHighQualityBicubic) AS LONG
		Dim Static Function CGraphCtxProc (hwnd AS HWND, uMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
End Class
CLASS_METHODS_START CGraphCtx
Constructor CGraphCtx(pWindow As CWindow Ptr,cID As Integer,wszTitle As char ptr , _
		                          x As Long,y As Long ,nWidth As Long,nHeight As Long, _
		                          dwStyle As DWORD, dwExStyle As DWORD, lpParam As LONG_PTR)
	Dim As ATOM wAtom
	Dim As WNDCLASSEXW wcexw
	Dim wszClassName$
	wszClassName$ = "AFX_GRAPHCTX"
	If GetClassInfoEx(GetModuleHandle(NULL), wszClassName, &wcexw) = 0 THEN
		With wcexw
			.cbSize			=	SizeOf(wcexw)
			.style			= 	CS_DBLCLKS BOR CS_HREDRAW BOR CS_VREDRAW
			.lpfnWndProc   = 	&CGraphCtxProc
			.cbClsExtra    = 	0
			.cbWndExtra    = 	SIZEOF(HANDLE)   ' // make room to store a pointer to the class
			.hInstance     = 	GetModuleHandle(NULL)
			.hCursor       = 	LoadCursor(NULL, (LPCWSTR) IDC_ARROW)
			.hbrBackground = 	(HBRUSH) WHITE_BRUSH
			.lpszMenuName  = 	NULL
			.lpszClassName = 	wszClassName
			.hIcon         = 	NULL
			.hIconSm       = 	NULL
		End With
		wAtom = RegisterClassEx(&wcexw)
		'If wAtom Then
		'	zTrace("wAtom = RegisterClassEx(&wcexw)")
		'EndIf
	EndIf
	'Create the control
	IF dwStyle = 0 Then
		dwStyle = WS_VISIBLE BOR WS_CHILD BOR WS_TABSTOP
	EndIf
	IF pWindow Then
		m_hCtl = pWindow->AddControl(wszClassName, pWindow->hWindow(), cID, wszTitle, x, y, nWidth, nHeight, dwStyle, dwExStyle, lpParam)
	EndIf
	If m_hCtl Then
		SetWindowLongPtr(m_hCtl, 0, (LONG_PTR)this)
		'Set the same font used by the parent
		DIM AS LOGFONTW lfw
		If pWindow->Font() Then
			If GetObject(pWindow->Font(), SIZEOF(lfw), &lfw) Then
				m_hFont = CreateFontIndirect(&lfw)
			EndIf
		END IF
		' Creates a compatible memory device context and bitmap
		GetClientRect( m_hCtl, &m_coord)
		Dim As HDC hDC = GetDC(m_hCtl)
		m_BitsPerPel = GetDeviceCaps(hDC, BITSPIXEL)
		m_hMemDc = CreateCompatibleDC(hDC)
		Dim As BITMAPINFO bi
		bi.bmiHeader.biSize = SIZEOF(bi.bmiHeader)
		bi.bmiHeader.biWidth = m_coord.right - m_coord.left
		bi.bmiHeader.biHeight = m_coord.bottom - m_coord.top
		bi.bmiHeader.biPlanes = 1
		bi.bmiHeader.biBitCount = m_BitsPerPel
		bi.bmiHeader.biCompression = BI_RGB
		m_hBmp = CreateDIBSection(hDC, &bi, DIB_RGB_COLORS, m_ppvBits, NULL, 0)
		m_hOldBmp = (HBITMAP)SelectObject(m_hMemDc, m_hBmp)
		FillRect( m_hMemDc, &m_coord, (HBRUSH) GetClassLongPtr(GetParent(m_hCtl), GCLP_HBRBACKGROUND))
		ReleaseDC( m_hCtl, hDC)
		' Set the virtual size equal to the size of the bitmap
		m_vWidth = m_coord.right - m_coord.left
		m_vHeight = m_coord.bottom - m_coord.top
		' Default stretch mode
		m_StretchMode = HALFTONE
		' Initializes scroll bar ranges
		m_MaxX = m_coord.right - m_coord.left
		m_MaxY = m_coord.bottom - m_coord.top
		Dim As SCROLLINFO si
		si.cbSize = SIZEOF(si)
		si.fMask = SIF_RANGE
		si.nMin = 0
		si.nMax = m_MaxX - m_coord.right - m_coord.left
		SetScrollInfo( m_hCtl, SB_HORZ, &si, 1)
		si.nMax = m_MaxY - m_coord.bottom - m_coord.top
		SetScrollInfo( m_hCtl, SB_VERT, &si, 1)
		' Invalidates the window to force redrawing
		InvalidateRect( m_hCtl, NULL, 0)
	EndIf
	
End Constructor
'==============================================================================
Destructor CGraphCtx()
	' // Restore the original bitmap
	SelectObject(m_hMemDc, m_hOldBmp)
	' // Destroys the bitmap and the memory device context
	If m_hBmp Then
		DeleteObject( m_hBmp)
	EndIf
	If m_hMemDc Then
		DeleteObject( m_hMemDc)
	EndIf
End Destructor
'==============================================================================

Function CGraphCtxProc (hwnd AS HWND, uMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
	Dim As CGraphCtx PTR pGraphCtx
	Dim As RECT rc,rcFill
	Dim As HDC hDC
	Dim As HBRUSH hBrush
	Dim As SCROLLINFO si
	Dim As PAINTSTRUCT ps
	Dim As NMHDR hdr
	Dim As Long nPage
	
	Select Case uMsg
		Case WM_CREATE
			Exit Function
		Case WM_COMMAND, WM_NOTIFY
			SendMessage(GetParent(hwnd),uMsg,wParam,lParam)
		Case WM_GETDLGCODE
			' Ensures that the control will process all the keystrokes by itself
			Function = DLGC_WANTALLKEYS
		Case WM_LBUTTONDOWN
			' Forwards the message to the parent window
			hdr.hwndFrom = hwnd
			hdr.idFrom = GetDlgCtrlID(hwnd)
			hdr.code = NM_CLICK
			SendMessage( GetParent(hwnd), WM_NOTIFY, GetDlgCtrlID(hwnd), (LPARAM) &hdr)
			Exit Function
		Case WM_RBUTTONDOWN
			' Forwards the message to the parent window
			hdr.hwndFrom = hwnd
			hdr.idFrom = GetDlgCtrlID(hwnd)
			hdr.code = NM_RCLICK
			SendMessage( GetParent(hwnd), WM_NOTIFY, GetDlgCtrlID(hwnd), (LPARAM) &hdr)
			Exit Function
		Case WM_LBUTTONDBLCLK
			' Forwards the message to the parent window
			hdr.hwndFrom = hwnd
			hdr.idFrom = GetDlgCtrlID(hwnd)
			hdr.code = NM_DBLCLK
			SendMessage( GetParent(hwnd), WM_NOTIFY, GetDlgCtrlID(hwnd), (LPARAM) &hdr)
			Exit Function
		Case WM_RBUTTONDBLCLK
			' Forwards the message to the parent window
			hdr.hwndFrom = hwnd
			hdr.idFrom = GetDlgCtrlID(hwnd)
			hdr.code = NM_RDBLCLK
			SendMessage( GetParent(hwnd), WM_NOTIFY, GetDlgCtrlID(hwnd), (LPARAM) &hdr)
			Exit Function
		Case WM_SETFOCUS
			' Forwards the message to the parent window
			hdr.hwndFrom = hwnd
			hdr.idFrom = GetDlgCtrlID(hwnd)
			hdr.code = NM_SETFOCUS
			SendMessage( GetParent(hwnd), WM_NOTIFY, GetDlgCtrlID(hwnd), (LPARAM) &hdr)
			Exit Function
		Case WM_KILLFOCUS	
			' Forwards the message to the parent window
			hdr.hwndFrom = hwnd
			hdr.idFrom = GetDlgCtrlID(hwnd)
			hdr.code = NM_KILLFOCUS
			SendMessage( GetParent(hwnd), WM_NOTIFY, GetDlgCtrlID(hwnd), (LPARAM) &hdr)
			Exit Function
		Case WM_KEYDOWN
			' Keyboard navigation
			Select Case LOWORD(wParam)
				Case VK_TAB
					If HIWORD(GetKeyState(VK_SHIFT)) = 0 Then
						SetFocus( GetNextDlgTabItem(GetParent(hwnd), hwnd, 0))
					Else
						SetFocus(GetNextDlgTabItem(GetParent(hwnd), hwnd, -1))
					End If
				Case VK_RETURN
					SendMessage(GetParent(hwnd), WM_COMMAND, MAKELONG(IDOK, BN_CLICKED), (LPARAM) hwnd)
				Case VK_ESCAPE
					SendMessage(GetParent(hwnd), WM_COMMAND, MAKELONG(IDCANCEL, BN_CLICKED), (LPARAM) hwnd)
				Case VK_RIGHT
					If GetAsyncKeyState(VK_CONTROL) Then
						SendMessage( hwnd, WM_HSCROLL, SB_PAGERIGHT, 0)
					Else
						SendMessage( hwnd, WM_HSCROLL, SB_LINERIGHT, 0)
					End If
				Case VK_LEFT
					If GetAsyncKeyState(VK_CONTROL) Then
						SendMessage( hwnd, WM_HSCROLL, SB_PAGELEFT, 0)
					Else
						SendMessage( hwnd, WM_HSCROLL, SB_LINELEFT, 0)
					End If
				Case VK_DOWN
					SendMessage( hwnd, WM_VSCROLL, SB_LINEDOWN, 0)
				Case VK_UP
					SendMessage(hwnd, WM_VSCROLL, SB_LINEUP, 0)
				Case VK_NEXT ' = VK_PGDN
					SendMessage(hwnd, WM_VSCROLL, SB_PAGEDOWN, 0)
				Case VK_PRIOR ' = VK_PGUP
					SendMessage(hwnd, WM_VSCROLL, SB_PAGEUP, 0)
				Case VK_HOME
					SendMessage(hwnd, WM_VSCROLL, SB_TOP, 0)
				Case VK_END
					SendMessage(hwnd, WM_VSCROLL, SB_BOTTOM, 0)
			End Select
			EXIT Function
		Case WM_MOUSEWHEEL
			Dim As SHORT zDelta = HIWORD(wParam)
			If (LOWORD(wParam) BAND MK_CONTROL) = MK_CONTROL Then 	' Horizontal scroll
				If zDelta > 0 Then									' Scroll to the left
					SendMessage(hwnd, WM_HSCROLL, SB_LINELEFT, 0)
				Else												' Scroll to the right
					SendMessage(hwnd, WM_HSCROLL, SB_LINERIGHT, 0)
				EndIf
			Else													' Vertical scroll
				If zDelta > 0 Then									'Scroll Up
					SendMessage(hwnd, WM_VSCROLL, SB_LINEUP, 0)
				Else												'Scroll Down	
					SendMessage(hwnd, WM_VSCROLL, SB_LINEDOWN, 0)
				EndIf
			EndIf
			Exit Function
		Case WM_HSCROLL
			' Get a pointer to the class
			pGraphCtx = (CGraphCtx PTR) GetWindowLongPtr(hwnd, 0)
			If pGraphCtx = NULL Then
				Exit Function
			EndIf
			' Horizontal scrolling
			Select Case LOWORD(wParam)
				Case SB_THUMBTRACK
					pGraphCtx->m_OrgX = HIWORD(wParam)
				Case SB_LINERIGHT
					If pGraphCtx->m_OrgX < pGraphCtx->m_MaxX - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left Then
						pGraphCtx->m_OrgX = pGraphCtx->m_OrgX + 10
						If pGraphCtx->m_OrgX > pGraphCtx->m_MaxX - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left Then
							pGraphCtx->m_OrgX = pGraphCtx->m_MaxX - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left
						End If
					End If
				Case SB_LINELEFT
					If pGraphCtx->m_OrgX > 0 Then
						pGraphCtx->m_OrgX = pGraphCtx->m_OrgX - 10
						If pGraphCtx->m_OrgX < 0 Then
							pGraphCtx->m_OrgX = 0
						EndIf
					End If
				Case SB_PAGERIGHT
					nPage = pGraphCtx->m_coord.right - pGraphCtx->m_coord.left
					If pGraphCtx->m_OrgX + nPage < pGraphCtx->m_MaxX - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left Then
						pGraphCtx->m_OrgX = pGraphCtx->m_OrgX + nPage
					Else
						pGraphCtx->m_OrgX = pGraphCtx->m_MaxX - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left
					End If
				Case SB_PAGELEFT
					nPage = pGraphCtx->m_coord.right - pGraphCtx->m_coord.left
					If pGraphCtx->m_OrgX - nPage > 0 Then
						pGraphCtx->m_OrgX = pGraphCtx->m_OrgX - nPage
					Else
						pGraphCtx->m_OrgX = 0
					End If 	
			End Select
			si.fMask = SIF_POS
			si.nPos = pGraphCtx->m_OrgX
			SetScrollInfo( hwnd, SB_HORZ, &si, 1)
			' Invalidates the window to force redrawing
			InvalidateRect( hwnd, NULL, 0)
			Exit Function
		Case WM_VSCROLL
			' Get a pointer to the class
			pGraphCtx = (CGraphCtx PTR) GetWindowLongPtr(hwnd, 0)
			If pGraphCtx = NULL Then
				Exit Function
			EndIf
			' Vertical scrolling
			Select Case LOWORD(wParam)
				Case SB_THUMBTRACK
					pGraphCtx->m_OrgY = HIWORD(wParam)
				Case SB_LINEDOWN
					If pGraphCtx->m_OrgY < pGraphCtx->m_MaxY - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top Then
						pGraphCtx->m_OrgY = pGraphCtx->m_OrgY + 10
						If pGraphCtx->m_OrgY > pGraphCtx->m_MaxY - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top Then
							pGraphCtx->m_OrgY = pGraphCtx->m_MaxY - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top
						End If
					End If
				Case SB_LINEUP
					If pGraphCtx->m_OrgY > 0 Then
						pGraphCtx->m_OrgY = pGraphCtx->m_OrgY - 10
						If pGraphCtx->m_OrgY < 0 Then
							pGraphCtx->m_OrgY = 0
						EndIf
					End If	
				Case SB_PAGEDOWN
					nPage = pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top
					If pGraphCtx->m_OrgY + nPage < pGraphCtx->m_MaxY - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top Then
						pGraphCtx->m_OrgY = pGraphCtx->m_OrgY + nPage
					Else
						pGraphCtx->m_OrgY = pGraphCtx->m_MaxY - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top
					End if 
				Case SB_PAGEUP
					nPage = pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top
					If pGraphCtx->m_OrgY - nPage > 0 Then
						pGraphCtx->m_OrgY = pGraphCtx->m_OrgY - nPage
					Else
						pGraphCtx->m_OrgY = 0
					End if
				Case SB_TOP
					pGraphCtx->m_OrgX = 0
					pGraphCtx->m_OrgY = 0
					si.fMask = SIF_POS
					si.nPos = pGraphCtx->m_OrgX
					SetScrollInfo( hwnd, SB_HORZ, &si, 1)
				Case SB_BOTTOM
					GetClientRect( hwnd, &rc)
					pGraphCtx->m_OrgX = pGraphCtx->m_MaxX - rc.right
					pGraphCtx->m_OrgY = pGraphCtx->m_MaxY - rc.bottom
					si.fMask = SIF_POS
					si.nPos = pGraphCtx->m_OrgX
					SetScrollInfo( hwnd, SB_HORZ, &si, 1)		
			End Select
			si.fMask = SIF_POS
			si.nPos = pGraphCtx->m_OrgY
			SetScrollInfo( hwnd, SB_VERT, &si, 1)
			' // Invalidates the window to force redrawing
			InvalidateRect( hwnd, NULL, 0)
			Exit Function
		Case WM_ENABLE
			' Redraws the control
			InvalidateRect( hwnd, NULL, 0)
			UpdateWindow( hwnd)
			Exit Function
		Case WM_ERASEBKGND
			' Get a pointer to the class
			pGraphCtx = (CGraphCtx PTR) GetWindowLongPtr(hwnd, 0)
			If pGraphCtx = NULL Then
				Exit Function
			EndIf
			If pGraphCtx->m_Stretchable = FALSE Then
				'Erases only the portion of the window not covered
				'by the graphic or image to avoid flicker
				GetClientRect( hwnd, &rc)
	            If rc.right - rc.left > pGraphCtx->m_vWidth OR rc.bottom - rc.top > pGraphCtx->m_vHeight Then
	            	hDC = (HDC) wParam
	            	hBrush = CreateSolidBrush(pGraphCtx->m_bkColor)
	            	If hBrush Then
						rcFill = rc
						rcFill.left = pGraphCtx->m_vWidth - pGraphCtx->m_OrgX
						FillRect( hDC, &rcFill, hBrush)
						rcFill = rc
						rcFill.top = pGraphCtx->m_vHeight - pGraphCtx->m_OrgY
						FillRect( hDC, &rcFill, hBrush)
						DeleteObject( hBrush)
	            		
	            	EndIf
	            EndIf
			EndIf
			Function = 1
		Case WM_SIZE
			' Get a pointer to the class
			pGraphCtx = (CGraphCtx PTR) GetWindowLongPtr(hwnd, 0)
			If pGraphCtx = NULL Then
				Exit Function
			EndIf
			Dim As Long Increment = LOWORD(lParam) - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left
			If Increment < 0 Then
				Increment = 0
			EndIf
			If (Increment > 0) AND (pGraphCtx->m_OrgX >= (LOWORD(lParam) - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left)) Then
				pGraphCtx->m_OrgX = pGraphCtx->m_OrgX - Increment
			EndIf
			If pGraphCtx->m_OrgX < 0 Then
				pGraphCtx->m_OrgX = 0
			EndIf
			Increment = HIWORD(lParam) - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top
			If Increment < 0 Then
				Increment = 0
			EndIf
			If(Increment > 0) AND (pGraphCtx->m_OrgY >= (HIWORD(lParam) - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top)) Then
				pGraphCtx->m_OrgY = pGraphCtx->m_OrgY - Increment
			EndIf
			If pGraphCtx->m_OrgY < 0 Then
				pGraphCtx->m_OrgY = 0
			EndIf
			' Stores new window extents
			pGraphCtx->m_coord.right  = LOWORD(lParam)
			pGraphCtx->m_coord.bottom = HIWORD(lParam)
			' If resizable or stretchable, disable scroll bars
        	IF pGraphCtx->m_Resizable = TRUE OR pGraphCtx->m_Stretchable = TRUE THEN
				' // Initializes scroll bar ranges
				GetClientRect( hwnd, &rc)
				pGraphCtx->m_MaxX = rc.right - rc.left
				pGraphCtx->m_MaxY = rc.bottom - rc.top
				si.cbSize = SIZEOF(si)
				si.fMask = SIF_RANGE
				si.nMin = 0
				si.nMax = pGraphCtx->m_MaxX - rc.right - rc.left
				SetScrollInfo( hwnd, SB_HORZ, &si, 1)
				si.nMax = pGraphCtx->m_MaxY - rc.bottom - rc.top
				SetScrollInfo( hwnd, SB_VERT, &si, 1)
        	ELSE
				' Reinitializes scroll bar ranges
				si.cbSize = SIZEOF(si)
				si.fMask = SIF_RANGE OR SIF_POS
				si.nMin = 0
				si.nMax = pGraphCtx->m_MaxX - pGraphCtx->m_coord.right - pGraphCtx->m_coord.left
				si.nPos = pGraphCtx->m_OrgX
				SetScrollInfo( hwnd, SB_HORZ, &si, 1)
				si.nMax = pGraphCtx->m_MaxY - pGraphCtx->m_coord.bottom - pGraphCtx->m_coord.top
				si.nPos = pGraphCtx->m_OrgY
				SetScrollInfo( hwnd, SB_VERT, &si, 1)
			END IF
			' // Invalidates the window to force redrawing
			InvalidateRect( hwnd, NULL, 0)
			Exit Function
		Case WM_PRINTCLIENT
			' Get a pointer to the class
			pGraphCtx = (CGraphCtx PTR) GetWindowLongPtr(hwnd, 0)
			If pGraphCtx = NULL Then
				Exit Function
			EndIf
			' // Copies the bitmap to the provided device context
			hDC = (HDC) wParam
			GetClientRect(hwnd, &rc)
			If pGraphCtx->m_Stretchable = FALSE Then
				BitBlt(hDC, 0, 0, GetDeviceCaps(hDC, HORZRES), GetDeviceCaps(hDC, VERTRES), _
			           pGraphCtx->m_hMemDc, rc.left + pGraphCtx->m_OrgX, rc.top + pGraphCtx->m_OrgY, SRCCOPY)
			Else
				SetStretchBltMode( hDC, pGraphCtx->m_StretchMode)
				If pGraphCtx->m_StretchMode = HALFTONE THEN
					SetBrushOrgEx( hDC, 0, 0, NULL)
				EndIf
				StretchBlt(hDC, 0, 0, GetDeviceCaps(hDC, HORZRES), GetDeviceCaps(hDC, VERTRES), _
			               pGraphCtx->m_hMemDc, 0, 0, pGraphCtx->m_vWidth, pGraphCtx->m_vHeight, SRCCOPY)
			End If
			Exit Function
		Case WM_PAINT
			' Get a pointer to the class
			pGraphCtx = (CGraphCtx PTR) GetWindowLongPtr(hwnd, 0)
			If pGraphCtx = NULL Then
				Exit Function
			EndIf
			'  Copies the bitmap to the control's window
			hDC = BeginPaint(hwnd, &ps)
			If pGraphCtx->m_Stretchable = FALSE Then
				BitBlt(hDC, ps.rcPaint.left, ps.rcPaint.top, _
			           ps.rcPaint.right - ps.rcPaint.left, _
			           ps.rcPaint.bottom - ps.rcPaint.top, _
			           pGraphCtx->m_hMemDc, ps.rcPaint.left + pGraphCtx->m_OrgX, ps.rcPaint.top + pGraphCtx->m_OrgY, SRCCOPY)
			Else
				GetClientRect(hwnd, &rc)
				SetStretchBltMode( hDC, pGraphCtx->m_StretchMode)
				If pGraphCtx->m_StretchMode = HALFTONE Then
					SetBrushOrgEx( hDC, 0, 0, NULL)
				EndIf
				StretchBlt(hDC, ps.rcPaint.left, ps.rcPaint.top, _
			               ps.rcPaint.right - ps.rcPaint.left, _
			               ps.rcPaint.bottom - ps.rcPaint.top, _
			               pGraphCtx->m_hMemDc, 0, 0,pGraphCtx->m_vWidth, pGraphCtx->m_vHeight, SRCCOPY)
			End If
			EndPaint( hwnd, &ps)
			Exit Function
	End Select
	' // Default processing for other messages.
	Function = DefWindowProc(hwnd, uMsg, wParam, lParam)
End Function
'==============================================================================
'==============================================================================
$IFUSE USE_CGraphCtx::hWindow
'------------------------------------------------------------------------------
' Returns the handle of the control
' ========================================================================================
FUNCTION hWindow () AS HWND
	Function = m_hCtl
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::SetVirtualBufferSize
'------------------------------------------------------------------------------
' Sets the size of the virtual buffer.
' Parameters:
' * nWidth = Width, in pixels, of the virtual buffer.
' * nHeight = Height, in pixels, of the virtual buffer.
' Return value:
' * If the function succeeds, the return value is S_OK.
' ========================================================================================
Function SetVirtualBufferSize ( nWidth AS LONG,  nHeight AS LONG) AS LONG
	' // Clone the memory bitmap to restore it later
	DIM AS BITMAPINFO bi
	DIM AS RECT rcClone
	SetRect(&rcClone, 0, 0, nWidth, nHeight)
	DIM AS HDC hDC = GetDC(m_hCtl)
	DIM AS HDC hCloneDC = CreateCompatibleDC(hDC)
	'   DIM hCloneBmp AS HBITMAP = .CreateBitmap(m_vWidth, m_vHeight, 1, m_BitsPerPel, NULL)
	' // --- Use CreateDIBSection instead of CreateBitmap --- //
	bi.bmiHeader.biSize = SIZEOF(bi.bmiHeader)
	bi.bmiHeader.biWidth = m_vHeight
	bi.bmiHeader.biHeight = nHeight
	bi.bmiHeader.biPlanes = 1
	bi.bmiHeader.biBitCount = m_BitsPerPel
	bi.bmiHeader.biCompression = BI_RGB
	DIM AS HBITMAP hCloneBmp = CreateDIBSection(hDC, &bi, DIB_RGB_COLORS, NULL, NULL, 0)
	If hCloneBmp Then
		SelectObject(hCloneDC, hCloneBmp)
		BitBlt(hCloneDC, 0, 0, m_vWidth, m_vHeight, m_hMemDc, 0, 0, SRCCOPY)
	End If
	ReleaseDC( m_hCtl, hDC)
	' Restore the original bitmap
	SelectObject(m_hMemDc, m_hOldBmp)
	' // Destroys the bitmap and the memory device context
	If m_hBmp Then
		DeleteObject( m_hBmp)
	EndIf
	If m_hMemDc Then
		DeleteObject( m_hMemDc)
	EndIf
	m_vWidth = nWidth
	m_vHeight = nHeight

	' // Creates a virtual compatible memory device context and bitmap.
	hDC = GetDC(m_hCtl)
	m_hMemDc = CreateCompatibleDC(hDC)
	bi.bmiHeader.biSize = SIZEOF(bi.bmiHeader)
	bi.bmiHeader.biWidth = nWidth
	bi.bmiHeader.biHeight = nHeight
	bi.bmiHeader.biPlanes = 1
	bi.bmiHeader.biBitCount = m_BitsPerPel
	bi.bmiHeader.biCompression = BI_RGB
	m_hBmp = CreateDIBSection(hDC, &bi, DIB_RGB_COLORS, m_ppvBits, NULL, 0)
	m_hOldBmp = (HBITMAP)SelectObject(m_hMemDc, m_hBmp)
	
	' Adjusts the coordinates
	m_coord.right = m_coord.left + nWidth
	m_coord.bottom = m_coord.top + nHeight
	'  Erases the background with the specified or default color
	If m_bkColor Then
		DIM AS HBRUSH hBrush = CreateSolidBrush(m_bkColor)
		If hBrush Then
			FillRect( m_hMemDc, &m_coord, hBrush)
			DeleteObject( hBrush)
		End If
	Else
		FillRect( m_hMemDc, &m_coord, (HBRUSH) GetClassLongPtr(GetParent(m_hCtl), GCLP_HBRBACKGROUND))
	End If
	ReleaseDC( m_hCtl, hDC)
	
	If m_Resizable = FALSE AND m_Stretchable = FALSE Then
		' Adds scrollbars if needed
		DIM AS RECT rcParent
		Dim AS DWORD  dwStyle
		GetClientRect( GetParent(m_hCtl), &rcParent)
		If nWidth > rcParent.right OR nHeight > rcParent.bottom Then
			dwStyle = GetWindowLongPtr(m_hCtl, GWL_STYLE)
			dwStyle = dwStyle BOR WS_HSCROLL BOR WS_VSCROLL
			SetWindowLongPtr(m_hCtl, GWL_STYLE, dwStyle)
		End If
		'  Initializes scroll bar ranges
		m_MaxX = m_coord.right - m_coord.left
		m_MaxY = m_coord.bottom - m_coord.top
		DIM AS SCROLLINFO si
		si.cbSize = SIZEOF(si)
		si.fMask = SIF_RANGE
		si.nMin = 0
		si.nMax = m_MaxX - m_coord.right - m_coord.left
		SetScrollInfo( m_hCtl, SB_HORZ, &si, 1)
		si.nMax = m_MaxY - m_coord.bottom - m_coord.top
		SetScrollInfo( m_hCtl, SB_VERT, &si, 1)
		' If a scrollbar isn't needed, hide it
		DIM AS RECT rc
		GetClientRect( m_hCtl, &rc)
		If nWidth <= rc.right Then
			ShowScrollBar( m_hCtl, SB_HORZ, FALSE)
		EndIf
		If nHeight <= rc.bottom Then
			ShowScrollBar( m_hCtl, SB_VERT, FALSE)
		EndIf
		' Invalidates the window to force redrawing
		InvalidateRect( m_hCtl, NULL, 0)
	END IF

	' // Copy the cloned bitmap
	If hCloneBmp Then
		BitBlt(m_hMemDc, 0, 0, m_vWidth, m_vHeight, hCloneDC, 0, 0, SRCCOPY)
		SelectObject(hCloneDC, 0)
		DeleteDC( hCloneDC)
		DeleteObject( hCloneBmp)
		InvalidateRect( m_hCtl, NULL, 0)
	END IF
	
	Function = S_OK
End Function
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::GetMemDC
'------------------------------------------------------------------------------
' Returns the handle of the memory device context of the control.
' ========================================================================================
FUNCTION GetMemDC () AS HDC
   FUNCTION = m_hMemDc
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::GethBmp
'------------------------------------------------------------------------------
' Returns the handle of the compatible bitmap.
' ========================================================================================
FUNCTION GethBmp () AS HBITMAP
	FUNCTION = m_hBmp
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::GetBits
'------------------------------------------------------------------------------
' Returns the location of the DIB bit values.
' ========================================================================================
FUNCTION GetBits () AS void PTR
	FUNCTION = m_ppvBits
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::Clear
'------------------------------------------------------------------------------
' Clears the graphic control with the specified RGB color.
' Parameter:
' * RGBColor = RGB color used to fill the control.
' Return value:
' * If the function succeeds, the return value is TRUE.
'   If the function fails, the return value is FALSE.
' ========================================================================================
FUNCTION Clear (RGBColor AS COLORREF) AS BOOL
	Local As BOOL bRet
	If m_hMemDc = NULL Then
		Exit Function
	EndIf
	DIM AS HBRUSH hBrush = CreateSolidBrush(RGBColor)
	DIM AS RECT rc
	GetClientRect( m_hCtl, &rc)
	If m_vWidth Then
		rc.right = rc.left + m_vWidth
	EndIf
	IF m_vHeight Then
		rc.bottom = rc.top + m_vHeight
	EndIf
	bRet =FillRect(m_hMemDc, &rc, hBrush)
	DeleteObject( hBrush)
	m_bkColor = RGBColor
	Function = bRet
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::GetVirtualBufferWidth
'------------------------------------------------------------------------------
' Returns the width of the virtual buffer.
' ========================================================================================
FUNCTION GetVirtualBufferWidth () AS LONG
	FUNCTION = m_vWidth
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::GetVirtualBufferHeight
'------------------------------------------------------------------------------
' Returns the height of the virtual buffer.
' ========================================================================================
FUNCTION GetVirtualBufferHeight () AS LONG
	FUNCTION = m_vHeight
END FUNCTION
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::Stretchable
'------------------------------------------------------------------------------
' Returns the value of the stretchable property
' Return value:
' * TRUE or FALSE
' ========================================================================================
Function Stretchable () AS BOOL
   Function = m_Stretchable
END Function
'==============================================================================
' Sets the value of the stretchable property
' Parameters:
' * bResizable = TRUE or FALSE
' Return value:
' * The previous value of the property
' ========================================================================================
Sub Stretchable ( bStretchable AS BOOL)
   m_Stretchable = bStretchable
END Sub
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::StretchMode
'------------------------------------------------------------------------------
' Returns the value of the stretch mode property
' ========================================================================================
Function StretchMode () AS LONG
   Function = m_StretchMode
End Function
' ========================================================================================

' ========================================================================================
' Sets the value of the stretch mode property
' Parameter:
' * nStretchMode = The stretching mode.
' BLACKONWHITE
'    Performs a Boolean AND operation using the color values for the eliminated and existing
'    pixels. If the bitmap is a monochrome bitmap, this mode preserves black pixels at the
'    expense of white pixels.
' COLORONCOLOR
'   Deletes the pixels. This mode deletes all eliminated lines of pixels without trying to
'   preserve their information.
' HALFTONE
'   Maps pixels from the source rectangle into blocks of pixels in the destination rectangle.
'   The average color over the destination block of pixels approximates the color of the
'   source pixels.
'   After setting the HALFTONE stretching mode, an application must call the SetBrushOrgEx
'   function to set the brush origin. If it fails to do so, brush misalignment occurs.
' STRETCH_ANDSCANS
'   Same as BLACKONWHITE.
' STRETCH_DELETESCANS
'   Same as COLORONCOLOR.
' STRETCH_HALFTONE
'   Same as HALFTONE.
' STRETCH_ORSCANS
'   Same as WHITEONBLACK.
' WHITEONBLACK
'   Performs a Boolean OR operation using the color values for the eliminated and existing
'   pixels. If the bitmap is a monochrome bitmap, this mode preserves white pixels at the
'   expense of black pixels.
' Return value:
' * The previous value of the property
' ========================================================================================
Sub StretchMode (nStretchMode AS LONG)
   m_StretchMode = nStretchMode
END Sub
' ========================================================================================
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::Resizable
'------------------------------------------------------------------------------
' Returns the value of the resizable property
' Return value:
' * TRUE or FALSE
' ========================================================================================
Function Resizable () AS BOOL
   Function = m_Resizable
End Function
' ========================================================================================

' ========================================================================================
' Sets the value of the resizable property
' Parameter:
' * bResizable = TRUE or FALSE
' Return value:
' * The previous value of the property
' ========================================================================================
Sub Resizable ( bResizable AS BOOL)
   m_Resizable = bResizable
End Sub
'------------------------------------------------------------------------------
$ENDUSE
'==============================================================================
$IFUSE USE_CGraphCtx::LoadImageFromFile
'------------------------------------------------------------------------------
' Shows the specified image in the specified device context
' Parameters:
' - wszFileName = Path of the file.
' Remarks:
'   A quirk in the GDI+ GdipCreateBitmapFromFile function causes that black and white
'   images are loaded with increased contrast. Therefore, it's better to use the
'   CreateBitmapFromFile method.
' ========================================================================================
SUB LoadImageFromFile ( wszFileName AS  const char ptr)
	' // Initialize Gdiplus
	Raw As Gdiplus::GdiplusStartupInput gdiplusStartupInput
	Dim As ULONG_PTR token  
	GdiplusStartup(&token, &gdiplusStartupInput, NULL)
	
	' // Load the image from file and display it in the control
	Raw As Gdiplus::Bitmap Ptr pBitmap = new Gdiplus::Bitmap(wszFileName)
	Dim As DWORD nWidth,nHeight
	Raw As Gdiplus::Graphics  Ptr pGraphics 

   
	IF pBitmap THEN
		' // Get the width and height of the image
		nWidth = pBitmap->GetWidth()
		nHeight = pBitmap->GetHeight()
   
		' // Set the virtual buffer size of the control
		this->SetVirtualBufferSize(nWidth, nHeight)
		this->Clear( m_bkColor)
      	' // Create a graphics object from the memory DC of the control
		pGraphics = new Gdiplus::Graphics(m_hMemDc)
		' // Draw the image
		If pGraphics Then
			pGraphics->DrawImage(pBitmap,0,0,nWidth,nHeight)
		EndIf
      ' // Dispose the image and delete the graphics object
      	delete pBitmap
      	delete pGraphics
   '   GdipDisposeImage(CAST(GpImage PTR, pBitmap))
   '   IF pGraphics THEN GdipDeleteGraphics(pGraphics)
   END IF
   '' // Shutdown Gdiplus
   Gdiplus::GdiplusShutdown(token)
END SUB
'------------------------------------------------------------------------------
$ENDUSE

CLASS_METHODS_END 
