' ########################################################################################
' Microsoft Windows
' File: CLayout.inc
' Contents: Layout manager
' Purpose: When a parent window is resized, this class manages the location and size
' of the child windows
' Based on code originally written by Patrice Terrier and used with his permission.
' ########################################################################################
'==============================================================================
CONST Anchor(LOPtr,Parent,Id,where) = LOPtr.AnchorControl(GetDlgItem(Parent,Id),where)
CONST DlgAnchor(LOPtr,Parent,Id,where) = LOPtr->AnchorControl(GetDlgItem(Parent,Id),where)
'==============================================================================
ENUM As AFX_ANCHORPOINT
   AFX_ANCHOR_NONE =  0
   AFX_ANCHOR_WIDTH
   AFX_ANCHOR_RIGHT
   AFX_ANCHOR_CENTER_HORZ
   AFX_ANCHOR_HEIGHT
   AFX_ANCHOR_HEIGHT_WIDTH
   AFX_ANCHOR_HEIGHT_RIGHT
   AFX_ANCHOR_BOTTOM
   AFX_ANCHOR_BOTTOM_WIDTH
   AFX_ANCHOR_BOTTOM_RIGHT
   AFX_ANCHOR_CENTER_HORZ_BOTTOM
   AFX_ANCHOR_CENTER_VERT
   AFX_ANCHOR_CENTER_VERT_RIGHT
   AFX_ANCHOR_CENTER
END ENUM

PPTYPE AFX_ANCHORPROPERTY
   Dim As HWND hwndCtl
   Dim As LONG anchor
   Dim As RECT rc
   DIm As LONG centerX
   Dim As LONG centerY 
END PPTYPE

Class CLayout
	private:
		Dim As HWND m_hwndParent
		Dim As AFX_ANCHORPROPERTY Ptr m_rgProps
		Dim As size_t m_ubound
		 
	public:
		Dim Constructor CLayout(hwndParent As HWND)
		Dim Destructor ~CLayout()
		Dim Function GetAnchorItem(hwndCtl As HWND) As LONG
		Dim Function AnchorControl(hwndCtl AS HWND, nAnchorMode AS LONG) AS BOOL
		Dim Function AnchorControl(cID As LONG, nAnchorMode AS LONG) AS BOOL
		Dim Sub AdjustControls()
		Dim static Function AfxLayoutEnumChildProc (hwnd AS HWND, lParam AS LPARAM) AS LONG 
		 	
End Class
'==============================================================================
CLASS_METHODS_START CLayout
'------------------------------------------------------------------------------
Constructor CLayout(hwndParent As HWND)
	m_hwndParent = hwndParent
	m_ubound = 1
	Raw As size_t dims[1] = {1}
	'm_pskAnchor = (AFX_ANCHORPROPERTY*)CreateArr(m_pskAnchor,SizeOf(AFX_ANCHORPROPERTY),0,1,dims)
	'Dim Dynamic m_rgProps[_ubound] AS AFX_ANCHORPROPERTY
	'Free m_rgProps
	m_rgProps = (AFX_ANCHORPROPERTY*)CreateArr(m_rgProps,SizeOf(AFX_ANCHORPROPERTY),0,1,dims)
	' m_rgProps = (AFX_ANCHORPROPERTY*)CreateArr(m_rgProps,SizeOf(AFX_ANCHORPROPERTY),0,1,dims)
End Constructor
'------------------------------------------------------------------------------
Destructor CLayout()
	DestroyArr((void **)m_rgProps, 1, 1)
End Destructor
'------------------------------------------------------------------------------
' Get the anchor item
' Parameter:
' - hwndCtl = The control's window handle
' Return Value:
'   The array index on success, or -1 on failure.
' ========================================================================================
Function GetAnchorItem (hwndCtl AS HWND) AS LONG
   
   Dim As LONG i
   For i = 0 To m_ubound
      If m_rgProps[i].hwndCtl = hwndCtl Then
      	Function = i 
      EndIf
   Next i
   Function = -1
END FUNCTION
' ========================================================================================
Function AnchorControl(hwndCtl AS HWND, nAnchorMode AS LONG) AS BOOL
	If ::IsWindow(hwndCtl) = FALSE Then
		Exit Function
	EndIf
	Dim AS LONG nItem = GetAnchorItem(hwndCtl)
	' // If it does not exist, make room for it
	If nItem = -1 Then
		m_ubound += 1
		!size_t dims[1] = {m_ubound};
		m_rgProps = (AFX_ANCHORPROPERTY*)CreateArr(m_rgProps,sizeof(AFX_ANCHORPROPERTY),1,1,dims)
		nItem = m_ubound
	EndIf
	m_rgProps[nItem].hwndCtl = hwndCtl
	' // Get the coordinates of the control
	Dim As RECT rcCtl
	Dim As POINT pt
	::GetWindowRect(hwndCtl,&rcCtl)
	pt.x = rcCtl.left
	pt.y = rcCtl.top
	' // Convert the screen coordinates to screen to client-area coordinates
	::ScreenToClient(GetParent(hwndCtl), &pt)
	' // Get the client coordinates of the parent window
	Dim As RECT rcParent
	::GetClientRect(GetParent(hwndCtl), &rcParent)
	With m_rgProps[nItem].
		anchor    = min(max(nAnchorMode,AFX_ANCHOR_NONE),AFX_ANCHOR_CENTER)
		rc.left   = pt.x
		rc.top    = pt.y
		rc.right  = rcParent.right - (rcCtl.right - rcCtl.left + pt.x)
		rc.bottom = rcParent.bottom - (rcCtl.bottom - rcCtl.top + pt.y)
		centerX   = pt.x - (rcParent.right / 2)
		centerY   = pt.y - (rcParent.bottom / 2)
	End With
	Function = TRUE
End Function
'------------------------------------------------------------------------------
Function AnchorControl(cID As LONG, nAnchorMode AS LONG) AS BOOL
	Dim As HWND hwndCtl = GetDlgItem(m_hwndParent, cID)
	If hwndCtl Then
		Function = AnchorControl(hwndCtl,nAnchorMode)
	EndIf
	Function = FALSE
End Function
'------------------------------------------------------------------------------
Function AfxLayoutEnumChildProc (hwndChild AS HWND, lParam AS LPARAM) AS LONG
	Dim As RECT rcParent,rcChild
	Dim As LONG nWidth,nHeight,x,y,xWidth,yHeight
	Dim As CLayout Ptr pLayout = (CLayout Ptr) lParam
	If pLayout Then
		Dim As LONG nItem = pLayout->GetAnchorItem(hwndChild)
		If nItem > -1 Then
			If pLayout->m_rgProps[nItem].anchor > AFX_ANCHOR_NONE Then
	            GetWindowRect(hwndChild, &rcChild)
	            nWidth = rcChild.right - rcChild.left
	            nHeight = rcChild.bottom - rcChild.top
	            GetClientRect(GetParent(hwndChild), &rcParent)
	            x = 0: y = 0: xWidth = 0: yHeight = 0
				Select Case pLayout->m_rgProps[nItem].anchor
					Case AFX_ANCHOR_NONE
						x = pLayout->m_rgProps[nItem].rc.left
						y = pLayout->m_rgProps[nItem].rc.top
						xWidth = max(rcParent.right - pLayout->m_rgProps[nItem].rc.left - pLayout->m_rgProps[nItem].rc.right, 0)
						yHeight = nHeight
					Case AFX_ANCHOR_WIDTH
						x = pLayout->m_rgProps[nItem].rc.left
						y = pLayout->m_rgProps[nItem].rc.top
						xWidth = max(rcParent.right - pLayout->m_rgProps[nItem].rc.left - pLayout->m_rgProps[nItem].rc.right, 0)
						yHeight = nHeight
					Case AFX_ANCHOR_RIGHT
						x = rcParent.right - nWidth - pLayout->m_rgProps[nItem].rc.right
						y = pLayout->m_rgProps[nItem].rc.top
						xWidth = nWidth
						yHeight = nHeight
					Case AFX_ANCHOR_CENTER_HORZ
						x = (rcParent.right / 2) + pLayout->m_rgProps[nItem].centerX
						y = pLayout->m_rgProps[nItem].rc.top
						xWidth = nWidth
						yHeight = nHeight
					Case AFX_ANCHOR_HEIGHT
						x = pLayout->m_rgProps[nItem].rc.left
						y = pLayout->m_rgProps[nItem].rc.top
						xWidth = nWidth
						yHeight = max(rcParent.bottom - pLayout->m_rgProps[nItem].rc.top - pLayout->m_rgProps[nItem].rc.bottom, 0)
					Case AFX_ANCHOR_HEIGHT_WIDTH
						x = pLayout->m_rgProps[nItem].rc.left
						y = pLayout->m_rgProps[nItem].rc.top
						xWidth = max(rcParent.right - pLayout->m_rgProps[nItem].rc.left - pLayout->m_rgProps[nItem].rc.right, 0)
						yHeight = max(rcParent.bottom - pLayout->m_rgProps[nItem].rc.top - pLayout->m_rgProps[nItem].rc.bottom, 0)
					Case AFX_ANCHOR_HEIGHT_RIGHT
						x = rcParent.right - nWidth - pLayout->m_rgProps[nItem].rc.right
						y = pLayout->m_rgProps[nItem].rc.top
						xWidth = nWidth
						yHeight = max(rcParent.bottom - pLayout->m_rgProps[nItem].rc.top - pLayout->m_rgProps[nItem].rc.bottom, 0)
					Case AFX_ANCHOR_BOTTOM
						x = pLayout->m_rgProps[nItem].rc.left
						y = rcParent.bottom - pLayout->m_rgProps[nItem].rc.bottom - nHeight
						xWidth = nWidth
						yHeight = nHeight
					Case AFX_ANCHOR_BOTTOM_WIDTH
						x = pLayout->m_rgProps[nItem].rc.left
						y = rcParent.bottom - pLayout->m_rgProps[nItem].rc.bottom - nHeight
						xWidth = max(rcParent.right - pLayout->m_rgProps[nItem].rc.left - pLayout->m_rgProps[nItem].rc.right, 0)
						yHeight = nHeight
					Case AFX_ANCHOR_BOTTOM_RIGHT
						x = rcParent.right - nWidth - pLayout->m_rgProps[nItem].rc.right
						y = rcParent.bottom - pLayout->m_rgProps[nItem].rc.bottom - nHeight
						xWidth = nWidth
						yHeight = nHeight
					Case AFX_ANCHOR_CENTER_HORZ_BOTTOM
						x = (rcParent.right / 2) + pLayout->m_rgProps[nItem].centerX
						y = rcParent.bottom - pLayout->m_rgProps[nItem].rc.bottom - nHeight
						xWidth = nWidth
						yHeight = nHeight
					Case AFX_ANCHOR_CENTER_VERT
						x = pLayout->m_rgProps[nItem].rc.left
						y = (rcParent.bottom - nHeight) / 2
						xWidth = nWidth
						yHeight = nHeight
					Case AFX_ANCHOR_CENTER_VERT_RIGHT
						x = rcParent.right - nWidth - pLayout->m_rgProps[nItem].rc.right
						y = (rcParent.bottom - nHeight) / 2
						xWidth = nWidth
						yHeight = nHeight
					Case AFX_ANCHOR_CENTER
						x = (rcParent.right / 2) + pLayout->m_rgProps[nItem].centerX
						y = (rcParent.bottom / 2) + pLayout->m_rgProps[nItem].centerY
						xWidth = nWidth
						yHeight = nHeight
				End Select
				MoveWindow(hwndChild, x, y, xWidth, yHeight, TRUE)
			EndIf
		EndIf
	EndIf
	Function = TRUE
End Function
'------------------------------------------------------------------------------
' Adjust the controls size and/or position
' ========================================================================================
Sub AdjustControls()
   EnumChildWindows(m_hwndParent, (WNDENUMPROC) &AfxLayoutEnumChildProc, (LPARAM) this)
End Sub

CLASS_METHODS_END

