' ########################################################################################
' Microsoft Windows
' File: CPgBar3D.inc
' Contents: Progress bar control
' Based on the PGBAR3D control originally written by Brje Hagsten.
' Compiler: Ported to bc9Basic jcfuller 09/01/2016  
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
' ========================================================================================
' wParam colors for PGB_SETBARCOL message
' ========================================================================================
CONST PGB_SILVER = 0
CONST PGB_RED    = 1
CONST PGB_GREEN  = 2
CONST PGB_BLUE   = 3
CONST PGB_CYAN   = 4   ' // Blue-green
CONST PGB_VIOLET = 5   ' // Red-blue
CONST PGB_GOLD   = 6   ' // Yellow
CONST PGB_BRONZE = 7   ' // Brown
' ========================================================================================

' ========================================================================================
' wParam colors for PGB_SETTXTON message
' ========================================================================================
CONST PGB_TEXTNONE    = 0
CONST PGB_TEXTAUTO    = 1
CONST PGB_TEXTCUSTOM  = 2
' ========================================================================================

' ========================================================================================
' wParam colors for PGB_SETBARDIR message
' ========================================================================================
CONST PGB_BD_LEFTTORIGHT = 0
CONST PGB_BD_RIGHTTOLEFT = 1
CONST PGB_BD_BOTTOMTOTOP = 2
CONST PGB_BD_TOPTOBOTTOM = 3
' ========================================================================================

' ========================================================================================
' wParam colors for PGB_SETGRADIENTDIR message
' ========================================================================================
CONST PGB_GD_HORIZONTAL = 0
CONST PGB_GD_VERTICAL   = 1
' ========================================================================================

' ========================================================================================
'Custom control messages
' ========================================================================================
CONST PGB_SETMAX         = WM_USER + 100   ' // wParam sets max number of steps
CONST PGB_STEPUP         = WM_USER + 103   ' // Increases step while < max; wParam and lParam must be 0
CONST PGB_STEPDN         = WM_USER + 104   ' // Decreases step while > 0; wParam and lParam must be 0
CONST PGB_SETVALUE       = WM_USER + 105   ' // wParam sets progess bar value, lParam controls redraw
CONST PGB_BUILDBARS      = WM_USER + 109   ' // Build/rebuild the progress bars, lParam controls redraw
CONST PGB_REFRESH        = WM_USER + 110   ' // Redraw the control; wParam and lParam must be 0

CONST PGB_GETMAX         = WM_USER + 120   ' // Returns max number of steps
CONST PGB_GETVALUE       = WM_USER + 121   ' // Returns step value
CONST PGB_GETTXTON       = WM_USER + 122   ' // Returns txtOnOff value
CONST PGB_GETTXTPOS      = WM_USER + 123   ' // Returns text position in control
CONST PGB_GETTXTCOLBAR   = WM_USER + 124   ' // Returns bar text color
CONST PGB_GETTXTCOLBKG   = WM_USER + 125   ' // Returns background text color
CONST PGB_GETCOLBKG      = WM_USER + 126   ' // Returns background color
CONST PGB_GETBARCOL      = WM_USER + 127   ' // Returns bar color scheme
CONST PGB_GETBARDIR      = WM_USER + 128   ' // Returns bar direction, 0:left/right 1:upside down 2:bottom/top 3:top/bottom
CONST PGB_GETGRADIENTDIR = WM_USER + 129   ' // Returns gradient direction - 0:horizontal, 1:vertical
CONST PGB_GETTXTANGLE    = WM_USER + 130   ' // Returns rotated font

CONST PGB_SETTXTON       = WM_USER + 150   ' // lParam sets: 0 = no text, 1 = auto text (%), 2 = custom text
CONST PGB_SETTXTBAR      = WM_USER + 151   ' // wParam points to text text for bar, lParam controls redraw
CONST PGB_SETTXTBKG      = WM_USER + 152   ' // wParam points to text text for background, lParam controls redraw
CONST PGB_SETTXTPOS      = WM_USER + 153   ' // wParam sets text position in control
CONST PGB_SETTXTCOLBAR   = WM_USER + 154   ' // wParam sets bar text color

CONST PGB_SETTXTCOLBKG   = WM_USER + 155   ' // wParam sets background text color
CONST PGB_SETCOLBKG      = WM_USER + 156   ' // wParam sets background color, lParam controls rebuild of control
CONST PGB_SETBARCOL      = WM_USER + 157   ' // wParam sets bar color scheme, lParam controls rebuild of control
CONST PGB_SETBARDIR      = WM_USER + 159   ' // wParam sets bar direction, 0:left/right 1:upside down 2:bottom/top 3:top/bottom, lParam controls rebuild of control
CONST PGB_SETGRADIENTDIR = WM_USER + 160   ' // wParam sets gradient direction - 0:horizontal, 1:vertical, lParam controls rebuild of control
CONST PGB_SETTXTANGLE    = WM_USER + 161   ' // wParam sets rotated font, lParam controls rebuild of control
CONST PGB_SETFONT        = WM_USER + 162   ' // wParam = Handle of the font, lParam controls rebuild of control
CONST PGB_GETFONT        = WM_USER + 162   ' // Returns the hanfle of the font
' ========================================================================================
Class CPgBar3D
	private: 
		Dim	m_hCtl        AS HWND            ' // Button handle
		Dim	m_pStep       AS LONG            ' // For tracking what step we are on
		Dim	m_pMax        AS LONG            ' // for storing max number of steps, usually 100 (%)
		Dim	m_direction   AS LONG            ' // Bar direction - left to right, or right to left?
		Dim	m_gradientDir AS LONG            ' // Gradient direction - left to right, or right to left?
		Dim	m_txtAngle    AS LONG            ' // Store given text angle
		Dim	m_bkgColor    AS LONG            ' // Background color
		Dim	m_barCol      AS LONG            ' // Bar color scheme
		Dim	m_txtColBar   AS LONG            ' // Custom text color in bar
		Dim	m_txtColBkg   AS LONG            ' // Custom text color on background
		Dim	m_txtOnOff    AS LONG            ' // 0 = no text, 1 = auto text (%), 2 = custom text
		Dim	m_txtPos      AS LONG            ' // Text position in control, see DrawText API..
		Dim m_txtBkg[255] AS char   		' // Text to be painted on background, increase/decrease size to suit your needs
		Dim m_txtBar[255] As char   		' // text to be painted on bar, increase/decrease size to suit your needs
		Dim	m_hbBack      AS HBRUSH          ' // Handle for background brush
		Dim	m_barDC       AS HDC             ' // Memory DC for progress bar
		Dim	m_barBit      AS HBITMAP         ' // Handle to progress bar bitmap
		Dim	m_barDC2      AS HDC             ' // Memory DC for progress bar buffer
		Dim	m_barBit2     AS HBITMAP         ' // Handle to progress bar buffer bitmap
		Dim	m_memDc       AS HDC             ' // Memory DC for main buffer
		Dim	m_hBit        AS HBITMAP         ' // Handle to main buffer bitmap
		Dim	m_hFont       AS HFONT           ' // Handle to font used by the control
		Dim	m_hRotateFont AS HFONT           ' // Handle to rotated font style
		DIM m_PalClr[192] AS LONG        	' // Array of colors for the gradients
	public:
		Dim Constructor CPgBar3D(pWindow As CWindow Ptr,cID As Integer,wszTitle As const char ptr = "", _
		                         x As Long = 0,y As Long = 0,nWidth As Long = 0,nHeight As Long = 0, _
		                         dwStyle As DWORD = 0, dwExStyle = 0,lParam As LONG_PTR = 0)
		Dim Destructor  ~CPgBar3D()  
		Dim Function hWindow() As HWND
		Dim Sub Redraw()
		Dim Sub Clear()
		Dim Sub CreateGradientBars
		Dim Function GetMaxSteps () AS LONG
		Dim Sub SetMaxSteps (nMaxSteps AS LONG)
		Dim Function GetStep () AS LONG
		Dim Sub SetStep (nStepValue AS LONG, fRedraw AS LONG = FALSE)
		Dim Function GetTextOn () AS LONG
		Dim Sub SetTextOn (nMode AS LONG)
		Dim Function GetTextPos () AS LONG
		Dim Sub SetTextPos (nPos AS LONG)
		Dim Function GetBarColor () AS LONG
		Dim Sub SetBarColor (nColor AS LONG)
		Dim Function GetBarTextColor () AS LONG
		Dim Sub SetBarTextColor (nColor AS LONG, fRedraw AS LONG = FALSE)
		Dim Function GetBarDirection () AS LONG
		Dim Sub SetBarDirection (nDirection AS LONG, fRedraw AS LONG = FALSE)
		Dim Function GetBackColor () AS LONG
		Dim Sub SetBackColor (nColor AS LONG, fRedraw AS LONG = FALSE)
		Dim Function GetTextBackColor () AS LONG
		Dim Sub SetTextBackColor (nColor AS LONG, fRedraw AS LONG = FALSE)
		Dim Function GetGradientDirection () AS LONG
		Dim Sub SetGradientDirection (nAngle AS LONG)
		Dim Function GetTextAngle () AS LONG
		Dim Sub SetTextAngle (nAngle AS LONG, fRebuild AS LONG = FALSE)
		Dim Sub StepUp()
		Dim Sub StepDown()
		Dim Sub BuildBar (fRedraw AS LONG = FALSE)
		Dim Sub SetBarText (wszText AS const char ptr, fRedraw AS LONG = FALSE)
		Dim Sub SetBackText (wszText AS const char ptr, fRedraw AS LONG = FALSE)
		Dim Sub SetFont (hFont AS HFONT, fRedraw AS LONG = FALSE)
		Dim Function GetFont () AS HFONT
		Dim STATIC FUNCTION CPgBar3DProc (hwnd AS HWND, uMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
End Class
CLASS_METHODS_START CPgBar3D
Constructor CPgBar3D(pWindow As CWindow Ptr,cID As Integer,wszTitle As const char ptr, _
		                         x As Long,y As Long,nWidth As Long,nHeight As Long, _
		                         dwStyle As DWORD, dwExStyle,lParam As LONG_PTR )
	'Register the class
	Dim As ATOM wAtom
	DIM AS WNDCLASSEXW wcexw
	Dim wszClassName$
	wszClassName$ = "AFX_PGBAR3D"
	If GetClassInfoEx(GetModuleHandle(NULL),wszClassName$,&wcexw) = 0 Then
		'Fill the struct
		With wcexw
			.cbSize = SizeOf(wcexw)
			
		End With
	EndIf
End Constructor
CLASS_METHODS_END
