' ########################################################################################
' Microsoft Windows
' File: CXpButton.inc
' Contents: Themed button control
' Draws a themed button in Windows XP or superior and a classic button in the other Windows versions.
' Do not use it with Windows 95 because the TrackMouseEvent is not available in that OS.
' Compiler: FreeBasic 32 & 64-bit
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
' ========================================================================================
' Image class
' ========================================================================================
CONST XPBI_NORMAL = 1
CONST XPBI_HOT = 2
CONST XPBI_DISABLED = 3
' ========================================================================================

' ========================================================================================
' Image position
' ========================================================================================
CONST XPBI_NONE = 0x0                                   ' // No image
CONST XPBI_LEFT = 0x1                                   ' // Left (default)
CONST XPBI_RIGHT = 0x2                                  ' // Right
CONST XPBI_CENTER = 0x4                                 ' // Center
CONST XPBI_VCENTER = 0x8                                ' // Vertically centered
CONST XPBI_TOP = 0x10                                   ' // Top
CONST XPBI_BOTTOM = 0x20                                ' // Bottom
CONST XPBI_ABOVE = XPBI_TOP BOR XPBI_CENTER              ' // Above the text
CONST XPBI_BELOW = XPBI_BOTTOM BOR XPBI_CENTER           ' // Below the text
CONST XPBI_CENTERCENTER = XPBI_CENTER BOR XPBI_VCENTER   ' // Center-center (no text)
' ========================================================================================
' ========================================================================================
' CXpButton class
' ========================================================================================
Class CXpButton
	private:
		Dim As HWND 	m_hCtl			' // Button handle
		Dim As LONG		m_fState		' // Button state
		Dim As DWORD	m_dwTextFlags	' // Format text flags
		Dim As HCURSOR	m_hCursor		' // Cursor handle
      	Dim As HFONT	m_hFont         ' // Default font handle
		Dim As HANDLE	m_hImage        ' // Image handle
		Dim As LONG		m_ImageType     ' // Image type
		Dim As LONG		m_ImageWidth    ' // Image width
		Dim As LONG		m_ImageHeight   ' // Image height
		Dim As LONG		m_ImagePos      ' // Image position
		Dim As LONG		m_ImageMargin   ' // Image margin (in pixels)
		Dim As LONG		m_TextMargin    ' // Text margin (in pixels)
		Dim As HANDLE	m_hHotImage     ' // Hot image handle
		Dim As HANDLE	m_hDisabledImage' // Disabled image handle
		Dim As LONG		m_bIsToggle     ' // Button is a toggle button
		Dim As LONG		m_bToggled      ' // Toggle flag
		Dim As LONG		m_bIsThemed     ' // Application is themed
		Dim As LONG		m_bClkDown      ' // Button received the button down message
	public:
		Dim Constructor CXpButton(pWindow As CWindow Ptr,cID AS INTEGER, szTitle As char Ptr = "", _
            x AS LONG = 0,y AS LONG = 0,nWidth AS LONG = 0,nHeight AS LONG = 0, _
            dwStyle AS DWORD = 0,dwExStyle AS DWORD = 0,lpParam AS LONG_PTR = 0)	
        Dim Destructor ~CXpButton()
        Dim Function hWindow() As HWND
        Dim Sub Redraw()
        Dim Sub SetFont(hFont AS HFONT, fRedraw AS LONG = FALSE)
        Dim Function GetFont() As HFONT
        Dim Sub SetTextFormat (dwTextFlags AS DWORD, fRedraw AS LONG = FALSE)
        Dim Function GetTextFormat() As DWORD
        Dim Sub SetCursor (hCursor As HCURSOR)
        Dim Function GetCursor() As HCURSOR
        Dim Sub SetToggle (fState AS LONG,fRedraw AS LONG = FALSE)
        Dim Function GetToggle() As LONG
        Dim Sub SetToggleState (fToggle AS LONG,fRedraw AS LONG = FALSE)
        Dim Function GetToggleState() AS LONG
        Dim Sub SetImageSize (nWidth AS LONG, nHeight AS LONG, fRedraw AS LONG = FALSE)
        Dim Sub SetImageWidth (nWidth AS LONG, fRedraw AS LONG = FALSE)
        Dim Function GetImageWidth () AS LONG
        Dim Sub SetImageHeight (nHeight AS LONG, fRedraw AS LONG = FALSE)
        Dim Function GetImageHeight () AS LONG
        Dim Sub SetImagePos (nPos AS LONG, fRedraw AS LONG = FALSE)
        Dim Function GetImagePos () AS LONG
        Dim Sub SetImageMargin (nMargin AS LONG,fRedraw AS LONG = FALSE)
        Dim Function GetImageMargin () AS LONG
		Dim Sub SetTextMargin (nMargin AS LONG, fRedraw AS LONG = FALSE)
		Dim Function GetTextMargin () AS LONG
		Dim Function GetButtonState () AS LONG
		Dim Function GetImageType () AS LONG
		Dim Function IsThemed () AS LONG
		Dim Sub EnableTheming
		Dim Sub DisableTheming
		Dim Function GetImage (ImageState AS LONG) AS HANDLE
		Dim Sub UxDrawPushButton (hDC AS HDC)
		Dim Sub SetImage (hImage AS HANDLE, ImageType AS LONG, ImageState AS LONG, fRedraw AS LONG = FALSE)
		Dim Sub SetIcon (hIcon AS HICON, ImageState AS LONG, fRedraw AS LONG = FALSE)
		Dim Sub SetBitmap (hBitmap AS HBITMAP, ImageState AS LONG, fRedraw AS LONG = FALSE)
		Dim Sub SetIconFromFile (szPath As char Ptr, ImageState AS LONG, fRedraw AS LONG = FALSE)
		Dim Sub SetBitmapFromFile (szPath AS char Ptr, ImageState AS LONG, fRedraw AS LONG = FALSE)
		Dim Static Function CXpButtonProc (hWnd AS HWND, uMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
End Class
CLASS_METHODS_START CXpButton
' CXpButton class constructor
' ========================================================================================
Constructor CXpButton (pWindow AS CWindow PTR, cID AS INTEGER, szTitle AS char ptr , _
   x AS LONG , y AS LONG , nWidth AS LONG , nHeight AS LONG , _
   dwStyle AS DWORD, dwExStyle AS DWORD , lpParam AS LONG_PTR )
   ' // Register the class
	Dim As ATOM wAtom
	Dim As WNDCLASSEXW wcexw 
	Dim szClassName$
	szClassName$ = "AFX_XPBUTTON"
	If ::GetClassInfoExW(::GetModuleHandleW(NULL), szClassName, &wcexw) = 0 THEN
		' // Fill the WNDCLASSEXW structure
		WITH wcexw
		.cbSize        = SIZEOF(wcexw)
		.style         = CS_DBLCLKS BOR CS_HREDRAW BOR CS_VREDRAW
		.lpfnWndProc   = &CXpButtonProc
		.cbClsExtra    = 0
		.cbWndExtra    = SIZEOF(HANDLE)
		.hInstance     = ::GetModuleHandleW(NULL)
		.hCursor       = ::LoadCursorW(NULL, (LPCWSTR) IDC_ARROW)
		.hbrBackground = NULL
		.lpszMenuName  = NULL
		.lpszClassName = (LPCWSTR)&szClassName
		.hIcon         = NULL
		.hIconSm       = NULL
		END WITH
		wAtom = ::RegisterClassExW(&wcexw)
		
		'If wAtom Then
		'	zTrace("YES")
		'Else
		'	zTrace("NO")
		'EndIf
	END IF
	' // Checks if the application is themed
	If IsAppThemed() AND IsThemeActive() Then
		m_bIsThemed = TRUE
	EndIf
	' // Default values
	m_dwTextFlags = DT_CENTER BOR DT_VCENTER BOR DT_SINGLELINE
	m_ImagePos = XPBI_LEFT BOR XPBI_VCENTER
	m_ImageMargin = 4
	m_TextMargin = 4
	m_hCursor = LoadCursor(NULL, IDC_ARROW)

	' // Create the control
	If dwStyle = 0 Then
		dwStyle = WS_VISIBLE BOR WS_TABSTOP BOR BS_PUSHBUTTON BOR BS_CENTER BOR BS_VCENTER
	EndIf
	If pWindow Then
		m_hCtl = pWindow->AddControl(szClassName, pWindow->hWindow(), cID, szTitle, x, y, nWidth, nHeight, dwStyle, dwExStyle, lpParam)
	EndIf
	If m_hCtl Then
		SetWindowLongPtrW(m_hCtl, 0, (LONG_PTR) this)
		' // Set the same font used by the parent
		Dim  AS LOGFONTW lfw 
		If pWindow->Font() Then
	 		If GetObjectW(pWindow->Font(), SIZEOF(lfw), &lfw) Then
	 			m_hFont = CreateFontIndirectW(&lfw)
	 		EndIf
		End If
	End If
	
End Constructor
'------------------------------------------------------------------------------
Destructor CXpButton()
	' // Deletes the images
	If m_hImage Then
		::DeleteObject(m_hImage)
	EndIf
	If m_hHotImage Then
		::DeleteObject(m_hHotImage)
	EndIf
	If m_hDisabledImage Then
		::DeleteObject( m_hDisabledImage)
	EndIf
	' // Deletes the font
	IF m_hFont Then
		::DeleteObject(m_hFont)
	EndIf
End Destructor
Function CXpButtonProc (hWnd AS HWND, uMsg AS UINT, wParam AS WPARAM, lParam AS LPARAM) AS LRESULT
	Raw As CXpButton PTR pButton 
	Dim As HDC hDc 
	Dim As PAINTSTRUCT pPaint 
	Dim As RECT rc 
	Dim As POINT pt
	Select Case uMsg
		Case WM_CREATE
			Exit Function
		Case WM_GETDLGCODE
			Dim As MSG Ptr pMsg 
			Dim As DWORD dwMsg
			If lParam = 0 Then
				' // Enables mnemonics
				Function = DLGC_BUTTON BOR DLGC_UNDEFPUSHBUTTON
				
			Else
				' // We want to handle the return key
				pMsg = (MSG PTR) lParam
				If pMsg > 0 Then
					dwMsg = pMsg->message
					If (dwMsg = WM_KEYDOWN) OR (dwMsg = WM_CHAR) Then
						If pMsg->wParam = VK_RETURN Then
							Function = DLGC_WANTALLKEYS
						End If
					End If
				End if
			End If
			Exit Function
		Case WM_ERASEBKGND
        	' // Don't erase the background to avoid flicker
         	Function = 1
		Case WM_PAINT
			' // Draws the button
			hDc = ::BeginPaint(hWnd, &pPaint)
			pButton = (CXpButton PTR) ::GetWindowLongPtrW(hWnd, 0)
			If pButton Then
				pButton->UxDrawPushButton(hDc)
			EndIf
			::EndPaint(hWnd, &pPaint)
			Exit Function
		Case WM_PRINTCLIENT
			' // Draws the button
			pButton = (CXpButton PTR) ::GetWindowLongPtrW(hWnd, 0)
			If pButton Then
				pButton->UxDrawPushButton((HDC) wParam)
			EndIf
			Exit Function
		Case WM_COMMAND, WM_NOTIFY
			' // Forwards the message to the parent window
			::SendMessageW(GetParent(hWnd), uMsg, wParam, lParam)
		Case WM_LBUTTONDBLCLK
			' // Forwards this message for rapid button clicking...
			SendMessageW(hWnd, WM_LBUTTONDOWN, wParam, lParam)
			Exit Function
		Case WM_LBUTTONDOWN
			' // Redraws the button in pushed state
			pButton = (CXpButton PTR) GetWindowLongPtrW(hWnd, 0)
			If pButton = NULL Then
				Exit Function
			EndIf
			GetWindowRect(pButton->hWindow(), &rc)
			GetCursorPos(&pt)
			If PtInRect(&rc, pt) Then
				pButton->m_bClkDown = TRUE
				If pButton->m_bToggled = FALSE Then
					pButton->m_bToggled = TRUE
				Else
					pButton->m_bToggled = FALSE
				End if
				pButton->m_fState = BST_PUSHED
				SetCapture(pButton->hWindow())
				If GetFocus() <> pButton->hWindow() Then
					SetFocus(pButton->hWindow())
				EndIf
				pButton->Redraw()
			End if
			Exit Function
			
		Case WM_KEYDOWN
			' // Redraws the button in pushed state
			pButton = (CXpButton PTR) GetWindowLongPtrW(hWnd, 0)
			If pButton = NULL Then
				Exit Function
			EndIf
			Select Case wParam
				Case VK_RETURN, VK_SPACE
					' // Toggles the toggle flag
					If pButton->m_bToggled = FALSE Then
			  			pButton->m_bToggled = TRUE
					Else
			  			pButton->m_bToggled = FALSE
					End If
					' // Redraws the button in pushed state
					pButton->m_fState = BST_PUSHED
					pButton->Redraw()
			End Select
			Exit Function
		Case WM_LBUTTONUP
			' // Redraws the button in normal state
			pButton = (CXpButton PTR) GetWindowLongPtrW(hWnd, 0)
			If pButton = NULL Then
				EXIT FUNCTION
			EndIf
			ReleaseCapture()
			GetWindowRect(pButton->hWindow(), &rc)
			GetCursorPos(&pt)
			pButton->m_fState = 0
			pButton->Redraw()
			' // Forwards the message to the parent window
			If IsWindowEnabled(pButton->hWindow()) Then
				If PtInRect(&rc, pt) Then
					If pButton->m_bClkDown Then
				   		SendMessageW(GetParent(pButton->hWindow()), WM_COMMAND, MAKELONG(GetDlgCtrlID(pButton->hWindow()), BN_CLICKED), (LPARAM) pButton->hWindow())
					EndIf
				End If
			End If
			pButton->m_bClkDown = FALSE
			Exit Function
			
		Case WM_KEYUP
			' // Redraws the button in normal state
			pButton = (CXpButton PTR) GetWindowLongPtrW(hWnd, 0)
			If pButton = NULL Then
				Exit Function
			EndIf
			If pButton->m_fState = BST_PUSHED Then
				pButton->m_fState = 0
				pButton->Redraw()
				' // Forwards the message to the parent window
				If IsWindowEnabled(pButton->hWindow()) Then
					SendMessageW( GetParent(pButton->hWindow()), WM_COMMAND, MAKELONG(GetDlgCtrlID(pButton->hWindow()), BN_CLICKED), (LPARAM) pButton->hWindow())
				End If
			End If
			Exit Function
		Case WM_ENABLE
			' // Redraws the button
			InvalidateRect(hWnd, NULL, 0)
			UpdateWindow(hWnd)
			Exit Function
			
	End Select
	Function = DefWindowProcW(hWnd, uMsg, wParam, lParam)
End Function
'------------------------------------------------------------------------------
' Returns the handle of the button
' ========================================================================================
Function hWindow () AS HWND
   Function = m_hCtl
End Function
'==============================================================================
' Redraws the button
' ========================================================================================
Sub Redraw()
   InvalidateRect(m_hCtl, NULL, 0)
   UpdateWindow(m_hCtl)
End Sub
'==============================================================================
' Sets the font for the button
' Parameters:
' - hFont = Font handle
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetFont (hFont AS HFONT, fRedraw AS LONG = FALSE)
	If hFont = NULL Then
		EXIT SUB
	EndIf
	If m_hFont Then
		DeleteObject(m_hFont)
	EndIf
	m_hFont = hFont
	If fRedraw Then
		Redraw()
	EndIf
End Sub
'==============================================================================
' Returns the font handle
' ========================================================================================
Function GetFont () AS HFONT
   Function = m_hFont
End Function
'==============================================================================
' Sets the text format
' Parameters:
' - dwTextFlags = Format values (default DT_CENTER OR DT_VCENTER OR DT_SINGLELINE)
'   - DT_BOTTOM = Renders the text string at the bottom of the display rectangle.
'     This value is used only with the DT_SINGLELINE value.
'   - DT_CALCRECT = Determines the width and height of the display rectangle.
'   - DT_CENTER = Centers text horizontally in the display rectangle.
'   - DT_EDITCONTRO = Duplicates the text-displaying characteristics of a multiline edit
'     control. Specifically, the average character width is calculated in the same manner
'     as for an edit control, and the function does not display a partially visible last
'     line.
'   - DT_END_ELLIPSIS = Truncates a text string that is wider than the display rectangle
'     and adds an ellipsis to indicate the truncation. The string is not modified unless
'     the DT_MODIFYSTRING flag is specified.
'   - DT_EXPANDTABS = Expands tab characters. The default number of characters per tab
'     is eight. The DT_WORD_ELLIPSIS, DT_PATH_ELLIPSIS, and DT_END_ELLIPSIS values cannot
'     be used with the DT_EXPANDTABS value.
'   - DT_EXTERNALLEADING = Includes the external leading of a font in the line height.
'     Normally, external leading is not included in the height of a line of text.
'   - DT_HIDEPREFIX = Ignores the prefix character & in the text. The letter that follows
'     is not underlined, but other prefix characters are still processed. For example:
'        input string: "A&bc&&d"
'        normal: "Abc&d"
'        DT_HIDEPREFIX: "Abc&d"
'   - DT_LEFT = Aligns text to the left.
'   - DT_MODIFYSTRING = Modifies a string to match the displayed text. This value has
'     no effect unless DT_END_ELLIPSIS or DT_PATH_ELLIPSIS is specified.
'   - DT_NOCLIP = Draws the text string without clipping the display rectangle.
'   - DT_NOFULLWIDTHCHARBREAK = Prevents a line break at a double-byte character set
'     (DBCS), so that the line-breaking rule is equivalent to single-byte character set
'     (SBCS). This can be used, for example, to make icon labels written in Korean text
'     more readable. This value has no effect unless DT_WORDBREAK is specified.
'   - DT_NOPREFIX = Turns off processing of prefix characters. Normally, DrawThemeText
'     interprets the prefix character & as a directive to underscore the character that
'     follows, and the prefix characters && as a directive to print a single &. By
'     specifying DT_NOPREFIX, this processing is turned off. For example:
'        input string: "A&bc&&d"
'        normal: "Abc&d"
'        DT_NOPREFIX: "A&bc&&d"
'   - DT_PATH_ELLIPSIS = Replaces characters in the middle of text with an ellipsis so
'     that the result fits in the display rectangle. If the string contains backslash (\)
'     characters, DT_PATH_ELLIPSIS preserves as much as possible of the text after the
'     last backslash. The string is not modified unless the DT_MODIFYSTRING flag is
'     specified.
'   - DT_PREFIXONLY = Draws only an underline at the position of the character following
'     the prefix character &. Normally DrawThemeText interprets the & as a directive to
'     underline the character that follows and the prefix characters && as a directive to
'     print a single &. By specifying DT_PREFIXONLY, no characters are drawn, only an
'     underline. White spaces are placed in the positions where characters would normally
'     appear. For example:
'        input string: "A&bc&&d"
'        normal: "Abc&d"
'        DT_PREFIXONLY: " _   "
'   - DT_RIGHT = Aligns text to the right.
'   - DT_RTLREADING = Lays out text in right-to-left order for bidirectional text, for
'     example, text in a Hebrew or Arabic font. The default direction for text is
'     left-to-right.
'   - DT_SINGLELINE = Displays text on a single line. Carriage returns and line feeds do
'     not break the line.
'   - DT_TABSTOPS = Sets tab stops.
'   - DT_TOP = Renders the text at the top of the display rectangle.
'   - DT_VCENTER = Centers text vertically. This value is used only with the DT_SINGLELINE
'     value.
'   - DT_WORDBREAK = Breaks lines between words if a word would extend past the edge of
'     the display rectangle. A carriage return/line feed (CR/LF) sequence also breaks the
'     line.
'   - DT_WORD_ELLIPSIS = Truncates any word that does not fit in the display rectangle
'     and adds an ellipsis.
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetTextFormat (dwTextFlags AS DWORD, fRedraw AS LONG = FALSE)
	m_dwTextFlags = dwTextFlags
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Returns the text format
' ========================================================================================
Function GetTextFormat () AS DWORD
	Function = m_dwTextFlags
End Function
'==============================================================================
' Sets the cursor for the button
' Parameter:
' - hCursor = Cursor handle
' ========================================================================================
Sub SetCursor ( hCursor AS HCURSOR)
	If m_hCursor Then
		DestroyCursor(m_hCursor)
	EndIf
	m_hCursor = hCursor
End Sub
'==============================================================================
' Gets the cursor for the button
' ========================================================================================
Function GetCursor () AS HCURSOR
	Function = m_hCursor
End Function
' ========================================================================================
' Sets button to toggle state (TRUE) or to pushbutton state (FALSE)
' Parameters:
' - fToggle = Toggle state (TRUE or FALSE)
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetToggle (fToggle AS LONG, fRedraw AS LONG = FALSE)
	If fToggle <> 0 Then
		fToggle = TRUE
	EndIf
	m_bIsToggle = fToggle
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Gets the button toggle state
' Returns CTRUE if the button is toggled or FALSE otherwise.
' ========================================================================================
Function GetToggle () AS LONG
	Function = m_bIsToggle
End Function
' ========================================================================================
' Set toggle state to pushed (TRUE) or unpushed (FALSE)
' Parameters:
' - fState = Toggled state (TRUE or FALSE)
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetToggleState ( fState AS LONG,  fRedraw AS LONG = FALSE)
	If fState <> 0 Then
		fState = -1
	EndIf
	m_bToggled = fState
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Returns TRUE if button is toggled (down) or FALSE otherwise
' ========================================================================================
Function  GetToggleState () AS LONG
	Function  = m_bToggled
End Function
' ========================================================================================
' Sets the size of the image
' Parameters:
' - nWidth = Width of the image, in pixels.
' - nHeight = Height of the image, in pixels.
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetImageSize ( nWidth AS LONG,  nHeight AS LONG,  fRedraw AS LONG = FALSE)
	m_ImageWidth = nWidth
	m_ImageHeight = nHeight
	If fRedraw Then
		Redraw()
	EndIf
End Sub
'==============================================================================
' Sets the width of the image
' Parameters:
' - nWidth = Width of the image, in pixels.
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetImageWidth ( nWidth AS LONG,  fRedraw AS LONG = FALSE)
	m_ImageWidth = nWidth
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Gets the width of the image.
' ========================================================================================
Function GetImageWidth () AS LONG
	Function = m_ImageWidth
End Function
' ========================================================================================
' Sets the height of the image
' Parameters:
' - nHeight = Height of the image, in pixels.
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetImageHeight ( nHeight AS LONG,  fRedraw AS LONG = FALSE)
	m_ImageHeight = nHeight
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Gets the height of the image.
' ========================================================================================
Function GetImageHeight () AS LONG
	Function = m_ImageHeight
End Function
' ========================================================================================
' Sets the position of the image
' Parameters:
' - nPos = Position.
'     %XPBI_NONE = 0x0                                    ' // No image
'     %XPBI_LEFT = 0x1                                    ' // Left (default)
'     %XPBI_RIGHT = 0x2                                   ' // Right
'     %XPBI_CENTER = 0x4                                  ' // Center
'     %XPBI_VCENTER = 0x8                                 ' // Vertically centered
'     %XPBI_TOP = 0x10                                    ' // Top
'     %XPBI_BOTTOM = 0x20                                 ' // Bottom
'     %XPBI_ABOVE = %XPBI_TOP OR %XPBI_CENTER             ' // Above the text
'     %XPBI_BELOW = %XPBI_BOTTOM OR %XPBI_CENTER          ' // Below the text
'     %XPBI_CENTERCENTER = %XPBI_CENTER OR %XPBI_VCENTER  ' // Center-center (no text)
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetImagePos ( nPos AS LONG,  fRedraw AS LONG = FALSE)
	m_ImagePos = nPos
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Gets the position of the image.
' ========================================================================================
Function GetImagePos () AS LONG
	Function = m_ImagePos
End Function
' ========================================================================================
' Sets the image margin
' Parameters:
' - nMargin = Margin (in pixels)
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetImageMargin ( nMargin AS LONG,  fRedraw AS LONG = FALSE)
	m_ImageMargin = nMargin
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Gets the margin of the image.
' ========================================================================================
Function GetImageMargin () AS LONG
	Function = m_ImageMargin
End Function
' ========================================================================================
' Sets the text margin
' Parameters:
' - nMargin = Margin (in pixels)
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetTextMargin ( nMargin AS LONG,  fRedraw AS LONG = FALSE)
	m_TextMargin = nMargin
	If fRedraw Then
		Redraw()
	EndIf
End Sub
' ========================================================================================
' Gets the text margin.
' ========================================================================================
Function GetTextMargin () AS LONG
	Function = m_TextMargin
End Function
' ========================================================================================
' Returns the button state
' ========================================================================================
Function GetButtonState () AS LONG
	Function = m_fState
End Function
'==============================================================================
' Returns the image type
' ========================================================================================
Function GetImageType () AS LONG
	Function = m_ImageType
End Function
' ========================================================================================
' Returns TRUE if themes are enabled or FALSE otherwise
' ========================================================================================
Function IsThemed () AS LONG
	Function = m_bIsThemed
End Function
' ========================================================================================
' Enables theming
' ========================================================================================
Sub EnableTheming()
	m_bIsThemed = TRUE
	Redraw()
End Sub
' ========================================================================================
' Disables theming
' ========================================================================================
Sub DisableTheming()
	m_bIsThemed = FALSE
	Redraw()
End Sub
' ========================================================================================
' Returns the handle of the image
' Parameters:
' - ImageState =
'      XPBI_NORMAL = 1
'      XPBI_HOT = 2
'      XPBI_DISABLED = 3
' ========================================================================================
Function GetImage ( ImageState AS LONG) AS HANDLE
	Select Case ImageState
		Case XPBI_NORMAL
			Function = m_hImage
		Case XPBI_HOT
			Function = m_hHotImage
		Case XPBI_DISABLED
			Function = m_hDisabledImage
	End Select
	Function = NULL
End Function
' ========================================================================================

'------------------------------------------------------------------------------
' Draws the button
' ========================================================================================

Sub UxDrawPushButton (hDc AS HDC)
	Dim As HTHEME hTheme                    ' // Theme handle
	Dim As RECT rcClient                   ' // Original coordinates of the button client area
	Dim As RECT rc                         ' // Coordinates of the button client area
	Dim As RECT rcContent                 ' // Coordinates of the button content
	Dim As LONG iStateId                   ' // Button state
	Dim As LONG bIsPressed                 ' // Boolean (pressed state)
	Dim As LONG bIsFocused                 ' // Boolean (focused state)
	Dim As LONG bIsDisabled                ' // Boolean (disabled state)
	Dim As LONG iy                          ' // x-coordinate of the image
	Dim As LONG  ix                         ' // y-coordinate of the image
	Dim As HANDLE hImage                   ' // Image handle
	Dim As LONG fImageDrawn                ' // Boolean
	Dim As LONG lStyle                     ' // Button styles
	Dim As DWORD uState                    ' // Initial state of the frame control
	Dim wszCaption$ * 260         ' // Button text
	
	' // Selects the font
	If hDc = 0 Then
		EXIT SUB
	EndIf
	SelectObject(hDc, m_hFont)
	
	' // Button style
	If m_bIsThemed = FALSE Then
		lStyle = GetWindowLongPtrW(m_hCtl, GWL_STYLE)
	EndIf
	
	' // Gets the text of the button
	GetWindowTextW(m_hCtl, wszCaption, SIZEOF(wszCaption))
	
	' // Opens the theme data for the specified window and class
	If m_bIsThemed Then
		hTheme = OpenThemeData(m_hCtl, "Button")
		If hTheme = 0 Then
			Exit Sub
		EndIf
	End If
	
	' // Gets the button state
	If m_hCtl = GetFocus() Then
		bIsFocused = TRUE
	EndIf
	If (m_fState BAND BST_PUSHED) = BST_PUSHED Then
		bIsPressed = TRUE
	EndIf
	If IsWindowEnabled(m_hCtl) = FALSE Then
		bIsDisabled = TRUE
	EndIf
	
	' // Sets the button state for the theme
	iStateId = PBS_NORMAL
	If bIsPressed Then
		iStateId = PBS_PRESSED
	EndIf
	If bIsDisabled Then
		iStateId = PBS_DISABLED
	EndIf
	If iStateId = PBS_NORMAL Then
		If bIsFocused Then
			iStateId = PBS_DEFAULTED
		EndIf
		If GetPropW(m_hCtl, _T("HOT")) Then
			iStateId = PBS_HOT
		EndIf
	End if
	
	' // Toggled button
	If (m_bIsToggle BAND m_bToggled) Then
		iStateId = PBS_PRESSED
	EndIf

	' // Gets the coordinates of the button's client area
	GetClientRect(m_hCtl, &rcClient)
	rc = rcClient

	' // Draws the button
	IF m_bIsThemed THEN
		' // Increase 1 pixel to include the edge
		InflateRect(&rc, 1, 1)
		' // Draws the theme-specified border and fills for the "iPartId" and "iStateId".
		DrawThemeBackground(hTheme, hDc, BP_PUSHBUTTON, iStateId, &rc, NULL)
		' // Gets the size of the content for the theme-defined background
		GetThemeBackgroundContentRect(hTheme, hDc, BP_PUSHBUTTON, iStateId, &rc, &rcContent)
	Else
		' // Uses GDI to draw the button
		rcContent = rc
		IF bIsFocused Then
			If m_bIsToggle = FALSE OR bIsPressed = FALSE Then
				FrameRect( hDc, &rcContent, GetSysColorBrush(COLOR_WINDOWTEXT))
			End If
			InflateRect(&rcContent, -1, -1)
		End If
		If m_bIsToggle Then
			If iStateId = PBS_PRESSED Then
				DrawEdge(hDc, &rcContent, EDGE_SUNKEN, BF_RECT BOR BF_MIDDLE BOR BF_SOFT)
			Else
				If (lStyle BAND BS_FLAT) = BS_FLAT Then
					DrawEdge(hDc, &rcContent, EDGE_RAISED, BF_RECT BOR BF_MIDDLE BOR BF_SOFT BOR BF_FLAT)
				Else
					DrawEdge( hDc, &rcContent, EDGE_RAISED, BF_RECT BOR BF_MIDDLE BOR BF_SOFT)
				End If
			End If
		Else
			FillRect( hDc, &rcContent, GetSysColorBrush(COLOR_BTNFACE))
			If bIsPressed Then
				FrameRect(hDc, &rcContent, GetSysColorBrush(COLOR_BTNSHADOW))
			Else
				uState = DFCS_BUTTONPUSH
				If iStateId = PBS_HOT Then
					uState = uState BOR DFCS_HOT
				EndIf
				If (lStyle BAND BS_FLAT) = BS_FLAT Then
					uState = uState BOR DFCS_FLAT
				EndIf
				DrawFrameControl( hDc, &rcContent, DFC_BUTTON, uState)
			End If
		End If
	End If

	' // Calculates the y-coordinate of the image
	If (m_ImagePos BAND XPBI_VCENTER) = XPBI_VCENTER Then
		iy = ((rcContent.bottom - rcContent.top - m_ImageHeight) / 2) + rcContent.top
	ElseIf (m_ImagePos BAND XPBI_BOTTOM) = XPBI_BOTTOM Then
		iy = rcContent.bottom - m_ImageHeight
	Else
		iy = rcContent.top
	End If
	
	' // Calculates the x-coordinate of the image
	IF (m_ImagePos BAND XPBI_RIGHT) = XPBI_RIGHT Then
		ix = rcContent.right - m_ImageMargin - m_ImageWidth
	ElseIf (m_ImagePos BAND XPBI_LEFT) = XPBI_LEFT Then
		ix = rcContent.left + m_ImageMargin
	ElseIf (m_ImagePos BAND XPBI_CENTER) = XPBI_CENTER Then
		ix = ((rcClient.right - rcClient.left) / 2) - (m_ImageWidth / 2)
	Else
		ix = rcContent.left + m_ImageMargin
	End If

	' // If state = pressed, show the icon slightly displaced
	If iStateId = PBS_PRESSED Then
		ix = ix + 1
		iy = iy + 1
	End If
	
	' // Draws the image (don't draw it if the icon for normal state doesn't exist)
	If m_hImage <> NULL AND m_ImagePos <> XPBI_NONE Then
		If iStateId = PBS_HOT Then
			hImage = m_hHotImage
		EndIf
		If iStateId = PBS_DISABLED Then
			hImage = m_hDisabledImage
		EndIf
		If hImage = 0 Then
			hImage = m_hImage
		EndIf
		If m_ImageType = IMAGE_BITMAP Then
			DrawStateW(hDc, NULL, NULL, (LPARAM) hImage, 0, ix, iy, 0, 0, DST_BITMAP)
		Else
			DrawIconEx( hDc, ix, iy, (HICON)hImage, m_ImageWidth, m_ImageHeight, 0, NULL, DI_NORMAL)
		End If
		fImageDrawn  = TRUE
	End if

	' // Calculates the coordinates of the text
	If fImageDrawn Then
		If (m_ImagePos BAND XPBI_LEFT) = XPBI_LEFT Then
			If ((m_dwTextFlags BAND DT_CENTER) <> DT_CENTER) AND ((m_dwTextFlags BAND DT_RIGHT) <> DT_RIGHT) Then
				If (m_ImagePos BAND XPBI_ABOVE) = (XPBI_ABOVE) Then
					rcContent.left = rcContent.left + m_TextMargin
				Else
					rcContent.left = rcContent.left + m_TextMargin + m_ImageWidth + m_ImageMargin
				End If
			EndIf
		ElseIf (m_ImagePos BAND XPBI_RIGHT) = XPBI_RIGHT Then
			If (m_dwTextFlags BAND DT_RIGHT) = DT_RIGHT Then
				If (m_ImagePos BAND XPBI_ABOVE) = (XPBI_ABOVE) Then
					rcContent.right = rcContent.right - m_TextMargin
				Else
					rcContent.right = rcContent.right - m_TextMargin - m_ImageWidth - m_ImageMargin
				End If
			End If
		End If
	End If
	
	IF (m_ImagePos BAND XPBI_CENTERCENTER) <> (XPBI_CENTERCENTER) Then
		If (m_dwTextFlags BAND DT_CENTER) = (DT_CENTER) Then
			' Do nothing
		ElseIf (m_dwTextFlags BAND DT_RIGHT) = (DT_RIGHT) Then
			rcContent.right = rcContent.right - m_TextMargin
		ElseIf (m_dwTextFlags BAND DT_LEFT) = (DT_LEFT) Then
			rcContent.left = rcContent.left + m_TextMargin
		End If
		If m_bIsThemed Then
			' // Draws the text using the theme-specified color and font for the "iPartId" and "iStateId"
			DrawThemeText(hTheme, hDc, BP_PUSHBUTTON, iStateId, @wszCaption, -1, m_dwTextFlags, 0, &rcContent)
		Else
			' // Uses GDI to draw the text
			SetBkMode(hDc, TRANSPARENT)
			If bIsDisabled Then
				OffsetRect(&rcContent, 1, 1)
				SetTextColor(hDc, GetSysColor(COLOR_3DHILIGHT))
				DrawTextW( hDc, wszCaption, -1, &rcContent, m_dwTextFlags)
				OffsetRect(&rcContent, -1, -1)
				SetTextColor( hDc, GetSysColor(COLOR_3DSHADOW))
				DrawTextW( hDc, wszCaption, -1, &rcContent, m_dwTextFlags)
			Else
				SetTextColor(hDc, GetSysColor(COLOR_BTNTEXT))
				SetBkColor(hDc, GetSysColor(COLOR_BTNFACE))
				DrawTextW(hDc, wszCaption, -1, &rcContent, m_dwTextFlags)
			End If
		End If
	End If

	If m_bIsThemed Then
		' // Draws the focus rect
		If bIsFocused Then
			InflateRect(&rc, -3, -3)
			DrawFocusRect( hDc, &rc)
		End If
		' // Closes the theme data handle
		CloseThemeData(hTheme)
	Else
		' // Draws the focus rect
		If bIsFocused Then
			InflateRect(&rc, -4, -4)
			DrawFocusRect( hDc, &rc)
		End If
	End If
	
End Sub
'==============================================================================
Sub SetImage ( hImage AS HANDLE,  ImageType AS LONG,  ImageState AS LONG,  fRedraw AS LONG = FALSE)
	If hImage = NULL Then
		EXIT SUB
	EndIf
	If (ImageType <> IMAGE_ICON) AND (ImageType <> IMAGE_BITMAP) Then
		EXIT SUB
	EndIf
	If ImageState <> XPBI_NORMAL AND ImageState <> XPBI_HOT AND ImageState <> XPBI_DISABLED Then
		EXIT SUB
	EndIf
	
	' // Destroys the old image
	Select Case ImageState
		Case XPBI_NORMAL
			If m_hImage Then
				DeleteObject( m_hImage)
			EndIf
			m_hImage = hImage
		Case XPBI_HOT
			If m_hHotImage Then
				DeleteObject(m_hHotImage)
			EndIf
			m_hHotImage = hImage
		Case XPBI_DISABLED
			If m_hDisabledImage Then
				DeleteObject(m_hDisabledImage)
			EndIf
			m_hDisabledImage = hImage
	End Select
	
	' // Image type
	m_ImageType = ImageType
	' // Default width and height of the image
	m_ImageWidth = 16
	m_ImageHeight = 16
	Dim AS ICONINFO tii 
	Dim AS BITMAP tbm 
	Select Case ImageType
		Case IMAGE_ICON
			If GetIconInfo((HICON)hImage, &tii) Then
				If GetObject(tii.hbmMask, SIZEOF(tbm), &tbm) Then
					m_ImageWidth = tbm.bmWidth
					m_ImageHeight = tbm.bmHeight
				End If
			End If
		Case IMAGE_BITMAP
			If GetObject(hImage, SIZEOF(tbm), &tbm) Then
				m_ImageWidth = tbm.bmWidth
				m_ImageHeight = tbm.bmHeight
			End if
	End Select
	' // Redraws the button
	If fRedraw Then
		Redraw()
	EndIf
	
End Sub
'==============================================================================
' Sets the icon for the button
' Parameters:
' - hIcon = Icon's handle
' - ImageState =
'      XPBI_NORMAL = 1
'      XPBI_HOT = 2
'      XPBI_DISABLED = 3
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetIcon ( hIcon AS HICON,  ImageState AS LONG,  fRedraw AS LONG = FALSE)
   this->SetImage(hIcon, IMAGE_ICON, ImageState, fRedraw)
End Sub
'==============================================================================
' Sets the bitmap for the button
' Parameters:
' - hBitmap = Bitmap's handle
' - ImageState =
'      XPBI_NORMAL = 1
'      XPBI_HOT = 2
'      XPBI_DISABLED = 3
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetBitmap ( hBitmap AS HBITMAP,  ImageState AS LONG,  fRedraw AS LONG = FALSE)
   this->SetImage(hBitmap, IMAGE_BITMAP, ImageState, fRedraw)
End Sub
' ========================================================================================
' Loads an icon from file and sets it as the image of the button.
' Parameters:
' - wszPath = Full path of the icon's file.
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetIconFromFile ( pwszPath AS char ptr,  ImageState AS LONG,  fRedraw AS LONG = FALSE)
   Dim AS HICON hIcon  = (HICON)LoadImageW(NULL, pwszPath, (UINT)IMAGE_ICON, 0, 0, LR_LOADFROMFILE)
   this->SetImage(hIcon, IMAGE_ICON, ImageState, fRedraw)
End Sub
' ========================================================================================
' Loads a bitmap from file and sets it as the image of the button.
' Parameters:
' - wszPath = Full path of the bitmap file.
' - fRedraw = TRUE or FALSE (redraws the button to reflect the changes)
' ========================================================================================
Sub SetBitmapFromFile ( pwszPath AS char ptr,  ImageState AS LONG,  fRedraw AS LONG = FALSE)
   Dim AS HBITMAP hBitmap  = (HBITMAP)LoadImageW(NULL, pwszPath, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE)
   this->SetImage(hBitmap, IMAGE_BITMAP, ImageState, fRedraw)
End Sub
' ========================================================================================


CLASS_METHODS_END
