/******************************************************************************
Module name: DlgTmplt.c
Written by: Jeffrey Richter
Notices: Copyright (c) 1995 Jeffrey Richter
Purpose: Dynamic Dialog Box template toolkit functions.
******************************************************************************/

///////////////////////////////////////////////////////////////////////////////


#define NEXTDWORDBOUNDARY(p) ((PBYTE) ((DWORD) ((3 + (PBYTE) (p))) & ~3))
#define LENINWORDS(ByteLen)  ((ByteLen + sizeof(WORD)  - 1) & ~1)
#define LENINDWORDS(ByteLen) ((ByteLen + sizeof(DWORD) - 1) & ~3)


///////////////////////////////////////////////////////////////////////////////


// Macro to calculate number of bytes needed for a (unicode) string, including
// the zero terminating character.
#define BYTESFORSTRING(sz) (sizeof(WCHAR) * (1 + wcslen(sz)))


///////////////////////////////////////////////////////////////////////////////


BOOL ANSIToUnicode (PCSTR szStrA, PWSTR *pszStrW) {

   int nLenOfWideCharStr;
   BOOL fOK = FALSE;              // Assume failure

   *pszStrW = NULL;               // Assume failure
   if (szStrA == NULL) {
      
      // It is OK to have a NULL string.
      return(TRUE);
   }

   // Get the number of bytes needed for the wide version of the string.
   nLenOfWideCharStr = MultiByteToWideChar(CP_ACP, 0, szStrA, -1, NULL, 0);

   // Allocate memory to accomodate the size of the wide character string.
//   *pszStrW = malloc(nLenOfWideCharStr * sizeof(WCHAR));
   *pszStrW = (PWSTR)malloc(nLenOfWideCharStr * sizeof(WCHAR));

   if (*pszStrW != NULL) {
      
      // Convert the multi-byte string to a wide-character string.
      MultiByteToWideChar(CP_ACP, 0, szStrA, -1, *pszStrW, nLenOfWideCharStr);
      fOK = TRUE;
   }
   return(fOK);
}


///////////////////////////////////////////////////////////////////////////////


PDLGTEMPLATEEX DlgTemplate_CreateW (
   DWORD  dwStyle,
   DWORD  dwExStyle,
   DWORD  dwHelpId,
   short  x,  short y,            // In dialog-box units
   short  cx, short cy,           // In dialog-box units
   PCWSTR szMenuName,             // "" if no menu
   PCWSTR szClassName,            // "" if standard dialog box class
   PCWSTR szCaptionText, 
   short  nPointSize,             // Only used if DS_SETFONT specified
   short  nWeight,                // Only used if DS_SETFONT specified
   short  fItalic,                // Only used if DS_SETFONT specified
   PCWSTR szTypeFace) {           // Only used if DS_SETFONT specified

   int            nBytesForNewTemplate;
   int            nMenuNameLen, nClassNameLen, nCaptionTextLen, nTypeFaceLen;
   PBYTE          pbDlgTemplate, pbDlgTypeFace;
   PDLGTEMPLATEEX pDlgTemplate;
   PFONTINFOEX    pFontInfo;

   // Calculate number of bytes required by following fields:
   nMenuNameLen    = BYTESFORSTRING(szMenuName);
   nClassNameLen   = BYTESFORSTRING(szClassName);
   nCaptionTextLen = BYTESFORSTRING(szCaptionText);

   // Block must be large enough to contain the following:
   nBytesForNewTemplate =
      adgMEMBEROFFSET(DLGTEMPLATEEX, bStartOfStrings) +
      nMenuNameLen +          // # bytes: menu name.
      nClassNameLen +         // # bytes: dialog class name.
      nCaptionTextLen;        // # bytes: dialog box caption.

   if (dwStyle & DS_SETFONT) {

      // Dialog box uses font other than System font.

      // Calculate # of bytes required for typeface name.
      nTypeFaceLen = BYTESFORSTRING(szTypeFace);

      // Block must be large enough to include font information.
      nBytesForNewTemplate +=
         adgMEMBEROFFSET(FONTINFOEX, bStartOfStrings) +
         nTypeFaceLen;        // # bytes for font typeface name.

   } else {

      // Dialog box uses the System font.
      nTypeFaceLen = 0;

      // Block length does not change.
   }

   // Allocate an even number of DWORDs for alignment.
   nBytesForNewTemplate = LENINDWORDS(nBytesForNewTemplate);

   // Allocate block of memory for dialog template.
   pDlgTemplate = (PDLGTEMPLATEEX) malloc(nBytesForNewTemplate);
   if (pDlgTemplate == NULL)
      return(NULL);

   // Set the members of the DLGTEMPLATE structure.
   pDlgTemplate->wDlgVer     = 1;         // Always 1
   pDlgTemplate->wSignature  = 0xFFFF;    // Always 0xFFFF
   pDlgTemplate->dwHelpID    = dwHelpId;
   pDlgTemplate->dwStyle     = dwStyle;
   pDlgTemplate->dwExStyle   = dwExStyle;
   pDlgTemplate->cDlgItems   = 0;         // AddControl increments this
   pDlgTemplate->x           = x;
   pDlgTemplate->y           = y;
   pDlgTemplate->cx          = cx;
   pDlgTemplate->cy          = cy;


   // pbDlgTemplate points to start of variable part of DLGTEMPLATEEX.
   pbDlgTemplate = (PBYTE) pDlgTemplate + 
      adgMEMBEROFFSET(DLGTEMPLATEEX, bStartOfStrings);

   // Append the menu name, class name, and caption text to block.
   CopyMemory(pbDlgTemplate, szMenuName, nMenuNameLen);
   pbDlgTemplate += nMenuNameLen;
   CopyMemory(pbDlgTemplate, szClassName, nClassNameLen);
   pbDlgTemplate += nClassNameLen;
   CopyMemory(pbDlgTemplate, szCaptionText, nCaptionTextLen);
   pbDlgTemplate += nCaptionTextLen;

   if (dwStyle & DS_SETFONT) {

      // Dialog box uses font other than system font.

      // pFontInfo points to start of FONTINFOEX structure.
      pFontInfo = (PFONTINFOEX) pbDlgTemplate;

      // Set the members of the FONTINFO structure.
      pFontInfo->nPointSize = nPointSize;
      pFontInfo->nWeight    = nWeight;
      pFontInfo->fItalic    = fItalic;

      // szTypeFace points to start of the variable part of FONTINFOEX.
      pbDlgTypeFace = (PBYTE) pFontInfo + 
         adgMEMBEROFFSET(FONTINFOEX, bStartOfStrings);

      // Append the typeface name to the block.
      CopyMemory(pbDlgTypeFace, szTypeFace, nTypeFaceLen);
   }

   return(pDlgTemplate);
}


///////////////////////////////////////////////////////////////////////////////


PDLGTEMPLATEEX DlgTemplate_CreateA (
   DWORD  dwStyle,
   DWORD  dwExStyle,
   DWORD  dwHelpId,
   short   x,  short y,           // In dialog-box units
   short   cx, short cy,          // In dialog-box units
   PCSTR  szMenuName,             // "" if no menu
   PCSTR  szClassName,            // "" if standard dialog box class
   PCSTR  szCaptionText, 
   short  nPointSize,             // Only used if DS_SETFONT specified
   short  nWeight,                // Only used if DS_SETFONT specified
   short  fItalic,                // Only used if DS_SETFONT specified
   PCSTR  szTypeFace) {           // Only used if DS_SETFONT specified

   PDLGTEMPLATEEX pDlgTemplate = NULL;
   PWSTR szMenuNameW = NULL, szClassNameW = NULL;
   PWSTR szCaptionTextW = NULL, szTypeFaceW = NULL;

   // Allocate temporary Unicode buffers and convert ANSI strings to Unicode.
   if (ANSIToUnicode(szMenuName,    &szMenuNameW)    &&
       ANSIToUnicode(szClassName,   &szClassNameW)   &&
       ANSIToUnicode(szCaptionText, &szCaptionTextW) &&
       ANSIToUnicode(szTypeFace,    &szTypeFaceW)) {

      pDlgTemplate = DlgTemplate_CreateW(dwStyle, dwExStyle,
         dwHelpId, x, y, cx, cy, szMenuNameW, szClassNameW, 
         szCaptionTextW, nPointSize, nWeight, fItalic, szTypeFaceW);
   }

   // Destroy temporary Unicode buffers.
   // NOTE: ANSI C states that it is OK to pass NULL to free.
   free(szTypeFaceW);
   free(szCaptionTextW);
   free(szClassNameW);
   free(szMenuNameW);

   return(pDlgTemplate);
}


///////////////////////////////////////////////////////////////////////////////


BOOL DlgTemplate_AddControlW (
   PDLGTEMPLATEEX* ppDlgTemplate, // Returned from Create or AddControl
   short  x,  short y,            // In dialog-box units
   short  cx, short cy,           // In dialog-box units
   DWORD  id,       
   DWORD  dwHelpId,
   DWORD  dwStyle,                // WS_CHILD is automatically added
   DWORD  dwExStyle,
   PCWSTR szClass,
   PCWSTR szText,
   WORD   wExtraDataCount,
   PBYTE  pbExtraData) {          // Number of additional data bytes

   PDLGTEMPLATEEX pDlgTemplateNew; // If the memory block expands.
   int nBytesForNewControl;
   int nBytesBeforeAddingControl;
   PDLGITEMTEMPLATEEX pDlgItemTemplate;
   PBYTE pbDlgItemTemplate;

   // Calculate number of bytes required by following fields:
   int nClassLen = BYTESFORSTRING(szClass);
   int nTextLen  = BYTESFORSTRING(szText);

   // Block must be increased by to contain the following:
   nBytesForNewControl =
      adgMEMBEROFFSET(DLGITEMTEMPLATEEX, bStartOfStrings) +
      nClassLen +                 // # bytes for control class
      nTextLen +                  // # bytes for control text
      sizeof(WORD);               // 1 byte  for size in bytes of CreateParams
                                  // data

   // Alignment padding to CreateParams data
   nBytesForNewControl = LENINDWORDS(nBytesForNewControl);
   nBytesForNewControl += wExtraDataCount; // # bytes for CreateParams data

   // Alignment padding to next DLGITEMTEMPLATEEX structure
   nBytesForNewControl = LENINDWORDS(nBytesForNewControl);

   // Increase the size of the memory block to include the new dialog item.
   nBytesBeforeAddingControl = _msize(*ppDlgTemplate);
   pDlgTemplateNew = (PDLGTEMPLATEEX)realloc(*ppDlgTemplate,
      nBytesBeforeAddingControl + nBytesForNewControl);
   if (pDlgTemplateNew == NULL)
      return(FALSE);  

   // Tell the caller the new address of the dialog template.
   *ppDlgTemplate = pDlgTemplateNew;

   // Increment the number of controls in the template.
   pDlgTemplateNew->cDlgItems++;

   // pDlgItemTemplate points to the start of the DLGITEMTEMPLATE being added.
   pDlgItemTemplate = (PDLGITEMTEMPLATEEX)
      (((PBYTE) pDlgTemplateNew) + nBytesBeforeAddingControl);

   // Set the members of the newly added DLGITEMTEMPLATE structure.
   pDlgItemTemplate->x         = x;
   pDlgItemTemplate->y         = y;
   pDlgItemTemplate->cx        = cx;
   pDlgItemTemplate->cy        = cy;
   pDlgItemTemplate->id        = id;
   pDlgItemTemplate->dwStyle   = dwStyle | WS_CHILD;
   pDlgItemTemplate->dwExStyle = dwExStyle;
   pDlgItemTemplate->dwHelpID  = dwHelpId;

   // pbDlgTemplate points to the start of the variable part of the
   // DLGITEMTEMPLATEEX.
   pbDlgItemTemplate = (PBYTE) pDlgItemTemplate + 
      adgMEMBEROFFSET(DLGITEMTEMPLATEEX, bStartOfStrings);

   // Append the strings for the control's class name and caption to the block.
   CopyMemory(pbDlgItemTemplate, szClass, nClassLen);
   pbDlgItemTemplate += nClassLen;
   CopyMemory(pbDlgItemTemplate, szText, nTextLen);
   pbDlgItemTemplate += nTextLen;

   // Append the control's extra data byte information.
   * (PWORD) pbDlgItemTemplate = wExtraDataCount;
   pbDlgItemTemplate += sizeof(WORD);

   // Append the control's CreateParams data bytes.
   CopyMemory(pbDlgItemTemplate, pbExtraData, wExtraDataCount);

   return(TRUE); 
}


///////////////////////////////////////////////////////////////////////////////


BOOL DlgTemplate_AddControlA (
   PDLGTEMPLATEEX* ppDlgTemplate, // Returned from Create or AddControl
   short  x,  short y,            // In dialog-box units
   short  cx, short cy,           // In dialog-box units
   DWORD  id,       
   DWORD  dwHelpId,
   DWORD  dwStyle,                // WS_CHILD is automatically added
   DWORD  dwExStyle,
   PCSTR  szClass,
   PCSTR  szText,
   WORD   wExtraDataCount,        // Number of additional data bytes
   PBYTE  pbExtraData) {          // Passed via CREATESTRUCT's pCreateParams

   BOOL fOK = FALSE;              // Assume failure
   PWSTR szClassW = NULL;
   PWSTR szTextW = NULL;

   // Allocate temporary Unicode buffers and convert ANSI strings to Unicode.
   if (ANSIToUnicode(szClass, &szClassW) &&
       ANSIToUnicode(szText, &szTextW)) {

      fOK = DlgTemplate_AddControlW(ppDlgTemplate, x, y, cx, cy, id, dwHelpId, 
         dwStyle, dwExStyle, szClassW, szTextW, wExtraDataCount, pbExtraData);
   }

   // Destroy temporary Unicode buffers.
   // NOTE: ANSI C states that it is OK to pass NULL to free.
   free(szTextW);
   free(szClassW);

   return(fOK);
}


///////////////////////////////////////////////////////////////////////////////


BOOL DlgTemplate_Free (PDLGTEMPLATEEX pDlgTemplate) {
   
   free((PVOID) pDlgTemplate);
   return(TRUE);
}
