/******************************************************************************
Module name: DlgTmplt.h
Written by: Jeffrey Richter
Notices: Copyright (c) 1995 Jeffrey Richter
Purpose: Dynamic Dialog Box template toolkit functions.
******************************************************************************/

typedef struct { // dlttex
   WORD    wDlgVer;        // Always 1
   WORD    wSignature;     // Always 0xFFFF
   DWORD   dwHelpID;
   DWORD   dwExStyle;      // Such as WS_EX_TOPMOST
   DWORD   dwStyle;        // Such as WS_CAPTION
   WORD    cDlgItems;
   short   x;           // In pixels
   short   y;           // In pixels
   short   cx;          // In dialog box units
   short   cy;          // In dialog box units
   BYTE    bStartOfStrings;
   // Zero-terminated Unicode string for the menu name;
   // Zero-terminated Unicode string for the class name;
   // Zero-terminated Unicode string for the window title;
} DLGTEMPLATEEX, *PDLGTEMPLATEEX;


typedef struct {
   short   nPointSize;
   short   nWeight;     // Such as FW_NORMAL
   short   fItalic;     // TRUE or FALSE
   BYTE    bStartOfStrings;
   // Zero-terminated Unicode string for the font name;
} FONTINFOEX, *PFONTINFOEX;


typedef struct {
   DWORD   dwHelpID;
   DWORD   dwExStyle;   // Such as WS_EX_CONTROLPARENT
   DWORD   dwStyle;     // Such as WS_TABSTOP
   short   x;        // In dialog box units
   short   y;        // In dialog box units
   short   cx;       // In dialog box units
   short   cy;       // In dialog box units
   DWORD   id;
   BYTE    bStartOfStrings;
   // Zero-terminated Unicode string for the class name;
   // Zero-terminated Unicode string for the control title;
   // WORD    wExtraCount; // Usually 0
   // wExtraCount bytes of raw data;
 } DLGITEMTEMPLATEEX, *PDLGITEMTEMPLATEEX;

PDLGTEMPLATEEX DlgTemplate_CreateW (
   DWORD  dwStyle,
   DWORD  dwExStyle,
   DWORD  dwHelpId,
   short  x,  short y,            // In dialog-box units
   short  cx, short cy,           // In dialog-box units
   PCWSTR szMenuName,             // "" if no menu
   PCWSTR szClassName,            // "" if standard dialog box class
   PCWSTR szCaptionText, 
   short  nPointSize,             // Only used if DS_SETFONT specified
   short  nWeight,                // Only used if DS_SETFONT specified
   short  fItalic,                // Only used if DS_SETFONT specified
   PCWSTR szTypeFace);            // Only used if DS_SETFONT specified


PDLGTEMPLATEEX DlgTemplate_CreateA (
   DWORD  dwStyle,
   DWORD  dwExStyle,
   DWORD  dwHelpId,
   short  x,  short y,            // In dialog-box units
   short  cx, short cy,           // In dialog-box units
   PCSTR  szMenuName,             // "" if no menu
   PCSTR  szClassName,            // "" if standard dialog box class
   PCSTR  szCaptionText, 
   short  nPointSize,             // Only used if DS_SETFONT specified
   short  nWeight,                // Only used if DS_SETFONT specified
   short  fItalic,                // Only used if DS_SETFONT specified
   PCSTR  szTypeFace);            // Only used if DS_SETFONT specified

#ifdef UNICODE
#define DlgTemplate_Create DlgTemplate_CreateW
#else
#define DlgTemplate_Create DlgTemplate_CreateA
#endif // !UNICODE


///////////////////////////////////////////////////////////////////////////////


BOOL DlgTemplate_AddControlW (
   PDLGTEMPLATEEX* ppDlgTemplate, // Returned from Create or AddControl
   short  x,  short y,            // In dialog-box units
   short  cx, short cy,           // In dialog-box units
   DWORD  id,       
   DWORD  dwHelpId,
   DWORD  dwStyle,                // WS_CHILD is automatically added
   DWORD  dwExStyle,
   PCWSTR szClass,
   PCWSTR szText,
   WORD   wExtraDataCount,        // Number of additional data bytes
   PBYTE  pbExtraData);           // Passed via CREATESTRUCT's pCreateParams

BOOL DlgTemplate_AddControlA (
   PDLGTEMPLATEEX* ppDlgTemplate, // Returned from Create or AddControl
   short  x,  short y,            // In dialog-box units
   short  cx, short cy,           // In dialog-box units
   DWORD  id,       
   DWORD  dwHelpId,
   DWORD  dwStyle,                // WS_CHILD is automatically added
   DWORD  dwExStyle,
   PCSTR  szClass,
   PCSTR  szText,
   WORD   wExtraDataCount,        // Number of additional data bytes
   PBYTE  pbExtraData);           // Passed via CREATESTRUCT's pCreateParams

#ifdef UNICODE
#define DlgTemplate_AddControl DlgTemplate_AddControlW
#else
#define DlgTemplate_AddControl DlgTemplate_AddControlA
#endif // !UNICODE


///////////////////////////////////////////////////////////////////////////////


//BOOL DlgTemplate_Free (PVOID pvTemplate);


//////////////////////////////// End of File //////////////////////////////////
