' ########################################################################################
' Microsoft Windows
' File: CW_GDI_DrawShadedTriangle.bas
' Contents: Drawing a Shaded Triangle
' To draw a shaded triangle, define a TRIVERTEX structure with three elements and a single
' GRADIENT_TRIANGLE structure. The following code example shows how to draw a shaded
' triangle using the GradientFill function with the GRADIENT_FILL_TRIANGLE mode defined.
' Compiler: Ported to bc9Basic jcfuller 09/01/2016 
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
$NOMAIN
$CPP
$ONEXIT "ULEX.EXE $FILE$.CPP TCHARXLATER_VC.TXT"
'$ONEXIT "VSCPP.BAT $FILE$ -m64 gui"		
$ONEXIT "TCLIB.BAT $FILE$"	
'==============================================================================
$HEADER
	#include <commctrl.h>					/*'For CWindow*/ 
	#pragma comment(lib,"comctl32.lib")		/*'For CWindow*/
	#pragma comment(lib,"gdi32.lib")		/*'For CWindow*/
	#pragma comment(lib,"advapi32.lib")		/*'For DPI aware check*/
	#pragma comment(lib,"Msimg32.lib")
$HEADER
'==============================================================================

$Include <Afx/CWindow.bi>
'==============================================================================
SUB DrawShadedTriangle (hdc AS HDC, rxRatio AS SINGLE, ryRatio AS SINGLE)
	Raw As TRIVERTEX vertex[3]
	vertex[0].x     = 250 * rxRatio
	vertex[0].y     = 50 * ryRatio
	vertex[0].Red   = 0xff00
	vertex[0].Green = 0x8000
	vertex[0].Blue  = 0x0000
	vertex[0].Alpha = 0x0000
	
	vertex[1].x     = 50 * rxRatio
	vertex[1].y     = 250 * ryRatio
	vertex[1].Red   = 0x9000
	vertex[1].Green = 0x0000
	vertex[1].Blue  = 0x9000
	vertex[1].Alpha = 0x0000
	
	vertex[2].x     = 450 * rxRatio
	vertex[2].y     = 250 * ryRatio
	vertex[2].Red   = 0x9000
	vertex[2].Green = 0x0000
	vertex[2].Blue  = 0x9000
	vertex[2].Alpha = 0x0000
	
	' // Create a GRADIENT_TRIANGLE structure that
	' // references the TRIVERTEX vertices.
	
	DIM AS GRADIENT_TRIANGLE gTriangle
	gTriangle.Vertex1 = 0
	gTriangle.Vertex2 = 1
	gTriangle.Vertex3 = 2
	
	' // Draw a shaded triangle.
	GradientFill( hdc, &vertex[0], 3, &gTriangle, 1, GRADIENT_FILL_TRIANGLE)
	
	
End Sub

Function WinMain()
	Dim As int iRetVal
	Dim As HWND hWin
	Raw As CWindow pWindow
	

	hWin = pWindow.Create(NULL, "Drawing a shaded triangle", &WndProc)
	pWindow.Brush((HBRUSH)GetStockObject(WHITE_BRUSH))
	pWindow.SetClientSize(500, 300)
	
	pWindow.Center()

	
	iRetVal = pWindow.Do_Events(CmdShow)
	Function = iRetVal
End Function	
'==============================================================================
'==============================================================================
CallBack Function WndProc()
	Static AS SINGLE rxRatio     ' // Horizontal scaling ratio
	Static AS SINGLE ryRatio     ' // Vertical scaling ratio

	Select Case CBMSG
		Case WM_CREATE
			' // Get a pointer to the CWindow class from the CREATESTRUCT structure
			DIM AS CWindow PTR pWindow = AfxCWindowPtr(lParam)
			' // Get the scaling ratios
			rxRatio = pWindow->rxRatio()
			ryRatio = pWindow->ryRatio()
			EXIT FUNCTION
		Case WM_COMMAND
			Select Case CBCTL
				Case IDCANCEL
					If CBCTLMSG = BN_CLICKED Then
						SendMessage(CBHNDL,WM_CLOSE,0,0)
						Exit Function
					EndIf
			End Select
		Case WM_PAINT
			DIM AS PAINTSTRUCT ps
			DIM AS HDC hdc = BeginPaint(CBHNDL, &ps)
			DrawShadedTriangle(hdc, rxRatio, ryRatio)
			EndPaint(CBHNDL, &ps)
			EXIT FUNCTION
			
		Case WM_DESTROY
			PostQuitMessage(0)
			Exit Function
	End Select
End Function
