' ########################################################################################
' Microsoft Windows
' File: CW_GDI_StretchBlt.bas
' Contents: StretchBlt demo
' This example captures an image of the entire screen, creates a compatible device context
' and a bitmap with the appropriate dimensions, selects the bitmap into the compatible DC,
' and then copies the image using the BitBlt function.
' Later, in the WM_PAINT and WM_ERASEBKGND messages, redisplays the image calling StretchBlt,
' specifying the compatible DC as the source DC and a window DC as the target DC.
' To get better image quality, the stretching mode is set to HALFTONE.
' Compiler: Ported to bc9Basic jcfuller 09/02/2016
' Copyright (c) 2016 Jos Roca. Freeware. Use at your own risk.
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER
' EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
' MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ########################################################################################
$NOMAIN
$CPP
$ONEXIT "ULEX.EXE $FILE$.CPP TCHARXLATER_VC.TXT"
'$ONEXIT "VSCPP.BAT $FILE$ -m64 gui"		
$ONEXIT "TCLIB.BAT $FILE$"	
'==============================================================================
$HEADER
	#include <commctrl.h>					/*'For CWindow*/ 
	#pragma comment(lib,"comctl32.lib")		/*'For CWindow*/
	#pragma comment(lib,"gdi32.lib")		/*'For CWindow*/
	#pragma comment(lib,"advapi32.lib")		/*'For DPI aware check*/
$HEADER
'==============================================================================
$Include <Afx/CWindow.bi>
'==============================================================================
Function WinMain()
	Dim As int iRetVal
	Dim As HWND hWin
	Raw As CWindow pWindow
	
	hWin = pWindow.Create(NULL, "StretchBlt demo", &WndProc)
	pWindow.SetClientSize(500, 320)
	pWindow.Center()

	iRetVal = pWindow.Do_Events(CmdShow)
	Function = iRetVal
End Function	
'==============================================================================
CallBack Function WndProc()
	STATIC hdcCompatible AS HDC           ' // Handle of the compatible device context
	STATIC cxBitmap      AS LONG          ' // Width of the bitmap
	STATIC cyBitmap      AS LONG          ' // Height of the bitmap
	STATIC cxClient      AS LONG          ' // Width of the client area
	STATIC cyClient      AS LONG          ' // Height of the client area
	DIM    hdcScreen     AS HDC           ' // Desktop device context
	DIM    hbmScreen     AS HBITMAP       ' // Screen bitmap handle
	DIM    hdc           AS HDC           ' // Window device context
	DIM    ps            AS PAINTSTRUCT   ' // PAINTSTRUCT structure

	Select Case CBMSG
		Case WM_CREATE
			' // Create a normal DC and a memory DC for the entire screen. The
			' // normal DC provides a "snapshot" of the screen contents. The
			' // memory DC keeps a copy of this "snapshot" in the associated
			' // bitmap.
			DIM AS HDC hdcScreen = CreateDC("DISPLAY", NULL, NULL, NULL)
			hdcCompatible = CreateCompatibleDC(hdcScreen)
			' // Create a compatible bitmap for hdcScreen.
			cxBitmap = GetDeviceCaps(hdcScreen, HORZRES)
			cyBitmap = GetDeviceCaps(hdcScreen, VERTRES)
			hbmScreen = CreateCompatibleBitmap(hdcScreen, cxBitmap, cyBitmap)
			' // Select the bitmaps into the compatible DC.
			SelectObject(hdcCompatible, hbmScreen)
			' // Copy color data for the entire display into the
			' // bitmap that is selected into a compatible DC.
			BitBlt(hdcCompatible, 0, 0, cxBitmap, cyBitmap, hdcScreen, 0, 0, SRCCOPY)
			' // Delete the screen DC and bitmap
			DeleteDC( hdcScreen)
			DeleteObject( hbmScreen)
			EXIT FUNCTION
			
		Case WM_COMMAND
			Select Case CBCTL
				Case IDCANCEL
					If CBCTLMSG = BN_CLICKED Then
						SendMessage(CBHNDL,WM_CLOSE,0,0)
						Exit Function
					EndIf
			End Select
		Case WM_SIZE
			cxClient = LOWORD(CBLPARAM)
			cyClient = HIWORD(CBLPARAM)
			EXIT FUNCTION
		Case WM_PAINT
			hdc = BeginPaint(CBHNDL, &ps)
			SetStretchBltMode( hdc, HALFTONE)
			SetBrushOrgEx( hdc, 0, 0, NULL)
			StretchBlt( hdc, 0, 0, cxClient, cyClient, hdcCompatible, 0, 0, cxBitmap, cyBitmap, SRCCOPY)
			EndPaint(CBHNDL, &ps)
			EXIT FUNCTION
		Case WM_ERASEBKGND
			hdc = (HDC)CBWPARAM
			SetStretchBltMode( hdc, HALFTONE)
			SetBrushOrgEx( hdc, 0, 0, NULL)
			StretchBlt( hdc, 0, 0, cxClient, cyClient, hdcCompatible, 0, 0, cxBitmap, cyBitmap, SRCCOPY)
			FUNCTION = TRUE
		Case WM_DESTROY
			If hdcCompatible Then
				DeleteDC(hdcCompatible)
			EndIf
			PostQuitMessage(0)
			Exit Function
	End Select
End Function
